Package: hockeystick
Title: Download and Visualize Essential Climate Change Data
Version: 0.8.7
Authors@R: 
    person(given = "Hernando",
           family = "Cortina",
           role = c("aut", "cre"),
           email = "hch@alum.mit.edu",
           comment = c(ORCID = "0000-0001-6790-4870"))
Description: Provides easy access to essential climate change datasets to non-climate experts. Users can download the latest raw data from authoritative sources and view it via pre-defined 'ggplot2' charts. Datasets include atmospheric CO2, methane, emissions, instrumental and proxy temperature records, sea levels, Arctic/Antarctic sea-ice, Hurricanes, and Paleoclimate data. Sources include: NOAA Mauna Loa Laboratory <https://gml.noaa.gov/ccgg/trends/data.html>, Global Carbon Project <https://www.globalcarbonproject.org/carbonbudget/>, NASA GISTEMP <https://data.giss.nasa.gov/gistemp/>, National Snow and Sea Ice Data Center <https://nsidc.org/home>, CSIRO <https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/>, NOAA Laboratory for Satellite Altimetry <https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/> and HURDAT Atlantic Hurricane Database <https://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html>, Vostok Paleo carbon dioxide and temperature data: <doi:10.3334/CDIAC/ATG.009>.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://cortinah.github.io/hockeystick/,
        https://github.com/cortinah/hockeystick
BugReports: https://github.com/cortinah/hockeystick/issues
Imports: ggplot2, lubridate, tools, readr, dplyr, tidyr, patchwork,
        scales, rvest, tibble, treemapify, RColorBrewer, jsonlite,
        readxl
Depends: R (>= 4.1.0)
Suggests: rmarkdown, knitr, spelling, viridisLite, quarto, svglite,
        testthat (>= 3.0.0)
Config/testthat/edition: 3
Language: en-US
VignetteBuilder: quarto
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-11 10:51:18 UTC; hernandocortina
Author: Hernando Cortina [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6790-4870>)
Maintainer: Hernando Cortina <hch@alum.mit.edu>
Repository: CRAN
Date/Publication: 2026-05-11 18:50:21 UTC
Built: R 4.6.0; ; 2026-05-11 19:39:12 UTC; unix

Package: hockeystick
Title: Download and Visualize Essential Global Heating Data
Version: 1.0.0
Authors@R: 
    person(given = "Hernando",
           family = "Cortina",
           role = c("aut", "cre"),
           email = "hch@alum.mit.edu",
           comment = c(ORCID = "0000-0001-6790-4870"))
Description: Provides easy access to essential climate change datasets to non-climate experts. Users can download the latest raw data from authoritative sources and view it via pre-defined 'ggplot2' charts. Datasets include atmospheric CO2, methane, emissions, instrumental and proxy temperature records, CMIP6 projections, sea levels, Arctic/Antarctic sea-ice, Hurricanes, Wildfires, and Paleoclimate data. Sources include: NOAA Mauna Loa Laboratory <https://gml.noaa.gov/ccgg/trends/data.html>, Global Carbon Project <https://www.globalcarbonproject.org/carbonbudget/>, NASA GISTEMP <https://data.giss.nasa.gov/gistemp/>, National Snow and Sea Ice Data Center <https://nsidc.org/home>, CSIRO <https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/>, NOAA Laboratory for Satellite Altimetry <https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/> and HURDAT Atlantic Hurricane Database <https://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html>, Vostok Paleo carbon dioxide and temperature data: <doi:10.3334/CDIAC/ATG.009>.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://cortinah.github.io/hockeystick/,
        https://github.com/cortinah/hockeystick
BugReports: https://github.com/cortinah/hockeystick/issues
Imports: ggplot2, lubridate, tools, readr, dplyr, tidyr, patchwork,
        scales, rvest, tibble, treemapify, RColorBrewer, jsonlite,
        readxl
Depends: R (>= 4.1.0)
Suggests: rmarkdown, knitr, spelling, viridisLite, quarto, terra,
        geodata, svglite, testthat (>= 3.0.0)
Config/testthat/edition: 3
Language: en-US
VignetteBuilder: quarto
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-08-26 23:56:14 UTC; hernandocortina
Author: Hernando Cortina [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6790-4870>)
Maintainer: Hernando Cortina <hch@alum.mit.edu>
Repository: CRAN
Date/Publication: 2026-08-28 09:50:16 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,hockeystick_cache_info)
export(climate_grid)
export(emissions_map)
export(get_carbon)
export(get_dailytemp)
export(get_dailytempcop)
export(get_emissions)
export(get_hurricanes)
export(get_icecurves)
export(get_methane)
export(get_paleo)
export(get_seaice)
export(get_sealevel)
export(get_temp)
export(get_temp2k)
export(hockeystick_cache_delete)
export(hockeystick_cache_delete_all)
export(hockeystick_cache_details)
export(hockeystick_cache_list)
export(hockeystick_update_all)
export(merge_carbontemp)
export(plot_carbon)
export(plot_carbontemp)
export(plot_dailytemp)
export(plot_emissions)
export(plot_emissions_with_land)
export(plot_hurricane_nrg)
export(plot_hurricanes)
export(plot_icecurves)
export(plot_methane)
export(plot_paleo)
export(plot_seaice)
export(plot_sealevel)
export(plot_temp)
export(plot_temp2k)
export(plot_temp_monthly)
export(plot_temp_scatter)
export(warming_stripes)
import(dplyr)
import(ggplot2)
import(lubridate)
import(patchwork)
import(rvest)
import(tidyr)
import(treemapify)
importFrom(RColorBrewer,brewer.pal)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarize_all)
importFrom(ggplot2,ggplot)
importFrom(jsonlite,fromJSON)
importFrom(lubridate,ceiling_date)
importFrom(lubridate,date_decimal)
importFrom(lubridate,days)
importFrom(lubridate,month)
importFrom(lubridate,round_date)
importFrom(lubridate,year)
importFrom(lubridate,ym)
importFrom(lubridate,ymd)
importFrom(patchwork,wrap_plots)
importFrom(readr,col_skip)
importFrom(readr,cols)
importFrom(readr,parse_number)
importFrom(readr,read_csv)
importFrom(readr,read_table)
importFrom(readxl,read_xlsx)
importFrom(scales,unit_format)
importFrom(stats,na.omit)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_longer)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,unzip)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# hockeystick <img src="man/figures/logo.png" align="right" height="139"/>

<!-- badges: start -->

[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R-CMD-check](https://github.com/cortinah/hockeystick/actions/workflows/check-standard.yaml/badge.svg)](https://github.com/cortinah/hockeystick/actions/workflows/check-standard.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/hockeystick)](https://CRAN.R-project.org/package=hockeystick)
[![](https://cranlogs.r-pkg.org/badges/hockeystick)](https://cran.r-project.org/package=hockeystick)

<!-- badges: end -->

The goal of `hockeystick` is to make essential Global Heating datasets
easily available in R to non-climate experts. `hockeystick` users can
download the latest raw data from authoritative sources as well as view
it via pre-defined ggplot2 charts. Datasets include climate projects
through 2100, atmospheric CO<sub>2</sub> and CH<sub>4</sub>, carbon
emissions, instrumental, reconstructed, and paleo ice-core temperature
records, sea levels, hurricanes, and Arctic/Antarctic sea-ice.

The choice of data was originally based on [Professor Stefan
Rahmstorf’s](http://www.pik-potsdam.de/~stefan/) presentation on [The 5
Most Important Data Sets of Climate
Science](http://www.pik-potsdam.de/~stefan/5datasets_rahmstorf.pdf). I
came across this on a post on the [Open Mind
blog](https://tamino.wordpress.com/2018/11/01/the-5-most-important-data-sets-of-climate-science/).
I wrote my own
[post](https://rethinking.rbind.io/2018/11/16/the-top-five-climate-charts-using-ggplot2/)
on obtaining and visualizing this data (now out of date), which is the
basis for this package. Additional datasets and visualizations have been
added over The name of the package stems from the well known
[hockeystick](https://en.wikipedia.org/wiki/Hockey_stick_graph)
temperature chart.

`hockeystick` was highlighted on the RStudio
[RViews](https://rviews.rstudio.com/2021/03/19/february-2021-top-40-new-cran-packages/)
blog by Joseph Rickert as one of the “Top 40” new packages on CRAN in
February 2021.

**New in version 1.0.0:** Retrieve and plot global maps of future cmip6
projections from WorldClim.

New in version 0.9.0: Global wildfire area burnt and emissions data from
GWIS.

## Installation

To install the latest `hockeystick` release from CRAN type:

``` r
install.packages("hockeystick")
```

You may alternatively install the development version from
<https://github.com/cortinah/hockeystick> with:

``` r
pak::pak("cortinah/hockeystick@testing")
```

## Downloading and viewing global heating data

Plot daily global temperatures since 1940 and current anomaly:

``` r
library(hockeystick)
globaldaily <- get_dailytempcop()
plot_dailytemp(globaldaily)
```

<img src="man/figures/README-dailytemp-1.png" alt="" width="60%" />

Retrieve NOAA/ESRL Mauna Loa CO<sub>2</sub> Observatory concentration
data and plot:

``` r
ml_co2 <- get_carbon()
plot_carbon(ml_co2)
```

<img src="man/figures/README-carbon-1.png" alt="" width="60%" />

Retrieve 2081-2100 temperature projections and map:

``` r
proj <- get_cmip6()
plot_cmip6(proj)
```

<img src="man/figures/README-cmip6-1.png" alt="" width="62%" />

Retrieve 2081-2100 precipation anomaly and map:

``` r
anom <- get_cmip6_anom(var = 'prec')
plot_cmip6_anom(anom)
```

<img src="man/figures/README-cmip6anom-1.png" alt="" width="62%" />

Retrieve GCP global CO<sub>2</sub> emissions and plot:

``` r
emissions <- get_emissions()
plot_emissions(emissions)
```

<img src="man/figures/README-emissions-1.png" alt="" width="60%" />

``` r
plot_emissions_with_land(emissions)
```

<img src="man/figures/README-emissions-2.png" alt="" width="60%" />

Visualize cumulative emissions by country:

``` r
emissions_map()
```

<img src="man/figures/README-emissionsmap-1.png" alt="" width="60%" />

Retrieve NASA/GISS global surface temperature anomaly data and plot:

``` r
anomaly <- get_temp()
plot_temp(anomaly)
```

<img src="man/figures/README-temp-1.png" alt="" width="60%" />

Plot relationship between temperature anomaly and carbon:

``` r
plot_carbontemp()
```

<img src="man/figures/README-tempcarbon-1.png" alt="" width="60%" />

Visualize warming using Ed Hawkins styled “warming stripes”:

``` r
warming_stripes()
```

<img src="man/figures/README-stripes-1.png" alt="" width="60%" />

``` r
warming_stripes(stripe_only = TRUE, col_strip = viridisLite::viridis(11))
```

<img src="man/figures/README-stripes2-1.png" alt="" width="60%" />

Retrieve tide gauge and satellite sea level data and plot:

``` r
gmsl <- get_sealevel()
plot_sealevel(gmsl)
```

<img src="man/figures/README-sl-1.png" alt="" width="60%" />

Retrieve July annual Arctic Sea Ice Index and plot:

``` r
seaice <- get_seaice()
plot_seaice(seaice)
```

<img src="man/figures/README-si-1.png" alt="" width="60%" />

`get_seaice()` arguments can be modified to download Antarctic sea ice,
and allow any month.

You can also visualize sea ice by month and year:

``` r
arcticice <- get_icecurves()
plot_icecurves(arcticice)
```

<img src="man/figures/README-icecurves-1.png" alt="" width="60%" />

Retrieve Common Era temperature reconstruction and plot it with
instrumental record:

``` r
anomaly2k <- get_temp2k()
plot_temp2k(anomaly2k)
```

<img src="man/figures/README-2ktemp-1.png" alt="" width="60%" />

Retrieve NOAA HURDAT2 hurricane data and plot:

``` r
hurricanes <- get_hurricanes()
plot_hurricanes(hurricanes)
```

<img src="man/figures/README-hurricanes-1.png" alt="" width="60%" />

``` r
plot_hurricane_nrg(hurricanes)
```

<img src="man/figures/README-hurricanes-2.png" alt="" width="60%" />

Retrieve GWIS wildfire data and plot:

``` r
usfires <- get_fires_area(place='USA', year = 2026)
plot_fires_area(usfires)
```

<img src="man/figures/README-fires-1.png" alt="" width="60%" />

``` r
plot_fires_area(usfires, style = 'weekly')
```

<img src="man/figures/README-fires-2.png" alt="" width="60%" />

Retrieve NOAA/ESRL CH<sub>4</sub> Globally averaged mean data and plot:

``` r
ch4 <- get_methane()
plot_methane(ch4)
```

<img src="man/figures/README-methane-1.png" alt="" width="60%" />

Retrieve Vostok paleo ice core data and plot:

``` r
vostok <- get_paleo()
plot_paleo(vostok)
```

<img src="man/figures/README-paleo-1.png" alt="" width="60%" />

### Managing the cache

By default, no climate data is cached, and all data is downloaded every
time any of the `get_` functions is called. To cache data for future
use, use the `write_cache = TRUE` option, available in all of the `get_`
functions. To download and cache all data use
`hockeystick_update_all()`. To view the files, date, and size of cached
data use `hockeystick_cache_details()`. To re-download data from the
source use the `use_cache = FALSE` argument in any of the `get_`
functions, for example:
`get_carbon(use_cache = FALSE, write_cache = TRUE)`. To delete all
cached data use `hockeystick_cache_delete_all()`.

Users may also cache data by default by adding
`options(hs_write_cache = TRUE)`to their script or `.Rprofile` file.

### All together now: climate data grid

``` r
climate_grid()
```

<img src="man/figures/README-grid-1.png" alt="" width="80%" />

## Acknowledgments

- Carbon Dioxide concentrations: Dr. Pieter Tans, NOAA/GML
  (<https://gml.noaa.gov/ccgg/trends/>) and Dr. Ralph Keeling, [Scripps
  Institution of Oceanography](https://scrippsco2.ucsd.edu/).
- Global temperature anomaly: *GISS Surface Temperature Analysis
  (GISTEMP), version 4.* GISTEMP Team, 2020: NASA Goddard Institute for
  Space Studies. <https://data.giss.nasa.gov/gistemp/>
- Warming Stripes design from Ed Hawkins’ Climate Lab.
  <https://www.climate-lab-book.ac.uk/2018/warming-stripes/>. In
  addition grateful to Dr. Dominic Royé for posting his approach to
  plotting them using ggplot2, which `warming_stripes()` is based on.
  <https://dominicroye.github.io/blog/how-to-create-warming-stripes-in-r/>
- Sea level data: [NOAA Laboratory for Satellite Altimetry
  (sat)](https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/) and
  [Commonwealth Scientific and Industrial Research Organisation (tide
  gauges)](https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/)
- Sea Ice Index: [National Snow & Ice Data
  Center](https://nsidc.org/home). Data Archive:
  <https://nsidc.org/data/explore-data>
- Vostok carbon dioxide and temperature data:
  <https://data.ess-dive.lbl.gov/datasets/doi:10.3334/CDIAC/ATG.009>
  <https://data.ess-dive.lbl.gov/datasets/doi:10.3334/CDIAC/ATG.009>
- Common Era reconstructed temperature data: [PAGES2k Consortium and
  NOAA](https://www.ncei.noaa.gov/access/paleo-search/study/26872)).
- Hurricanes: National Oceanic and Atmospheric Administration HURDAT
  Atlantic Hurricane Database [Re-analysis
  Project](https://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html),
  particularly Dr. Chris Landsea.
- Carbon Dioxide emissions: [Global Carbon
  Project](https://www.globalcarbonproject.org/carbonbudget/) and [Our
  World In Data](https://github.com/owid/co2-data)
- Methane: Lan, X., K.W. Thoning, and E.J. Dlugokencky, NOAA Global
  Monitoring Laboratory (<https://gml.noaa.gov/ccgg/trends_ch4/>).
- Wildfire data: [Global Wildfire Information System
  (GWIS)](https://gwis.jrc.ec.europa.eu/)
- WorldClim [CMIP6](https://wcrp-cmip.org/cmip-phases/cmip6/) datasets:
  [WorldClim](https://worldclim.org/data/cmip6/cmip6climate.html#).
- Thank you Dirk Eddelbuettel for providing the .isConnected function
  from his [tint](https://CRAN.R-project.org/package=tint) package to
  test for internet connectivity.
- Thank you to Chris Campbell and Steven Bernard of the FT Climate
  Graphics Explained newsletter for sharing their code for
  climatereanalyzer.org daily temperature chart (Temperature Records
  Broken…Again, 9 July 2023). This code is the foundation of the
  get_dailytemp() and plot_dailytemp() functions.
- Thank you Nan Xiao for [PNG compression
  approach](https://nanx.me/blog/post/rpkgs-pngquant-ragg/).

## Notes and resources

- All data is compiled to the best of my ability from reliable and
  peer-reviewed sources. Please open an issue if you are aware of
  enhanced or additional data that may be added to the package. Building
  `hockeystick` is driven by my interest in tracking climate data and
  making it easily available to the community.
- Here are some online resources I have found very helpful to learn more
  about climate science:
- MIT edX Global Warming Science.
  <https://www.edx.org/learn/global-warming/massachusetts-institute-of-technology-global-warming-science>
- SDG Academy: Climate Change: The Science and Global Impact.
  <https://sdgacademy.org/course/climate-change-the-science-and-global-impact/>

#' Download and plot essential climate data
#'
#' Retrieves the Common Era Global Surface Temperature Reconstructions.
#' Source is PAGES2k Consortium and NOAA National Centers for Environmental Information.
#' \url{https://www.ncei.noaa.gov/access/paleo-search/study/26872}
#'
#' @name get_temp2k
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with filtered and unfiltered temperature reconstructions and Cowtan & Way instrumental temperatures.
#'
#' `get_temp2k` invisibly returns a tibble with the PAGES2k Consortium temperature reconstruction (years 1-2000 CE) and instrumental record (years 1850-2017 CE).  Temperatures represent deviations from the 1961-1990 mean.
#'
#' The returned object includes annual average temperature anomalies as well as filtered anomalies using a 31-year Butterworth filter.
#' Reconstructions use seven different statistical methods that draw from a global collection of temperature-sensitive palaeoclimate records.
#'
#' Methodology described in PAGES2k (2019) \url{https://www.nature.com/articles/s41561-019-0400-0}
#'
#' @importFrom readr read_csv
#' @importFrom lubridate ymd
#' @importFrom utils download.file
#'
#' @examples
#' \donttest{
#' # Fetch temp anomaly from cache if available:
#' anomaly <- get_temp2k()
#' #
#' # Force cache refresh:
#' anomaly <- get_temp2k(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_temp2k(anomaly) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item PAGES2k Common Era Surface Temperature Reconstructions. \url{https://www.ncei.noaa.gov/access/paleo-search/study/26872}
#' \item PAGES 2k Consortium., Neukom, R., Barboza, L.A. et al. Consistent multidecadal variability in global temperature reconstructions and simulations over the Common Era. \emph{Nat. Geosci.} 12, 643–649 (2019).  \doi{10.1038/s41561-019-0400-0}
#'  }
#'
#' @export
get_temp2k <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

hs_path <- tools::R_user_dir("hockeystick","cache")

if (use_cache) {
  if (file.exists(file.path(hs_path,'temp2k.rds'))) return(invisible(readRDS((file.path(hs_path,'temp2k.rds')))))   }

file_url <- 'https://www.ncei.noaa.gov/pub/data/paleo/pages2k/neukom2019temp/recons/Full_ensemble_median_and_95pct_range.txt'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

dl <- tempfile()
status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

temp2k <- suppressMessages( read_table(dl, col_names = FALSE, skip = 5,
                                         col_types = 'innnnnnnn') )

colnames(temp2k) <- c('year', 'instrumental', 'ensemble_median', 'ensemble_2.5', 'ensemble_97.5', 'filtered_instrumental',
                       'filtered_ensemble_median', 'filtered_ensemble_2.5', 'filtered_ensemble_97.5')

temp2k_l <- pivot_longer(temp2k, -year, names_to='measure')

temp2k_l <- filter(temp2k_l, measure %in% c('instrumental', 'ensemble_median', 'filtered_instrumental', 'filtered_ensemble_median'))

dir.create(hs_path, showWarnings = FALSE, recursive = TRUE)
if (write_cache) saveRDS(temp2k_l, file.path(hs_path, 'temp2k.rds'))

invisible(temp2k_l)
}


#' Download and plot essential climate data
#'
#' Plots the Common Era 2000-year global temperature anomaly retrieved using `get_temp2k()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_temp2k
#' @param dataset Name of the tibble generated by `get_temp2k`
#' @param instrumental (boolean) Include the Cowtan & Way instrumental temperatures through 2017. Defaults to TRUE.
#' @param filtered (boolean) Use the filtered temperatures provided by PAGES2k Consortium. Temperatures filtered using a 31-year Butterworth filter. Defaults to TRUE.
#' @param print (boolean) Display temperature anomaly ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#'
#' @return Invisibly returns a ggplot2 object with temperature anomaly chart
#'
#' @details `plot_temp2k` invisibly returns a ggplot2 object with a pre-defined temperature anomaly chart using data from `get_temp2k`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#'
#' @examples
#' \donttest{
#' # Fetch temperature anomaly:
#' anomaly <- get_temp2k()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_temp2k(anomaly)
#'
#' # Or just call plot_temp2k(), which defaults to get_temp2k() dataset
#' plot_temp2k()
#'
#' p <- plot_temp2k(anomaly, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Temperature Increase in the Common Era') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_temp2k <- function(dataset = get_temp2k(), instrumental = TRUE,
                        filtered = TRUE, print = TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  if (filtered) dataset <- filter(dataset, substr(measure,1, 8) == "filtered") else
                dataset <- filter(dataset, substr(measure,1, 8) != "filtered")

  if (!instrumental) dataset <- filter(dataset, measure != "filtered_instrumental" & measure != "instrumental")

plot <- ggplot(dataset, aes(x=year, y=value, color=measure)) +geom_line(aes(color=measure), linewidth=1.05) + theme_bw(base_size=12) +  scale_y_continuous(n.breaks = 10, minor_breaks = NULL) +
  scale_color_manual(name=NULL, values=c('dodgerblue2','firebrick1')) +theme(legend.position = c(0.3, 0.85),legend.background=element_blank()) +
  labs(title='Global Common Era Temperature Reconstruction', subtitle='Global surface temperature relative to 1961-1990 mean', x='Year C.E.',
       y='Temperature Anomaly (C\U00B0)', caption='Source: PAGES2k Consortium, NOAA\nhttps://www.ncei.noaa.gov/access/paleo-search/study/26872')

if (print) suppressWarnings( print(plot) )
invisible(plot)
}

.onAttach <- function(libname, pkgname) {
  options(hs_write_cache=getOption("hs_write_cache", FALSE))

  packageStartupMessage(paste0("hockeystick ", utils::packageVersion("hockeystick"),": Use hockeystick_cache_details() to view cached climate data.\n", "hockeystick ", utils::packageVersion("hockeystick"), ": Use hockeystick_update_all() to update and cache all climate data."))
}

#' Download and plot essential climate data
#'
#' Retrieves the daily air or sea-surface temperature data since 1940 from ClimateReanalyzer.org
#' Source is University of Maine Climate Change Institute.
#' \url{https://climatereanalyzer.org/clim/t2_daily/}
#'
#' @name get_dailytemp
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#' @param region (string) Region selection, defaults to world air temperature. Options are: World Air "W", Northern Hemisphere Air "NW", Southern Hemisphere Air "SW", Tropics Air "TR", Arctic Air "AR", Antarctica Air "AN", World Sea Surface "WS", and North Atlantic Sea Surface "NS".
#' @param mean_start (numeric) Start year for historic mean, defaults to 1979.
#' @param mean_end (numeric) End year for historic mean, defaults to 2000.
#'
#' @return Invisibly returns a tibble with the daily 2-meter air or sea surface temperatures since 1940 as well as historic mean by day-of-year and current anomaly versus mean.
#'
#' `get_dailytemp` invisibly returns a tibble with the daily temperatures since 1940 as well as mean by day-of-year and anomaly. Default to world data, but region can be selected among six options.
#'
#' Region options include world air (default), Northern Hemisphere air, Southern Hemisphere air, Tropics air, Arctic air, Antarctic air, World sea surface and North Atlantic sea surface and is stored in attribute of output.
#' The historic daily mean-by-day period defaults to 1979-2000. This range can be optionally modified.
#'
#' Data are updated daily. For day-of-year mean removes observations from February 29 on leap years.
#'
#' @importFrom jsonlite fromJSON
#' @importFrom utils download.file head tail
#' @importFrom readr parse_number
#' @import tidyr
#' @import dplyr
#'
#' @examples
#' \donttest{
#' # Fetch temp anomaly from cache if available:
#' dailytemps <- get_dailytemp()
#' #
#' # Force cache refresh:
#' dailytemps <- get_dailytemp(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_dailytemp(dailytemps)
#'
#' # Change region to Arctic
#' arctictemp <- get_dailytemp(region='AR', use_cache=FALSE)}
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item ClimateReanalyzer.org: \url{https://climatereanalyzer.org/clim/t2_daily/}
#'
#' Notes: daily mean surface air temperature (2-meter height) estimates from the ECMWF Reanalysis version 5 (ERA5) for the period January 1940 to present. ERA5 is a state-of-the-art numerical climate/weather modeling framework that ingests surface, radiosonde, and satellite observations to estimate the state of the atmosphere through time.
#' ERA5 files have a horizontal grid resolution of 0.25° x 0.25° (about 31km x 31km at 45°N). Each daily temperature represents an average across all model gridcells within the defined latitude/longitude bounds for the selected domain. The means are area-weighted to account for the convergence of longitude lines at the poles
#'
#' }
#'
#' @export

get_dailytemp <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache"),
                          region = 'W',
                          mean_start = if(region %in% c('WS','NS', 'ws', 'ns')) 1982 else 1979,
                          mean_end = 2000) {

  hs_path <- tools::R_user_dir("hockeystick", "cache")
  region <- toupper(region)

  if (use_cache) {
    if (file.exists(file.path(hs_path, 'dailytemp.rds')))

    {cached_temp <- readRDS((file.path(hs_path, 'dailytemp.rds')))
     cached_region <- attr(cached_temp, "hs_daily_region")

      if (region==cached_region) return(invisible(cached_temp)) }
  }

  file_url <- switch(region,
                      W = 'https://climatereanalyzer.org/clim/t2_daily/json/era5_world_t2_day.json',       # World Air
                     NW = 'https://climatereanalyzer.org/clim/t2_daily/json/era5_nh_t2_day.json',          # Northern Hemi Air
                     SW = 'https://climatereanalyzer.org/clim/t2_daily/json/era5_sh_t2_day.json',          # Southern Hemi Air
                     AR = 'https://climatereanalyzer.org/clim/t2_daily/json/era5_arctic_t2_day.json',      # Arctic Air
                     AN = 'https://climatereanalyzer.org/clim/t2_daily/json/era5_antarctic_t2_day.json',   # #Antarctic Air
                     TR = 'https://climatereanalyzer.org/clim/t2_daily/json/era5_tropics_t2_day.json',     # Tropics Air
                     WS = 'https://climatereanalyzer.org/clim/sst_daily/json/oisst2.1_world2_sst_day.json',     # World Sea
                     NS = 'https://climatereanalyzer.org/clim/sst_daily/json/oisst2.1_natlan1_sst_day.json')    # North Atlantic Sea

                     connected <- .isConnected(file_url)
  if (!connected) {message("Retrieving remote data requires connectivity to source.\nAvailable regions are: W, NW, SW, AR, AN, TR, WS, NS."); return(invisible(NULL))}

  dl <- tempfile()
  status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
  if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

  temp_json <- tryCatch (jsonlite::fromJSON(dl),
                          error=function(error_msg) {
                            message('Invalid JSON file, please check temperature data url.')
                            return(NULL)
                            } )
  if (is.null(temp_json)) return(invisible(NULL))

leap_years <- seq.int(1940, 2032, 4)

if (region!='WS' && region!='NS') {

suppressWarnings(
daily_temperature <- as_tibble(temp_json) |> head(-3) |>
  separate_wider_delim('data', delim=', ', names_sep = '') |>
  mutate(data1 = parse_number(data1),
         data366 = parse_number(data366, na = 'NA)')) |>
  select(-data_source) |> mutate_all(as.numeric) |>
  pivot_longer(2:367, names_to = 'day_of_year') |>
  rename(year = name,
         temp = value) |>
  mutate(day_of_year = parse_number(day_of_year)) |>
  filter(!(day_of_year==60 & year %in% leap_years)) |>
  drop_na()
) }      else {

  suppressWarnings(
  daily_temperature <- as_tibble(temp_json) |> head(-3) |> tail(-1) |>
      separate_wider_delim('data', delim=',', names_sep = '') |>
    mutate(data1 = parse_number(data1),
             data366 = parse_number(data366, na = 'NA)')) |>
    mutate_all(as.numeric) |>
      pivot_longer(2:367, names_to = 'day_of_year') |>
      rename(year = name,
             temp = value) |>
      mutate(day_of_year = parse_number(day_of_year)) |>
      filter(!(day_of_year==60 & year %in% leap_years)) |>
      drop_na()
  ) }



daily_temperature <- daily_temperature |> group_by(year) |>
  mutate(day_of_year = row_number()) |> ungroup() |>
  group_by(day_of_year) |>
  mutate(mean_temp = mean(temp[year >= mean_start & year <= mean_end]),
         temp_anom = temp - mean_temp) |>
  ungroup()

dates <- data.frame(day_of_year=1:365,
                    dummy_date=seq(as.Date('1925-01-01'), as.Date('1925-12-31'), by = '1 days'))

daily_temperature <- left_join(daily_temperature, dates, by='day_of_year')

colnames(daily_temperature)[4] <- paste0(mean_start, '-', mean_end, ' mean')

daily_temperature <- mutate(daily_temperature, date = as.Date(paste0(year, '-', substr(dummy_date,6,7), '-', substr(dummy_date, 9, 10))))

daily_temperature <- daily_temperature |> relocate(date, .after=day_of_year)

if (region == 'WS' | region == 'NS') daily_temperature <- daily_temperature |> filter(year!=1981)

attr(daily_temperature, "hs_daily_region") <- region

if (write_cache) saveRDS(daily_temperature, file.path(hs_path, 'dailytemp.rds'))

invisible(daily_temperature)
}


#' Download and plot essential climate data
#'
#' Plots the daily temperatures since 1940 and current anomaly data retrieved using `get_dailytempcop()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_dailytemp
#' @param dataset Name of the tibble generated by `get_dailytempcop` or `get_dailytemp`
#' @param print (boolean) Display daily temperature ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param anomaly (boolean) Display current anomaly versus historic mean, defaults to FALSE.
#' @param maxtemp (boolean) Display current deviation versus historic max, defaults to FALSE.
#' @param current_year (numeric) Year to highlight in alternate color, defaults to current year.
#' @param title_lab (string) Title to override default chart title. Default title pulls region name from dataset attributes.
#' @param cop (boolean) Flag for chart caption, TRUE = Copernicus, FALSE =. ClimateReanalyzer.org
#'
#' @return Invisibly returns a ggplot2 object with daily temperature anomaly chart
#'
#' @details `plot_temp` invisibly returns a ggplot2 object with a pre-defined daily temperature anomaly chart using data from `get_dailytemp`.
#' By default the chart is also displayed. Plots one line per year, as well as mean and anomaly (which may be disabled). Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom RColorBrewer brewer.pal
#'
#' @examples
#' \donttest{
#' # Fetch temperature anomaly:
#' dailydata <- get_dailytempcop()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_dailytemp(dailydata)
#'
#' # Don't plot anomaly shading and highight specific year
#' plot_dailytemp(anomaly = FALSE, current_year = 2012)
#'
#' # Or just call plot_temp(), which defaults to get_dailytempcop() dataset
#' plot_dailytemp()
#'
#' p <- plot_dailytemp(dailydata, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Record Temperatures in 2023') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_dailytemp <- function(dataset = get_dailytempcop(), print = TRUE, anomaly = FALSE, maxtemp = FALSE,
                           current_year = as.numeric(substr(Sys.Date(), 1, 4)),
                           title_lab = 'Daily Average Air Temperature', cop=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  if (anomaly && maxtemp) stop("Only one of anomaly or maxtemp parameters can be TRUE")

latest <- pull(tail(dataset, 1)['date'])
meanperiod <- colnames(dataset)[5]
colnames(dataset)[5] <- 'mean_temp'

region <- attr(dataset, "hs_daily_region")

subtitle_lab <- '2-meter temperature since 1940 and mean'

if (region == 'WS' ||region == 'NS') subtitle_lab <- 'Sea surface temperature since 1982 and mean'

if (title_lab == "Daily Average Air Temperature") {

    if (region == 'WS' | region == 'NS') title_lab <- 'Daily Average Sea Surface Temperature'

    region <- switch(region,
                      W = 'World',
                     NW = 'Northern Hemisphere',
                     SW = 'Southern Hemisphere',
                     AR = 'Arctic',
                     AN = 'Antarctic',
                     TR = 'Tropics',
                     WS = 'World (60S-60N)',
                     NS = 'North Atlantic')

    title_lab <- paste(region, title_lab)}

# Color code notes: 'A': all years, 'M': mean, 'L': latest.

captionsource <- if (cop) paste0('Source: EU Copernicus Climate Service\ncds.climate.copernicus.eu as of ', latest)
                    else
                          paste0('Source: Climate Change Institute, University of Maine\nClimateReanalyzer.org as of ', latest)

plot <- ggplot(dataset) +
    geom_line(aes(x = dummy_date, y = temp, group = year, color = 'A'), alpha = 0.7) +
    geom_line(aes(dummy_date, mean_temp, color = 'M'), linetype = 'dashed', linewidth = 1.1) +
    scale_y_continuous(n.breaks = 9) +
    theme_bw(base_size = 12) +
    scale_x_date(name = 'Day of year', breaks = c(as.Date('1925-01-01'), as.Date('1925-04-01'),
                                                  as.Date('1925-07-01'), as.Date('1925-10-01'), as.Date('1925-12-31')),
                 date_labels = '%b%est', date_minor_breaks = '1 month') +
    labs(title = title_lab, subtitle = subtitle_lab,
         y = 'Temperature (C\U00B0)',
         caption = captionsource,
         color = NULL) +
    scale_color_manual(values = c('firebrick', 'black', 'grey'), labels = c(current_year, meanperiod, 'All years'), breaks = c('L', 'M', 'A')) +
    geom_line(data = filter(dataset, year == current_year),
              aes(dummy_date, temp, color = 'L'), linewidth = 1.3) +
    theme(legend.position = "top") + theme(legend.key.size = unit(0.5, 'cm'),
                                         legend.margin = margin(5, 0, 0, 0))

if (anomaly) {
  subtitle_lab <- '2-meter temperature since 1940, mean, and current anomaly'

  if (region == 'World (60S-60N)' | region == 'North Atlantic' | region == 'WS' | region == 'NS')
      subtitle_lab <- 'Sea surface temperature since 1982, mean, and current anomaly'

  plot <- plot + scale_fill_gradientn(name = 'Anomaly (C\U00B0)', colors = RColorBrewer::brewer.pal(9, 'YlOrRd'),
                                      labels = scales::label_number(accuracy = 0.1)) +
          geom_rect(data = filter(dataset, year == current_year),
          aes(xmin = dummy_date - 1,
              xmax = dummy_date,
              ymin = mean_temp,
              ymax = mean_temp + temp_anom,
              fill = temp_anom)) +
          geom_line(data = filter(dataset, year == current_year),
            aes(dummy_date, temp, color='L'), linewidth = 1.3) +
    geom_line(aes(dummy_date, mean_temp, color = 'M'), linetype = 'dashed', linewidth = 1.2) +
  labs(subtitle = subtitle_lab) }

if (maxtemp) {
  subtitle_lab <- '2-meter temperature since 1940, mean, and deviation from historic max'

  if (region == 'World (60S-60N)' | region == 'North Atlantic' | region == 'WS' | region == 'NS')
    subtitle_lab <- 'Sea surface temperature since 1982, mean, and current anomaly'

  max_temp <- dataset |> filter(year!=current_year) |> group_by(day_of_year) |> summarize(max_temp=max(temp))
  dataset <- left_join(dataset, max_temp, by = "day_of_year")

  plot <- plot +
    scale_fill_gradientn(name = 'Deviation (C\U00B0)',
                         colors = rev(RColorBrewer::brewer.pal(9, 'RdYlGn')), labels = scales::label_number(accuracy = 0.1)) +
    geom_rect(data = filter(dataset, year == current_year),
              aes(xmin = dummy_date - 1,
                  xmax = dummy_date,
                  ymin = max_temp,
                  ymax = temp,
                  fill = temp - max_temp)) +
    geom_line(data = filter(dataset, year == current_year),
              aes(dummy_date, temp, color='L'), linewidth = 1.3) +
    labs(subtitle = subtitle_lab) }

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}


#' Download and plot essential climate data
#'
#' Retrieves the daily air temperature since 1940 from the EU Copernicus Service
#' \url{https://cds.climate.copernicus.eu/#!/home}
#'
#' @name get_dailytempcop
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#' @param region (string) Region selection, defaults to world air temperature. Options are: World Air "W".
#'
#' @return Invisibly returns a tibble with the daily 2-meter air temperatures since 1940 as well as historic mean by day-of-year and current anomaly versus mean.
#'
#' `get_dailytempcop` invisibly returns a tibble with the daily temperatures since 1940 as well as mean by day-of-year and anomaly.
#'
#' Region options include world air (default).
#' The historic daily mean-by-day period defaults to 1991-2020.
#'
#' Data are updated daily.
#'
#' @importFrom utils download.file head tail
#' @importFrom readr parse_number
#' @import tidyr
#' @import dplyr
#'
#' @examples
#' \donttest{
#' # Fetch temp anomaly from cache if available:
#' dailytemps <- get_dailytempcop()
#' #
#' # Force cache refresh:
#' dailytemps <- get_dailytempcop(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_dailytemp(dailytemps) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item Copernicus: \url{https://cds.climate.copernicus.eu/#!/home}
#'
#' Notes: daily mean surface air temperature (2-meter height) estimates from the ECMWF Reanalysis version 5 (ERA5) for the period January 1940 to present. ERA5 is a state-of-the-art numerical climate/weather modeling framework that ingests surface, radiosonde, and satellite observations to estimate the state of the atmosphere through time.
#' ERA5 files have a horizontal grid resolution of 0.25° x 0.25° (about 31km x 31km at 45°N). Each daily temperature represents an average across all model gridcells within the defined latitude/longitude bounds for the selected domain. The means are area-weighted to account for the convergence of longitude lines at the poles
#'
#' }
#'
#' @export


get_dailytempcop <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache"), region = 'W') {

  hs_path <- tools::R_user_dir("hockeystick", "cache")

  if (use_cache) {
    if (file.exists(file.path(hs_path, 'dailytempcop.rds')))

    {cached_temp <- readRDS((file.path(hs_path, 'dailytempcop.rds')))
    cached_region <- attr(cached_temp, "hs_daily_region")

    if (region==cached_region) return(invisible(cached_temp)) }
  }

  file_url <- 'https://sites.ecmwf.int/data/climatepulse/data/series/era5_daily_series_2t_global.csv'

  connected <- .isConnected(file_url)
  if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

  dl <- tempfile()
  status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
  if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

  temp_csv <- read.csv(dl, skip = 18)

  colnames(temp_csv) <- c('date', 'temp', '1991-2020 mean', 'temp_anom', 'status' )
  temp_csv$date <- as.Date(temp_csv$date)
  temp_csv$year <- lubridate::year(temp_csv$date)
  temp_csv$dummy_date <- as.Date(paste("1925", lubridate::month(temp_csv$date), lubridate::day(temp_csv$date),sep = '-'))

  temp_csv <- temp_csv |> filter(!is.na(dummy_date))
  temp_csv <- temp_csv |> group_by(year) |> mutate(day_of_year = row_number()) |> ungroup()

  daily_temperature <- temp_csv |> select(year, day_of_year, date, temp, `1991-2020 mean`, temp_anom, dummy_date) |> as_tibble()

  attr(daily_temperature, "hs_daily_region") <- region

  if (write_cache) saveRDS(daily_temperature, file.path(hs_path, 'dailytempcop.rds'))

  invisible(daily_temperature) }

  #' Download and plot essential climate data
#'
#' Retrieves Arctic or Antarctic monthly Sea Ice Index time series (in million square km).
#' Source is the National Snow and Ice Data Center, defaults to Arctic (Northern Hemisphere) monthly sea ice extent since 1979.
#' \url{https://nsidc.org/data/explore-data} \cr \cr
#'
#' @name get_icecurves
#' @param pole 'N' for Arctic or 'S' for Antarctic
#' @param measure Must be 'extent' or 'area', defaults to 'extent'. Please see terminology link in references for details.
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data, or to change pole or month in cache.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with the series of monthly Sea Ice Index since 1979 (in million square km).
#'
#' `get_icecurves` invisibly returns a tibble with time series of monthly Sea Ice Index since 1979 (in million square km).
#'
#' User may select monthly Arctic or Antarctic sea ice extent or area (in millions of square kilometers).
#' Defaults to Arctic sea ice extent.  \url{https://nsidc.org/sea-ice-today/about-data#area_extent}
#'
#' @importFrom utils download.file read.csv
#' @importFrom tibble tibble
#'
#' @examples
#' \donttest{
#' # Fetch monthly sea ice history from cache if available:
#' icecurves <- get_icecurves()
#' #
#' # Force cache refresh:
#' icecurves <- get_icecurves(use_cache = FALSE)
#' # change region
#' south_icecurves <- get_icecurves(pole='S',  use_cache = FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_icecurves(icecurves) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item NSIDC Data Archive: \url{https://nsidc.org/data/explore-data}
#' \item All About Sea Ice: \url{https://nsidc.org/learn/parts-cryosphere/sea-ice}
#' \item Sea Ice terminology: \url{https://nsidc.org/learn}
#'  }
#'
#' @export

get_icecurves <- function(pole='N', measure='extent', use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

  if (pole != 'S' & pole != 'N') stop("pole must be 'N' or 'S'")
    if (measure != 'extent' & measure != 'area') stop("measure must be 'extent' or 'area'")

  hs_path <- tools::R_user_dir("hockeystick","cache")

  if (use_cache) {
    if (file.exists(file.path(hs_path, 'icecurves.rds')))

      {cached_ice <- readRDS((file.path(hs_path,'icecurves.rds')))
       cached_type <- attr(cached_ice, 'hs_ice_type')

      if (all(c(pole, measure) == cached_type)) return(invisible(cached_ice)) }
  }

  connected <- .isConnected('https://noaadata.apps.nsidc.org/NOAA/G02135/seaice_analysis/Sea_Ice_Index_Monthly_Data_with_Statistics_G02135_v4.0.xlsx')
  if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

  filename <- 'https://noaadata.apps.nsidc.org/NOAA/G02135/seaice_analysis/Sea_Ice_Index_Monthly_Data_with_Statistics_G02135_v4.0.xlsx'

  dl <- tempfile()
  status <- tryCatch(suppressWarnings(download.file(filename, dl, mode='wb')), error = function(e) 1L)
  if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

  curve <- function(month, pole) {
    sheet <- paste0(month.name[month], '-', pole, 'H')
    seaice <- suppressMessages( read_xlsx(dl, skip=9, sheet=sheet) )
    seaice$date <- lubridate::ceiling_date(lubridate::ymd(paste(seaice$year, seaice$month, '01', sep='-')), 'month')-1

    if (measure == 'extent') seaice <- seaice[,c('year', 'month', 'extent')] else seaice <- seaice[,c('year', 'month', 'area')]
    seaice <- tibble::tibble(seaice)

    return(seaice)
  }

  icecurves <- lapply(1:12, curve, pole=pole)
  icecurves <- do.call("rbind", icecurves)
  attr(icecurves, "hs_ice_type") <- c(pole, measure)

    if (write_cache) saveRDS(icecurves, file.path(hs_path, 'icecurves.rds'))

  invisible(icecurves) }



#' Download and plot essential climate data
#'
#' Plots the monthly Sea Ice Index data retrieved using `get_icecurves()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_icecurves
#' @param dataset Name of the tibble generated by `get_icecurves`, defaults to calling `get_icecurves`
#' @param print (boolean) Display sea ice ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @return Invisibly returns a ggplot2 object with Sea Ice Index chart
#'
#' @details `plot_icecurves` invisibly returns a ggplot2 object with a pre-defined Sea Ice Index chart using data from `get_icecurves`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#' Chart consists of one line per year showing monthly sea ice from January through December. Current year is highlighted.
#'
#' @import ggplot2
#'
#' @examples
#' \donttest{
#' # Fetch historic monthly sea ice data since 1979:
#' icecurves <- get_icecurves()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_icecurves(icecurves)
#'
#' # Or just call plot_icecurves(), which defaults to get_icecurves() dataset
#' plot_icecurves()
#'
#' p <- plot_icecurves(icecurves, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Shrinking Arctic Sea Ice') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_icecurves <- function(dataset = get_icecurves(), print = TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  measure_name <- colnames(dataset)[3]
  measure_name <- paste(toupper(substr(measure_name, 1, 1)), substr(measure_name, 2, nchar(measure_name)), sep="")

  ice_region <- if (attr(dataset, "hs_ice_type")[1] == 'S') 'Antarctic' else 'Arctic'

  title <- paste0(ice_region, ' Sea Ice ', measure_name, ' by Year and Month')

  colnames(dataset) <- c('year', 'month', 'measure')
  current_year <- max(dataset$year)
  dataset$linew <- ifelse(dataset$year == current_year, current_year, 'Previous')
  dataset <- na.omit(dataset)

  plot <- ggplot(dataset, aes(x=month, y=measure, alpha=as.factor(linew), linewidth=as.factor(linew), color=as.factor(year))) +
    geom_line() +scale_y_continuous(limits=c(0, 20)) +theme_bw() +
    scale_x_continuous(breaks = seq(1, 12, 1), minor_breaks = NULL) +
    scale_color_manual(guide = 'none', values = c(rep('darkgrey', length(unique(dataset$year))-1), 'dodgerblue')) +
    scale_linewidth_manual(labels = c(paste0('1979-', current_year-1), current_year), values = c(0.7, 2), breaks = c('Previous', as.character(current_year))) +
    labs(title = title, x = 'Month', y = expression("Million km"^2),
         caption = 'Source: National Snow & Ice Data Center\nhttps://nsidc.org/data/seaice_index',
         linewidth = NULL) +
    theme(legend.position = c(0.41, 0.91), legend.background = element_blank()) +
    scale_alpha_manual(guide = 'none', labels = c('Previous', as.character(current_year)), values = c(0.4, 1), breaks = c('Previous', '2023')) +
    guides(linewidth = guide_legend(override.aes=list(color = c('darkgrey', "dodgerblue"))))


  if (print) suppressMessages( print(plot) )
  invisible(plot)
}

# This file is part of the R package "hockeystick".
# It is made available under the terms of the GNU General Public
# License, version 2 (GPL-2), or (at your option) any later version.

library(testthat)
library(hockeystick)

test_check("hockeystick")

#' Download and plot essential climate data
#'
#' Retrieves global wildfire burnt area and fire count data from the
#' Global Wildfire Information System (GWIS), operated by the Copernicus
#' Emergency Management Service.
#' \url{https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend}
#'
#' @name get_fires_area
#' @param place (string) Region or country for which to retrieve data. May be one of the
#'   following regional aggregates: \code{"WORLD"} (default), \code{"UN_EUR"} (Europe),
#'   \code{"UN_AFR"} (Africa), \code{"UN_ASI"} (Asia), \code{"UN_AME"} (Americas),
#'   or \code{"UN_OCE"} (Oceania). Alternatively, supply an ISO 3166-1 alpha-3 country
#'   code such as \code{"USA"}, \code{"CAN"}, or \code{"BRA"}.
#' @param year (numeric) Four-digit year for which to retrieve data. Must be 2012 or later.
#'   Defaults to the current year.
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with weekly and cumulative wildfire burnt area and fire
#'   count data for the requested place and year. Weekly columns include \code{date},
#'   \code{events}, \code{events_min}, \code{events_max}, \code{events_avg},
#'   \code{area_ha}, \code{area_ha_min}, \code{area_ha_avg}, and \code{area_ha_max}.
#'   Cumulative columns are prefixed with \code{cum_}.
#'
#' @details `get_fires_area` invisibly returns a tibble with weekly and year-to-date cumulative
#'   wildfire burnt area (in hectares) and fire event counts sourced from the GWIS API.
#'   Historical minimum, maximum, and average baselines are included for comparison.
#'   Data coverage begins in 2012. Cache filenames include the place and year so that
#'   multiple queries can be cached independently.
#'
#' @examples
#' \donttest{
#' # Fetch global data for the current year from cache if available:
#' fires <- get_fires_area()
#' #
#' # Fetch data for Canada in 2023:
#' fires_can <- get_fires_area(place = 'CAN', year = 2023)
#' #
#' # Force cache refresh:
#' fires <- get_fires_area(use_cache = FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details() }
#'
#' @importFrom jsonlite fromJSON
#' @importFrom tibble as_tibble
#' @importFrom dplyr bind_cols select mutate
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' Global Wildfire Information System (GWIS), Copernicus Emergency Management Service / European Commission Joint Research Centre.
#' \url{https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend}
#'
#'
#' @export
get_fires_area <- function(place='WORLD', year=as.numeric(format(Sys.Date(), "%Y")), use_cache = TRUE,
                      write_cache = getOption("hs_write_cache")) {

  if (year < 2012) stop("year must be greater than 2011")

  url <- "https://api2.effis.emergency.copernicus.eu/statistics/v2/gwis/"

    valid_aois <- c(
    "UN_EUR",   # Europe
    "UN_AFR",   # Africa
    "UN_ASI",   # Asia
    "UN_AME",   # Americas
    "UN_OCE",   # Oceania
    "WORLD"     # Global
  )

  if (place %in% valid_aois) (url <- paste0(url, "weeklyaoi","?aoi=", place, "&year=", year)) else
    (url <- paste0(url, "weekly","?country=", place, "&year=", year))

    cachename <- paste0("fires_", place, year, ".rds")

  hs_path <- tools::R_user_dir("hockeystick","cache")

  if (use_cache) {
    if (file.exists(file.path(hs_path, cachename))) {
    cached_fires <- readRDS((file.path(hs_path, cachename)))
    return(invisible(cached_fires)) }}

  connected <- .isConnected('https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend')
  if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}


  con <- tryCatch(url(url, open = "rb"), error = function(e) NULL)
  if (is.null(con)) {message("Unable to access remote resource."); return(invisible(NULL))}

  resp <- readLines(con, warn = FALSE)
  close(con)

  raw <- fromJSON(resp, flatten = TRUE)

  weekly <- raw$banfweekly |> as_tibble()
  cumul <- raw$banfcumulative |> as_tibble()

  cumul <- cumul |> select(cum_events=events, cum_events_min=events_min, cum_events_max=events_max, cum_events_avg=events_avg,
             cum_area_ha=area_ha, cum_area_ha_min=area_ha_min, cum_area_ha_avg=area_ha_avg,  cum_area_ha_max=area_ha_max)

  fires <- bind_cols(weekly, cumul)
  fires <- fires |> mutate(date= as.Date(mddate, format = "%Y%m%d"), .keep='unused', .after="week")
  if (any(is.na(fires$date)) || !all(lubridate::year(fires$date) == year))
    warning("GWIS returned dates outside the requested year ", year)
  colnames(fires)[1] <- 'place'

  if (write_cache) saveRDS(fires, file.path(hs_path, cachename))
  return(fires)

}



#' Download and plot essential climate data
#'
#' Retrieves global wildfire greenhouse gas emissions data from the
#' Global Wildfire Information System (GWIS), operated by the Copernicus
#' Emergency Management Service.
#' \url{https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend}
#'
#' @name get_fires_emissions
#' @param place (string) Region or country for which to retrieve data. May be one of the
#'   following regional aggregates: \code{"WORLD"} (default), \code{"UN_EUR"} (Europe),
#'   \code{"UN_AFR"} (Africa), \code{"UN_ASI"} (Asia), \code{"UN_AME"} (Americas),
#'   or \code{"UN_OCE"} (Oceania). Alternatively, supply an ISO 3166-1 alpha-3 country
#'   code such as \code{"USA"}, \code{"CAN"}, or \code{"BRA"}.
#' @param year (numeric) Four-digit year for which to retrieve data. Must be 2012 or later.
#'   Defaults to the current year.
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with weekly and cumulative wildfire emissions data for
#'   the requested place and year. The \code{plt} column identifies the pollutant
#'   (e.g. \code{"CO2"}, \code{"CO"}, \code{"PM2.5"}). Weekly value columns include
#'   \code{curv} (current), \code{minv}, \code{maxv}, and \code{avgv}. Cumulative
#'   columns are prefixed with \code{cum_}.
#'
#' @details `get_fires_emissions` invisibly returns a tibble with weekly and year-to-date
#'   cumulative wildfire emissions by pollutant, sourced from the GWIS emissions API.
#'   Historical minimum, maximum, and average baselines are included for comparison.
#'   Data coverage begins in 2012. Filter on the \code{plt} column to select a specific
#'   pollutant (e.g. \code{filter(plt == "CO2")}). Cache filenames include the place
#'   and year so that multiple queries can be cached independently.
#'
#' @examples
#' \donttest{
#' # Fetch global emissions data for the current year from cache if available:
#' emissions <- get_fires_emissions()
#' #
#' # Fetch CO2 emissions for the USA in 2023:
#' emissions_usa <- get_fires_emissions(place = 'USA', year = 2023)
#' #
#' # Force cache refresh:
#' emissions <- get_fires_emissions(use_cache = FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details() }
#'
#' @importFrom jsonlite fromJSON
#' @importFrom tibble as_tibble
#' @importFrom dplyr bind_cols select mutate
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' Global Wildfire Information System (GWIS), Copernicus Emergency Management Service / European Commission Joint Research Centre.
#' \url{https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend}
#'
#'
#' @export
get_fires_emissions <- function(place='WORLD', year=as.numeric(format(Sys.Date(), "%Y")), use_cache = TRUE,
                           write_cache = getOption("hs_write_cache")) {

  if (year < 2012) stop("year must be greater than 2011")

  url <- "https://api2.effis.emergency.copernicus.eu/statistics/v2/emissions/"

  valid_aois <- c(
    "UN_EUR",   # Europe
    "UN_AFR",   # Africa
    "UN_ASI",   # Asia
    "UN_AME",   # Americas
    "UN_OCE",   # Oceania
    "WORLD"     # Global
  )

  if (place %in% valid_aois) (url <- paste0(url, "weeklyaoi","?aoi=", place, "&year=", year)) else
    (url <- paste0(url, "weekly","?country=", place, "&year=", year))

  cachename <- paste0("emissions_", place, year, ".rds")

  hs_path <- tools::R_user_dir("hockeystick","cache")

  if (use_cache) {
    if (file.exists(file.path(hs_path, cachename))) {
      cached_fires <- readRDS((file.path(hs_path, cachename)))
      return(invisible(cached_fires)) }}

  connected <- .isConnected('https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend')
  if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

  con <- url(url, open = "rb")
  resp <- readLines(con, warn = FALSE)
  close(con)
  raw <- fromJSON(resp, flatten = TRUE)

  weekly <- raw$emissionsweekly |> as_tibble()
  cumul <- raw$emissionsweeklycum |> as_tibble()

  cumul <- cumul |> select(plt=plt, dt=dt, cum_curv=curv, cum_minv=minv, cum_maxv=maxv, cum_avgv=avgv)

  emissions <- left_join(weekly, cumul, by=c('plt', 'dt'))
  emissions <- emissions |>  mutate(date = as.Date(dt, format = "%Y%m%d"), .keep='unused', .before = 'plt')
  emissions <- emissions |> mutate(place = place, .keep='unused', .before = 'date')

  if (any(is.na(emissions$date)) || !all(lubridate::year(emissions$date) == year))
    warning("GWIS returned dates outside the requested year ", year)

  if (write_cache) saveRDS(emissions, file.path(hs_path, cachename))
  return(emissions) }


#' Download and plot essential climate data
#'
#' Plots the wildfire burnt area or fire count data retrieved using `get_fires_area()` with ggplot2.
#' The output ggplot2 object may be further modified.
#'
#'
#' @name plot_fires_area
#' @param dataset Name of the tibble generated by `get_fires_area`
#' @param var (string) Select which variable to plot. May be \code{"area"} (burnt area in hectares, the default)
#'   or \code{"count"} (number of fire events).
#' @param style (string) Select cumulative or weekly view. May be \code{"cumulative"} (year-to-date totals,
#'   the default) or \code{"weekly"} (weekly values).
#' @param print (boolean) Display the ggplot2 chart, defaults to TRUE. Use FALSE to suppress display.
#'
#' @return Invisibly returns a ggplot2 object with the wildfire chart
#'
#' @details `plot_fires_area` invisibly returns a ggplot2 object with a pre-defined wildfire chart using
#' data from `get_fires_area`. By default the chart is also displayed. The shaded ribbon shows the
#' historical minimum-to-maximum range since 2012, and the blue line shows the historical average.
#' Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#'
#' @examples
#' \donttest{
#' # Fetch wildfire data:
#' fires <- get_fires_area()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_fires_area(fires)
#'
#' # Or just call plot_fires_area(), which defaults to get_fires_area() dataset
#' plot_fires_area()
#'
#' # Plot weekly fire count instead of cumulative burnt area
#' plot_fires_area(fires, var = 'count', style = 'weekly')
#'
#' p <- plot_fires_area(fires, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Global Wildfire Burnt Area') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export
plot_fires_area <- function(dataset = get_fires_area(), var=c('area', 'count'), style=c('cumulative', 'weekly'), print=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))
  style <- match.arg(style)
  var <- match.arg(var)

  fireyear <- dataset[1,3] |> pull() |> year()
  firelocation <- dataset[1, 'place'] |> pull()
  if (var=='area') firetitle <- paste0('Wildfire Area Burnt in ', fireyear)
  if (var=='count') firetitle <- paste0('Number of Wildfires in ', fireyear)
  plot <- NULL

  if(var=='area' && style=='cumulative') {
  plot <- ggplot(dataset, aes(x = date)) +theme_bw(base_size = 13) +geom_ribbon(aes(ymin=cum_area_ha_min, ymax=cum_area_ha_max, fill='Min - Max Range\n(since 2012)')) +
  geom_line(aes(y=cum_area_ha, col='Year-to-date'), linewidth=1.1, na.rm = T) + geom_line(aes(y=cum_area_ha_avg, col="Average\n(since 2012)")) +
  labs(y='Cumulative Burnt Area (thousands of ha)', x=NULL, title=firetitle, subtitle=firelocation, caption='Source: Global Wildfire Information System.') + scale_y_continuous(n.breaks = 8, labels = scales::label_comma(scale = .001)) +
  scale_colour_manual("",values=c("red", "dodgerblue"), breaks=c("Year-to-date","Average\n(since 2012)"))+
  scale_fill_manual('', values="grey90") +theme(legend.position = "top")
  }

  if(var=='area' && style=='weekly') {
  plot <- ggplot(dataset, aes(x = date)) +theme_bw(base_size = 13) +geom_ribbon(aes(ymin=area_ha_min, ymax=area_ha_max, fill='Min - Max Range\n(since 2012)')) +
  geom_line(aes(y=area_ha, col='Week'), linewidth=1.1, na.rm = T) + geom_line(aes(y=area_ha_avg, col="Average\n(since 2012)")) +
  labs(y='Weekly Burnt Area (thousands of ha)', x=NULL, title=firetitle, subtitle=firelocation, caption='Source: Global Wildfire Information System.') + scale_y_continuous(n.breaks = 8, labels = scales::label_comma(scale = .001)) +
  scale_colour_manual("",values=c("red", "dodgerblue"), breaks=c("Week","Average\n(since 2012)"))+
  scale_fill_manual('', values="grey90") +theme(legend.position = "top")
  }

  if(var=='count' && style=='cumulative') {
  plot <- ggplot(dataset, aes(x = date)) +theme_bw(base_size = 13) +geom_ribbon(aes(ymin=cum_events_min, ymax=cum_events_max, fill='Min - Max Range\n(since 2012)')) +
  geom_line(aes(y=cum_events, col='Year-to-date'), linewidth=1.1, na.rm=T) + geom_line(aes(y=cum_events_avg, col="Average\n(since 2012)")) +
  labs(y='Cumulative Number', x=NULL, title=firetitle, subtitle=firelocation, caption='Source: Global Wildfire Information System.') + scale_y_continuous(n.breaks = 8, labels = scales::label_comma()) +
  scale_colour_manual("",values=c("red", "dodgerblue"), breaks=c("Year-to-date","Average\n(since 2012)"))+
  scale_fill_manual('', values="grey90") +theme(legend.position = "top")
  }

  if(var=='count' && style=='weekly') {
  plot <- ggplot(dataset, aes(x = date)) +theme_bw(base_size = 13) +geom_ribbon(aes(ymin=events_min, ymax=events_max, fill='Min - Max Range\n(since 2012)')) +
  geom_line(aes(y=events, col='Week'), linewidth=1.1, na.rm=T) + geom_line(aes(y=events_avg, col="Average\n(since 2012)")) +
  labs(y='Weekly Number', x=NULL, title=firetitle, subtitle=firelocation, caption='Source: Global Wildfire Information System.') + scale_y_continuous(n.breaks = 8, labels = scales::label_comma()) +
  scale_colour_manual("",values=c("red", "dodgerblue"), breaks=c("Week","Average\n(since 2012)"))+
  scale_fill_manual('', values="grey90") +theme(legend.position = "top")
  }

  if (print) print(plot)
  invisible(plot)
}

#' Download and plot essential climate data
#'
#' Plots the wildfire greenhouse gas emissions data retrieved using `get_fires_emissions()` with ggplot2.
#' The output ggplot2 object may be further modified.
#'
#'
#' @name plot_fires_emissions
#' @param dataset Name of the tibble generated by `get_fires_emissions`
#' @param pollutant (string) Select which pollutant to plot. May be one of \code{"CO2"} (the default),
#'   \code{"BC"}, \code{"C"}, \code{"CH4"}, \code{"CO"}, \code{"NMHC"}, \code{"NOX"},
#'   \code{"OC"}, \code{"PM2.5"}, \code{"SO2"}, or \code{"TC"}.
#' @param style (string) Select cumulative or weekly view. May be \code{"cumulative"} (year-to-date totals,
#'   the default) or \code{"weekly"} (weekly values).
#' @param print (boolean) Display the ggplot2 chart, defaults to TRUE. Use FALSE to suppress display.
#'
#' @return Invisibly returns a ggplot2 object with the wildfire emissions chart
#'
#' @details `plot_fires_emissions` invisibly returns a ggplot2 object with a pre-defined wildfire
#' emissions chart using data from `get_fires_emissions`. By default the chart is also displayed.
#' The shaded ribbon shows the historical minimum-to-maximum range since 2003, and the blue line
#' shows the historical average. Emissions are expressed in millions of tons. Users may further
#' modify the output ggplot2 chart.
#'
#' @import ggplot2
#'
#' @examples
#' \donttest{
#' # Fetch wildfire emissions data:
#' emissions <- get_fires_emissions()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_fires_emissions(emissions)
#'
#' # Or just call plot_fires_emissions(), which defaults to get_fires_emissions() dataset
#' plot_fires_emissions()
#'
#' # Plot weekly PM2.5 emissions instead of cumulative CO2
#' plot_fires_emissions(emissions, pollutant = 'PM2.5', style = 'weekly')
#'
#' p <- plot_fires_emissions(emissions, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Global CO2 Emissions from Wildfires') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export
plot_fires_emissions <- function(dataset = get_fires_emissions(), pollutant=c('CO2', 'BC', 'C', 'CH4', 'CO', 'NMHC', 'NOX', 'OC', 'PM2.5', 'SO2', 'TC' ),
 style=c('cumulative', 'weekly'), print=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))
  pollutant <- match.arg(pollutant)
  style <- match.arg(style)

  fireyear <- dataset[1, "date"] |> pull() |> year()
  firearea <- dataset[1,"place"] |> pull()
  firetitle <- paste0(fireyear," ", pollutant, " Emissions from Wildfires")
  plot <- NULL

if(style=='cumulative') {
  plot <- dataset |> filter(plt==pollutant) |> ggplot(aes(x = date)) +theme_bw(base_size = 13) +geom_ribbon(aes(ymin=cum_minv, ymax=cum_maxv, fill='Min - Max Range\n(since 2003)')) +
  geom_line(aes(y=cum_curv, col='Year-to-date'), linewidth=1.1, na.rm = T) + geom_line(aes(y=cum_avgv, col="Average\n(since 2003)")) + labs(y='Cumulative Emissions (millions of tons)', x=NULL, title=firetitle, subtitle=firearea, caption='Source: Global Wildfire Information System.') + scale_y_continuous(n.breaks = 8, labels = scales::label_comma(scale = .000001)) +
  scale_colour_manual("",values=c("red", "dodgerblue"), breaks=c("Year-to-date","Average\n(since 2003)"))+
  scale_fill_manual('', values="grey90") +theme(legend.position = "top")
}

if(style=='weekly') {
  plot <- dataset |> filter(plt==pollutant) |> ggplot(aes(x = date)) +theme_bw(base_size = 13) +geom_ribbon(aes(ymin=minv, ymax=maxv, fill='Min - Max Range\n(since 2003)')) +
  geom_line(aes(y=curv, col='Week    '), linewidth=1.1, na.rm = T) + geom_line(aes(y=avgv, col="Average\n(since 2003)")) + labs(y='Weekly Emissions (millions of tons)', x=NULL, title=firetitle, subtitle=firearea, caption='Source: Global Wildfire Information System.') + scale_y_continuous(n.breaks = 8, labels = scales::label_comma(scale = .000001)) +
  scale_colour_manual("",values=c("red", "dodgerblue"), breaks=c("Week    ","Average\n(since 2003)"))+
  scale_fill_manual('', values="grey90") +theme(legend.position = "top") }

  if (print) print(plot)
  invisible(plot)
}

# ---------------------------------------------------------------------------
# Fixture data — mirrors the structure returned by each get_* function.
# All tests use these fixtures so no network access is required.
# ---------------------------------------------------------------------------

carbon_fixture <- tibble::tibble(
  year    = c(2020L, 2020L, 2020L),
  month   = c(1L, 2L, 3L),
  date    = as.Date(c("2020-01-31", "2020-02-29", "2020-03-31")),
  average = c(412.5, 413.4, 414.1),
  trend   = c(411.8, 412.7, 413.4),
  ndays   = c(28L, 27L, 30L),
  stdev   = c(0.5, 0.4, 0.5),
  unc     = c(0.1, 0.1, 0.1)
)

temp_fixture <- tibble::tibble(
  Year = as.Date(c("1980-12-31", "1981-12-31", "1982-12-31")),
  Jan  = c(0.20, 0.30, 0.10),
  Feb  = c(0.10, 0.20, 0.15),
  Mar  = c(0.30, 0.40, 0.20),
  Apr  = c(0.20, 0.30, 0.10),
  May  = c(0.10, 0.20, 0.00),
  Jun  = c(0.00, 0.10, -0.10),
  Jul  = c(0.10, 0.20, 0.00),
  Aug  = c(0.20, 0.30, 0.10),
  Sep  = c(0.30, 0.40, 0.20),
  Oct  = c(0.40, 0.50, 0.30),
  Nov  = c(0.30, 0.40, 0.20),
  Dec  = c(0.20, 0.30, 0.10),
  `J-D` = c(0.20, 0.30, 0.10),
  `D-N` = c(0.20, 0.30, 0.10),
  DJF   = c(0.15, 0.25, 0.12),
  MAM   = c(0.20, 0.30, 0.10),
  JJA   = c(0.10, 0.20, 0.00),
  SON   = c(0.33, 0.43, 0.23)
)

seaice_fixture <- tibble::tibble(
  date   = as.Date(c("1979-07-31", "1980-07-31", "1981-07-31")),
  extent = c(8.5, 8.2, 8.0)
)

icecurves_fixture <- {
  df <- tibble::tibble(
    year   = rep(c(2022L, 2023L), each = 12L),
    month  = rep(1:12, 2L),
    extent = c(14.0, 15.0, 14.0, 13.0, 11.0, 10.0,  7.0,  5.0,  6.0,  8.0, 11.0, 13.0,
               13.0, 14.0, 13.0, 12.0, 10.0,  9.0,  7.0,  5.5,  6.5,  8.5, 10.5, 12.5)
  )
  attr(df, "hs_ice_type") <- c("N", "extent")
  df
}

hurricanes_fixture <- data.frame(
  Year                   = 1990:1995,
  RevisedNamedStorms     = c(12, 8, 6, 8, 7, 19),
  RevisedHurricanes      = c( 8, 4, 4, 4, 3, 11),
  RevisedMajorHurricanes = c( 1, 2, 1, 1, 0,  3),
  RevisedACE             = c(91, 34, 75, 38, 32, 228),
  RevisedUShurricanes    = c( 1, 0, 1, 1, 0,  2)
)

emissions_fixture <- tibble::tibble(
  country             = rep("World", 5L),
  year                = 2000:2004,
  co2                 = c(25000, 26000, 27000, 28000, 29000),
  land_use_change_co2 = c(3000, 3100, 3200, 3300, 3400)
)

merged_fixture <- tibble::tibble(
  Date    = as.Date(c("2020-01-31", "2020-02-29", "2020-03-31")),
  Year    = c(2020, 2020, 2020),
  Carbon  = c(412.5, 413.4, 414.1),
  Anomaly = c(0.80, 0.90, 1.00)
)

# ---------------------------------------------------------------------------
# Input validation — no network access required
# ---------------------------------------------------------------------------

test_that("get_seaice rejects invalid pole", {
  expect_error(get_seaice(pole = "X"), "pole must be")
})

test_that("get_seaice rejects invalid month", {
  expect_error(get_seaice(month = 13), "Month must be")
})

test_that("get_seaice rejects invalid measure", {
  expect_error(get_seaice(measure = "volume"), "measure must be")
})

test_that("get_icecurves rejects invalid pole", {
  expect_error(get_icecurves(pole = "E"), "pole must be")
})

test_that("get_icecurves rejects invalid measure", {
  expect_error(get_icecurves(measure = "volume"), "measure must be")
})

# ---------------------------------------------------------------------------
# Plot functions return ggplot objects given fixture data
# ---------------------------------------------------------------------------

test_that("plot_carbon returns a ggplot", {
  p <- plot_carbon(carbon_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_temp returns a ggplot", {
  p <- plot_temp(temp_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_temp_monthly returns a ggplot", {
  p <- plot_temp_monthly(temp_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_temp_scatter returns a ggplot", {
  p <- plot_temp_scatter(temp_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("warming_stripes returns a ggplot", {
  p <- warming_stripes(temp_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("warming_stripes stripe_only = TRUE returns a ggplot", {
  p <- warming_stripes(temp_fixture, stripe_only = TRUE, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_seaice returns a ggplot", {
  p <- plot_seaice(seaice_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_icecurves returns a ggplot", {
  p <- plot_icecurves(icecurves_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_hurricanes returns a ggplot", {
  p <- plot_hurricanes(hurricanes_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_hurricane_nrg returns a ggplot", {
  p <- plot_hurricane_nrg(hurricanes_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_emissions returns a ggplot", {
  p <- plot_emissions(emissions_fixture, print = FALSE, annot = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_emissions_with_land returns a ggplot", {
  p <- plot_emissions_with_land(emissions_fixture, print = FALSE, annot = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_carbontemp returns a ggplot", {
  p <- plot_carbontemp(merged_fixture, print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

# ---------------------------------------------------------------------------
# NULL propagation — every plot function should return NULL silently
# ---------------------------------------------------------------------------

test_that("plot functions return NULL invisibly when passed NULL", {
  expect_null(plot_carbon(NULL,               print = FALSE))
  expect_null(plot_temp(NULL,                 print = FALSE))
  expect_null(plot_temp_monthly(NULL,         print = FALSE))
  expect_null(plot_temp_scatter(NULL,         print = FALSE))
  expect_null(warming_stripes(NULL,           print = FALSE))
  expect_null(plot_seaice(NULL,               print = FALSE))
  expect_null(plot_icecurves(NULL,            print = FALSE))
  expect_null(plot_hurricanes(NULL,           print = FALSE))
  expect_null(plot_hurricane_nrg(NULL,        print = FALSE))
  expect_null(plot_emissions(NULL,            print = FALSE))
  expect_null(plot_emissions_with_land(NULL,  print = FALSE))
  expect_null(plot_carbontemp(NULL,           print = FALSE))
})

# ---------------------------------------------------------------------------
# Fixtures for fires functions
# ---------------------------------------------------------------------------

fires_area_fixture <- tibble::tibble(
  place        = rep("WORLD", 4L),
  week         = 1:4,
  date         = as.Date(c("2024-01-07", "2024-01-14", "2024-01-21", "2024-01-28")),
  events       = c(100L, 120L, 110L, 130L),
  events_min   = c( 80L,  90L,  85L, 100L),
  events_max   = c(150L, 160L, 155L, 170L),
  events_avg   = c(110L, 115L, 112L, 120L),
  area_ha      = c(5000L, 6000L, 5500L, 7000L),
  area_ha_min  = c(3000L, 3500L, 3200L, 4000L),
  area_ha_avg  = c(4500L, 5000L, 4800L, 5500L),
  area_ha_max  = c(8000L, 9000L, 8500L, 9500L),
  cum_events      = c(100L,  220L,  330L,  460L),
  cum_events_min  = c( 80L,  170L,  255L,  355L),
  cum_events_max  = c(150L,  310L,  465L,  635L),
  cum_events_avg  = c(110L,  225L,  337L,  457L),
  cum_area_ha     = c(5000L, 11000L, 16500L, 23500L),
  cum_area_ha_min = c(3000L,  6500L,  9700L, 13700L),
  cum_area_ha_avg = c(4500L,  9500L, 14300L, 19800L),
  cum_area_ha_max = c(8000L, 17000L, 25500L, 35000L)
)

fires_emissions_fixture <- tibble::tibble(
  place      = rep("WORLD", 6L),
  date       = rep(as.Date(c("2024-01-07", "2024-01-14", "2024-01-21")), 2L),
  plt        = c(rep("CO2", 3L), rep("PM2.5", 3L)),
  curv       = c(100, 120, 110, 10, 12, 11),
  minv       = c( 80,  90,  85,  8,  9,  8),
  maxv       = c(150, 160, 155, 15, 16, 15),
  avgv       = c(110, 115, 112, 11, 11, 11),
  cum_curv   = c(100, 220, 330, 10, 22, 33),
  cum_minv   = c( 80, 170, 255,  8, 17, 25),
  cum_maxv   = c(150, 310, 465, 15, 31, 46),
  cum_avgv   = c(110, 225, 337, 11, 22, 33)
)

# ---------------------------------------------------------------------------
# Input validation — fires functions
# ---------------------------------------------------------------------------

test_that("get_fires_area rejects years before 2012", {
  expect_error(get_fires_area(year = 2011), "year must be greater than 2011")
})

test_that("get_fires_emissions rejects years before 2012", {
  expect_error(get_fires_emissions(year = 2011), "year must be greater than 2011")
})

# ---------------------------------------------------------------------------
# plot_fires_area — all var/style combinations return ggplot
# ---------------------------------------------------------------------------

test_that("plot_fires_area var=area style=cumulative returns a ggplot", {
  p <- plot_fires_area(fires_area_fixture, var = "area", style = "cumulative", print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_fires_area var=area style=weekly returns a ggplot", {
  p <- plot_fires_area(fires_area_fixture, var = "area", style = "weekly", print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_fires_area var=count style=cumulative returns a ggplot", {
  p <- plot_fires_area(fires_area_fixture, var = "count", style = "cumulative", print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_fires_area var=count style=weekly returns a ggplot", {
  p <- plot_fires_area(fires_area_fixture, var = "count", style = "weekly", print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

# ---------------------------------------------------------------------------
# plot_fires_emissions — pollutant/style combinations return ggplot
# ---------------------------------------------------------------------------

test_that("plot_fires_emissions pollutant=CO2 style=cumulative returns a ggplot", {
  p <- plot_fires_emissions(fires_emissions_fixture, pollutant = "CO2", style = "cumulative", print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_fires_emissions pollutant=CO2 style=weekly returns a ggplot", {
  p <- plot_fires_emissions(fires_emissions_fixture, pollutant = "CO2", style = "weekly", print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

test_that("plot_fires_emissions pollutant=PM2.5 style=cumulative returns a ggplot", {
  p <- plot_fires_emissions(fires_emissions_fixture, pollutant = "PM2.5", style = "cumulative", print = FALSE)
  expect_true(inherits(p, "ggplot"))
})

# ---------------------------------------------------------------------------
# NULL propagation — fires plot functions
# ---------------------------------------------------------------------------

test_that("plot_fires_area returns NULL invisibly when passed NULL", {
  expect_null(plot_fires_area(NULL, print = FALSE))
})

test_that("plot_fires_emissions returns NULL invisibly when passed NULL", {
  expect_null(plot_fires_emissions(NULL, print = FALSE))
})

# ---------------------------------------------------------------------------
# merge_carbontemp output structure
# ---------------------------------------------------------------------------

test_that("merge_carbontemp returns expected columns", {
  result <- merge_carbontemp(carbon = carbon_fixture, temp = temp_fixture)
  expect_true(is.data.frame(result))
  expect_true(all(c("Date", "Year", "Carbon", "Anomaly") %in% colnames(result)))
})

test_that("merge_carbontemp returns NULL when either input is NULL", {
  expect_null(merge_carbontemp(carbon = NULL,          temp = temp_fixture))
  expect_null(merge_carbontemp(carbon = carbon_fixture, temp = NULL))
})

#' Manage cached datasets
#'
#' @name hockeystick_cache
#' @param files (character) one or more complete file names
#' @param force (logical) Should files be force deleted? Default: `TRUE`
#'
#' @details `cache_delete` only accepts 1 file name, while `cache_delete_all`
#' doesn't accept any names, but deletes all files. For deleting many
#' specific files, use `cache_delete` in a [lapply()] type call
#'
#' @details We cache using [tools::R_user_dir()], find your cache
#' folder by executing `tools::R_user_dir("hockeystick","cache")`
#'
#' @section Functions:
#' \itemize{
#'  \item `hockeystick_cache_list()` returns a character vector of full path
#'  file names in cache
#'  \item `hockeystick_cache_delete()` deletes one or more files, returns nothing
#'  \item `hockeystick_cache_delete_all()` delete all files, returns nothing
#'  \item `hockeystick_cache_details()` prints file name and file size of each file, supply with one or more files, or no files (and get details for all available)
#'  \item `hockeystick_update_all()` updates the cache with the latest co2, temperature, sea level, and sea ice data.
#' }
#'
#' @examples \donttest{
#' # list files in cache
#' hockeystick_cache_list()
#'
#' # List info for single files
#' hockeystick_cache_details(files = hockeystick_cache_list()[1])
#' hockeystick_cache_details(files = hockeystick_cache_list()[2])
#'
#' # List info for all files
#' hockeystick_cache_details()
#'
#' # Delete cached files by name
#' hockeystick_cache_delete(files = hockeystick_cache_list()[1])
#'
#' # Update all hockeystick data and place in cache
#' hockeystick_update_all()
#'
#' # Delete all cached data
#' hockeystick_cache_delete_all()
#' }
#' @references
#' Caching data sets: ROpenSci guide to persistent config and data for R packages: https://blog.r-hub.io/2020/03/12/user-preferences/

#' @export
#' @return `hockeystick_cache_list()` returns a character vector of full path filenames in cache.
#' @rdname hockeystick_cache
hockeystick_cache_list <- function() {
  list.files(hscache_path(), pattern = ".rds", ignore.case = TRUE,
             recursive = TRUE, full.names = TRUE)
}

#' @export
#' @return `hockeystick_cache_delete()` no return value, called for side effect.
#' @rdname hockeystick_cache
hockeystick_cache_delete <- function(files, force = TRUE) {
  if (!all(file.exists(files))) {
    message("These files don't exist or can't be found: \n",
         strwrap(files[!file.exists(files)], indent = 5), call. = FALSE)
  }
  invisible(file.remove(files))
}

#' @export
#' @return `hockeystick_cache_delete_all()` no return value, called for side effect.
#' @rdname hockeystick_cache
hockeystick_cache_delete_all <- function(force = TRUE) {
  files <- list.files(hscache_path(), pattern = ".rds", ignore.case = TRUE,
                      full.names = TRUE, recursive = TRUE)
  invisible(file.remove(files))
}

#' @export
#' @return `hockeystick_cache_details()` returns list of filenames and sizes of cached files.
#' @rdname hockeystick_cache
hockeystick_cache_details <- function(files = NULL) {
  if (is.null(files)) {
    files <- list.files(hscache_path(), pattern = ".rds", ignore.case = TRUE,
                        full.names = TRUE, recursive = TRUE)
    structure(lapply(files, file_info_), class = "hockeystick_cache_info")
  } else {
    structure(lapply(files, file_info_), class = "hockeystick_cache_info")
  }
}

#' Display cached file info
#'
#' Internal function
#' @param x filenames
file_info_ <- function(x) {
  fs <- file.size(x)
  list(file = x,
       type = "rds",
       size = if (!is.na(fs)) getsize(fs) else NA,
       date = if (!is.na(fs)) file.mtime(x) else NA
  )
}

#' Get rounded size of file in kB
#'
#' Internal function
#' @param x filenames
getsize <- function(x) {
  round(x/10^3, 1)
}

#' Display data cache info
#' Shows filenames and cache file sizes
#' @param x filenames
#' @param ... Additional parameters
#' @return Display data cache info. No return value, called for side effect.
#' @export
#' @method print hockeystick_cache_info
print.hockeystick_cache_info <- function(x, ...) {
  cat("<hockeystick cached files>", sep = "\n")
  cat(sprintf("  directory: %s\n", hscache_path()), sep = "\n")
  for (i in seq_along(x)) {
    cat(paste0("  file: ", sub(hscache_path(), "", x[[i]]$file)), sep = "\n")
    cat(paste0("  size: ", x[[i]]$size, " kB"), sep = "\n")
    cat(paste0("  date: ", x[[i]]$date), sep='\n')
    cat("\n")
  }
}

#' Return path of data cache directory
#'
#' Internal Function
#' @return Return path of data cache directory
hscache_path <- function() tools::R_user_dir("hockeystick","cache")


#' @export
#' @return `hockeystick_update_all()` updates all datasets and caches them. No return value, called for side effect.
#' @rdname hockeystick_cache
hockeystick_update_all <- function() {
  get_carbon(use_cache = FALSE, write_cache = TRUE)
  get_temp(use_cache = FALSE, write_cache = TRUE)
  get_seaice(use_cache = FALSE, write_cache = TRUE)
  get_sealevel(use_cache = FALSE, write_cache = TRUE)
  get_icecurves(use_cache = FALSE, write_cache = TRUE)
  get_temp2k(use_cache = FALSE, write_cache = TRUE)
  get_hurricanes(use_cache = FALSE, write_cache = TRUE)
  get_emissions(use_cache = FALSE, write_cache = TRUE)
  get_methane(use_cache = FALSE, write_cache = TRUE)
  get_dailytemp(use_cache = FALSE, write_cache = TRUE)
get_dailytempcop(use_cache = FALSE, write_cache = TRUE)
get_fires_area(use_cache = FALSE, write_cache = TRUE)
get_fires_emissions(use_cache = FALSE, write_cache = TRUE)
  invisible(0)
}

# Setting global variables to avoid R CMD check notices
utils::globalVariables(c('average', 'trend', 'year','Year', 'J-D', 'gmsl', 'method', 'month', 'name',
                         'age_ice', 'co2', 'land_use_change_co2', 'temp', 'value', 'size', 'measure','RevisedMajorHurricanes', 'RevisedACE',
                         'cumco2', 'country', 'data1', 'data366', 'day_of_year', 'dummy_date', 'linew', 'mean_temp', 'temp_anom',
                         'hurr_type', 'ice_region', 'data_source', 'Anomaly', 'Month', 'Date', 'Carbon','1991-2020 mean',
                         'area_ha', 'area_ha_avg', 'area_ha_max', 'area_ha_min', 'avgv', 'cum_area_ha',
                         'cum_area_ha_avg', 'cum_area_ha_max', 'cum_area_ha_min', 'cum_avgv', 'cum_curv',
                         'cum_events', 'cum_events_avg', 'cum_events_max', 'cum_events_min', 'cum_maxv',
                         'cum_minv curv', 'dt', 'events', 'events_avg', 'events_max', 'events_min', 'fires', 'maxv',
                         'mddate', 'minv', 'plt', 'cum_minv', 'curv' ))

## This function has been provided by Dirk Eddelbuettel's tint package
## https://github.com/eddelbuettel/tint/blob/master/R/isConnected.R
## It's purpose is to check for internet connectivity.
##
##
## borrowed from our dang package, but made internal here, in order to not have
## to declare a dependency for just these few lines of code
##
## Copyright Dirk Eddelbuettel
## Licensed as GPL (>= 2)
## https://cran.r-project.org/package=dang

.isConnected <- function(site="https://www.google.com") {
  uoc <- function(site) {
    con <- url(site)                # need to assign so that we can close
    open(con)                       # in case of success we have a connection
    close(con)                      # ... so we need to clean up
  }
  suppressWarnings(!inherits(try(uoc(site), silent=TRUE), "try-error"))
}

#' Download and plot essential climate data
#'
#' Retrieves Global Carbon Project (GCP) annual global carbon dioxide emissions since 1750 from Our World In Data repository
#' \url{https://github.com/owid/co2-data}
#'
#' @name get_emissions
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with annual carbon dioxide emissions
#'
#' @details `get_emissions` invisibly returns a tibble with GCP's annual carbon dioxide emissions from fossil fuels in aggregate and for every nation.
#' The returned object includes ISO code, country, year, co2 emissions, growth rates, per capita, and decompositions by industry and gas type.
#' Please refer to above website for details.
#'
#' @importFrom readr read_csv col_skip cols
#' @importFrom utils download.file
#'
#' @examples
#' \donttest{
#' # Fetch from cache if available:
#' emissions <- get_emissions()
#' #
#' # Force cache refresh:
#' emissions <- get_emissions(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_emissions(emissions) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \url{https://www.globalcarbonproject.org/carbonbudget/}
#'
#'
#' Friedlingstein, P. et al (2020), Global Carbon Budget 2020, \emph{Earth System Science Data}, vol. 12, 3269-3340 \doi{10.5194/essd-12-3269-2020}
#' @export
get_emissions <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

hs_path <- tools::R_user_dir("hockeystick","cache")

if (use_cache & !write_cache) {
  if (file.exists(file.path(hs_path,'emissions.rds'))) return(invisible(readRDS((file.path(hs_path,'emissions.rds')))))
  }

file_url <- 'https://github.com/owid/co2-data/raw/master/owid-co2-data.csv'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

dl <- tempfile()
status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}
emissions <- suppressMessages( read_csv(dl,  col_types = cols(other_industry_co2 = col_skip(), other_co2_per_capita = col_skip())) )

dir.create(hs_path, showWarnings = FALSE, recursive = TRUE)
if (write_cache) saveRDS(emissions, file.path(hs_path, 'emissions.rds'))

invisible(emissions)
}



#' Download and plot essential climate data
#'
#' Plots carbon dioxide emissions retrieved using `get_emissions()` with ggplot2. The output ggplot2 object may be modified. Alternative columns from the dataset may also be plotted.
#'
#'
#' @name plot_emissions
#' @param dataset Name of the tibble generated by `get_emissions`
#' @param print (boolean) Display carbon dioxide emissions ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param annot (boolean) Include chart annotation with latest date and value, defaults to TRUE.
#' @param start_year Year to start plot at. Defaults to 1900. Data is available since 1750.
#' @param region ISO code of region to plot. Defaults to 'OWID_WRL' which signifies entire world.
#' @param field Field from GCP dataset to be plotted, defaults to 'co2'
#' @param title_expression Chart title, defaults to CO2 emissions
#' @param yaxis_expression y-axis label, defaults to Gt CO2 emissions
#'
#'
#' @return Invisibly returns a ggplot2 object with carbon dioxide emissions chart
#'
#' @details `plot_emissions` invisibly returns a ggplot2 object with a pre-defined carbon dioxide emissions chart using data from `get_emissions`.  Use the `field` parameter to select alternative columns from the data set such as co2_per_capita.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom dplyr pull slice
#'
#' @examples
#' \donttest{
#' # Fetch carbon dioxide emissions:
#' emissions <- get_emissions()
#'
#' # Plot output using package's built-in ggplot2 defaults
#' plot_emissions(emissions)
#'
#' # Or just call plot_emissions(), which defaults to get_emissions() dataset
#' plot_emissions()
#'
#' # You can also select a region by name and start year
#' plot_emissions(region='United States', start_year=1950)
#' p <- plot_emissions(emissions, print = FALSE)
#'
#' # Modify plot such as: p + ggplot2::labs(title='Anthropogenic Carbon Emissions')
#'
#' # Plot a different field from GCP dataset
#' plot_emissions(field='co2_per_capita', yaxis_expression=expression(CO[2]*' per capita' ))}
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_emissions <- function(dataset = get_emissions(),
                  start_year = 1900, region = 'World',
                  field = 'co2', print = TRUE, annot = TRUE,
                  title_expression = expression('Fossil Combustion '*CO[2]*' Emissions'),
                  yaxis_expression = expression('Gt '*CO[2]*' per year' )) {

  if (is.null(dataset)) return(invisible(NULL))

  dataset <- filter(dataset, country == region)
  dataset <- filter(dataset, year >= start_year)
  dataset$co2 <- dataset$co2/1000

plot <- ggplot(dataset, aes(x=year, y=get(field))) +geom_line(linewidth=1, color='firebrick1') + theme_bw(base_size=12) +
  scale_x_continuous(n.breaks = 10, minor_breaks = NULL) + scale_y_continuous(limits = c(0, max(dataset[, field])), n.breaks = 6)  +
  labs(title =  title_expression, subtitle=dataset$country[1], x=NULL,
    y = yaxis_expression, caption='Source: Global Carbon Project and Our World In Data.\nhttps://github.com/owid/co2-data')

if (annot) {

dtmax <- pull(slice(dataset, which.max(year)), year)
dtmin <- pull(slice(dataset, which.min(year)), year)
vl <- pull(slice(dataset, which.max(year)), field)
vl <- round(vl, 1)

plot <- plot + annotate("text", x = dtmin+(dtmax-dtmin)/10, y=vl*0.9, label=paste(dtmax, vl,sep=": "), color='red')
}

if (print) print(plot)
invisible(plot)
}



#' Download and plot essential climate data
#'
#' Plots carbon dioxide emissions (including land use change) retrieved using `get_emissions()` with ggplot2. The output ggplot2 object may be modified. Alternative columns from the dataset may also be plotted.
#'
#'
#' @name plot_emissions_with_land
#' @param dataset Name of the tibble generated by `get_emissions`
#' @param print (boolean) Display carbon dioxide emissions ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param annot (boolean) Include chart annotation with latest date and value, defaults to TRUE.
#' @param start_year Year to start plot at. Defaults to 1900. Data is available since 1750.
#' @param region ISO code of region to plot. Defaults to 'OWID_WRL' which signifies entire world.
#' @param title_expression Chart title, defaults to CO2 emissions
#' @param yaxis_expression y-axis label, defaults to Gt CO2 emissions
#'
#'
#' @return Invisibly returns a ggplot2 object with carbon dioxide emissions chart
#'
#' @details `plot_emissions_with_land` invisibly returns a ggplot2 object with a pre-defined carbon dioxide emissions (including land use change) chart using data from `get_emissions`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom dplyr pull slice
#' @importFrom stats na.omit
#'
#' @examples
#' \donttest{
#' # Fetch carbon dioxide emissions:
#' emissions <- get_emissions()
#'
#' # Plot output (including land use change) using package's built-in ggplot2 defaults
#' plot_emissions_with_land(emissions)
#'
#' # Or just call plot_emissions_with_land(), which defaults to get_emissions() dataset
#' plot_emissions_with_land()
#'
#' # You can also select a region by name and starting year
#' plot_emissions_with_land(region='United States', start_year=1950)
#' p <- plot_emissions_with_land(emissions, print = FALSE)
#'
#' # Modify plot such as: p + ggplot2::labs(title='Anthropogenic Carbon Emissions')}
#'
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_emissions_with_land <- function(dataset = get_emissions(),
                           start_year = 1900, region = 'World',
                           print = TRUE, annot = TRUE,
                           title_expression = expression('Fossil + Land Use Change '*CO[2]*' Emissions'),
                           yaxis_expression = expression('Gt '*CO[2]*' per year' )) {

  if (is.null(dataset)) return(invisible(NULL))

  dataset <- filter(dataset, country == region & year >= start_year)
  dataset <- select(dataset, year, country, co2, land_use_change_co2)
  maxcombo <- max(dataset$co2+dataset$land_use_change_co2)/1000
  dataset <- pivot_longer(dataset,-c(year,country), values_to = c("co2"))
  dataset <- na.omit(mutate(dataset, co2=co2/1000))

  plot <- ggplot(dataset, aes(x=year, y=co2, fill=name)) +geom_area(linewidth=1, position='stack') + theme_bw(base_size=12) +
    scale_x_continuous(n.breaks = 10, minor_breaks = NULL) + theme(legend.position=c(0.18, 0.85), legend.title = element_blank()) +
    scale_y_continuous(limits = c( min(0, 1.2*min(dataset[, "co2"])) , 1.2*maxcombo ), n.breaks = 6) +
    scale_fill_manual(labels=c('Fossil Combustion', 'Land Use Change'), values=c("firebrick1","burlywood4")) +
    labs(title =  title_expression, subtitle=dataset$country[1], x=NULL, y = yaxis_expression, caption='Source: Global Carbon Project and Our World In Data.\nhttps://github.com/owid/co2-data')

  if (annot) {

    dtmax <- pull(slice(dataset, which.max(year)), year)
    dtmin <- pull(slice(dataset, which.min(year)), year)
    sumco2 <- filter(dataset, year==dtmax)
    vl <- as.numeric(summarize(sumco2, sum(co2)))
    #vl <- round(vl, 1)

    plot <- plot + annotate("text", x = dtmin+(dtmax-dtmin)/12, y=vl*0.9, label=paste(dtmax, signif(vl, 2),sep=": "), color='red')
  }

  if (print) print(plot)
  invisible(plot)
}

#' Download and plot essential climate data
#'
#' Retrieves Atlantic basin hurricane data since 1851 from National Oceanic and Atmospheric Administration
#' HURDAT Atlantic Hurricane Database Re-analysis Project.
#' \url{https://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html}
#'
#' @name get_hurricanes
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with the annual HURDAT hurricane data since 1851
#'
#' @details `get_hurricanes` invisibly returns a tibble with NOAA's annual North Atlantic revised HURDAT hurricane data since 1851.
#' The returned object includes Year, and number of named storms, hurricanes, major hurricanes (S-S scale 3-5), Accumulated Cyclone Energy (ACE), and U.S. hurricane strikes.
#'
#' ACE is an index that combines the number of systems, how long they existed and how intense they became. It is calculated by squaring the maximum sustained surface wind in the system every six hours that the cyclone is a Named Storm and summing it up for the season. Please refer to above website for details.
#'
#' @import rvest
#'
#' @examples
#' \donttest{
#' # Fetch from cache if available:
#' hurricanes <- get_hurricanes()
#' #
#' # Force cache refresh:
#' hurricanes <- get_hurricanes(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_hurricanes(hurricanes) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' HURDAT North Atlantic Hurricane Database Re-analysis Project, Hurricane Research Division, NOAA \url{https://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html}.
#'
#' Data from: \url{https://www.aoml.noaa.gov/hrd/hurdat/comparison_table.html}
#'
#'
#' \url{https://en.wikipedia.org/wiki/Accumulated_cyclone_energy}
#' @export
get_hurricanes <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

hs_path <- tools::R_user_dir("hockeystick","cache")

if (use_cache) {
  if (file.exists(file.path(hs_path,'hurricanes.rds'))) return(invisible(readRDS((file.path(hs_path,'hurricanes.rds')))))
  }

file_url <- "https://www.aoml.noaa.gov/hrd/hurdat/comparison_table.html"
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

hurricanes <- tryCatch(read_html(file_url), error = function(e) NULL)
if (is.null(hurricanes)) {message("Unable to access remote resource."); return(invisible(NULL))}

hurricanes <- html_node(hurricanes, xpath='//*[(@id = "tdcontent")]//table')
hurricanes <- html_table(hurricanes)

colnames(hurricanes) <- hurricanes[2,]
hurricanes <- hurricanes[3:nrow(hurricanes), c(1,3,5,7,9,11)]

hurricanes <- tidyr::separate(hurricanes, 6, sep=1, into=c("RevisedUShurricanes", NA), fill = "right")

hurricanes <- dplyr::mutate_all(hurricanes, as.numeric)

dir.create(hs_path, showWarnings = FALSE, recursive = TRUE)
if (write_cache) saveRDS(hurricanes, file.path(hs_path, 'hurricanes.rds'))

invisible(hurricanes)
}


#' Download and plot essential climate data
#'
#' Plots the hurricane data retrieved using `get_hurricanes()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_hurricanes
#' @param dataset Name of the tibble generated by `get_hurricanes`
#' @param print (boolean) Display hurricane ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param cat (string) Select which category of hurricane to plot. May be "major", "hurricane", or "storm". Defaults to "major".
#'
#' @return Invisibly returns a ggplot2 object with hurricanes chart
#'
#' @details `plot_hurricanes` invisibly returns a ggplot2 object with a pre-defined hurricane data chart using data from `get_hurricanes`.
#' By default the chart is also displayed. Smooths using ggplot2's built-in loess smoother. Users may further modify the output ggplot2 chart.
#' Categories may be "major" (category 3-5), "hurricane" (category 1-5), or "storm" (named storm).
#'
#' @import ggplot2
#'
#' @examples
#' \donttest{
#' # Fetch hurricane data:
#' hurricanes <- get_hurricanes()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_hurricanes(hurricanes)
#'
#' # Or just call plot_hurricanes(), which defaults to get_hurricanes() dataset
#' plot_hurricanes()
#'
#' p <- plot_hurricanes(hurricanes, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Growing number of North Atlantic named storms') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export
plot_hurricanes <- function(dataset = get_hurricanes(), cat='major', print=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  if(cat=='major') {dataset <- dataset[,c('Year', 'RevisedMajorHurricanes')]
                    sub_label <- 'Major Hurricanes (category 3-5)' }

  if(cat=='hurricane') {dataset <- dataset[,c('Year', 'RevisedHurricanes')]
                        sub_label <- 'Hurricanes (category 1-5)' }

  if(cat=='storm') {dataset <- dataset[,c('Year', 'RevisedNamedStorms')]
                    sub_label <- 'Named Storms' }

  colnames(dataset)[2] <- 'hurr_type'
  if (ncol(dataset) != 2) return(invisible(NULL))


    plot <- ggplot(dataset, aes(x=Year, y=hurr_type)) +geom_line(alpha=0.75, aes(color='Hurricanes')) + theme_bw(base_size=12) +
    scale_x_continuous(name=NULL, breaks=seq(1850, 2025,25)) +
    scale_y_continuous(n.breaks = 8) +geom_smooth(linewidth=1.1, se=FALSE, aes(color='Loess smooth')) +
    scale_color_manual(name=NULL, values=c('dodgerblue2','firebrick1')) +theme(legend.position = c(0.15, 0.825),legend.background=element_blank()) +
    labs(title='North Atlantic Hurricane Basin', subtitle = sub_label,
        y='Number per season', caption='Source: NOAA Hurricane Research Division, HURDAT2 re-analysis\nhttps://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html')

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}


#' Download and plot essential climate data
#'
#' Plots the hurricane energy data (ACE) retrieved using `get_hurricanes()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_hurricane_nrg
#' @param dataset Name of the tibble generated by `get_hurricanes`
#' @param print (boolean) Display hurricane ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#'
#' @return Invisibly returns a ggplot2 object with hurricane energy chart
#'
#' @details `plot_hurricane_nrg` invisibly returns a ggplot2 object with a pre-defined hurricane energy data chart using data from `get_hurricanes`.
#' By default the chart is also displayed. Smooths using ggplot2's built-in loess smoother. Users may further modify the output ggplot2 chart.
#'
#' ACE is an index that combines the number of systems, how long they existed and how intense they became. It is calculated by squaring the maximum sustained surface wind in the system every six hours that the cyclone is a Named Storm and summing it up for the season. Please refer to above website for details.
#'
#' @import ggplot2
#'
#' @examples
#' \donttest{
#' # Fetch hurricane data:
#' hurricanes <- get_hurricanes()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_hurricane_nrg(hurricanes)
#'
#' # Or just call plot_hurricane_nrg(), which defaults to get_hurricanes() dataset
#' plot_hurricane_nrg()
#'
#' p <- plot_hurricane_nrg(hurricanes, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Accumulated Cyclone Energy') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export
plot_hurricane_nrg <- function(dataset = get_hurricanes(), print=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  plot <- ggplot(dataset, aes(x=Year, y=RevisedACE)) +geom_line(alpha=0.75, aes(color='ACE')) + theme_bw(base_size=12) +
    scale_x_continuous(name=NULL, breaks=seq(1850, 2025,25)) +
    scale_y_continuous(n.breaks = 8) +geom_smooth(linewidth=1.1, se=FALSE, aes(color='Loess smooth')) +
    scale_color_manual(name=NULL, values=c('dodgerblue2','firebrick1')) +theme(legend.position = c(0.15, 0.875),legend.background=element_blank()) +
    labs(title='North Atlantic Hurricane Basin', subtitle='Accumulated Cyclone Energy (ACE)',
         y=expression('10'^4*' kn'^2 ), caption='Source: NOAA Hurricane Research Division, HURDAT2 re-analysis\nhttps://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html')

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}

#' Download and plot essential climate data
#'
#' Retrieves historical carbon dioxide and temperature records from the Vostok ice core during the past 420,000 years.
#' Source of data is the U.S. Department of Energy’s (DOE) Environmental System Science Data Infrastructure for a Virtual Ecosystem (ESS-DIVE).
#' \url{https://ess-dive.lbl.gov/} and \url{https://data.ess-dive.lbl.gov/datasets/doi:10.3334/CDIAC/CLI.006}
#'
#' @name get_paleo
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with the age of the ice (years before C.E.), carbon dioxide (ppm) and temperature (degrees C).
#'
#' `get_paleo` invisibly returns a tibble with Vostok ice core data: the age of the ice (years before C.E.), carbon dioxide (ppm) and temperature (degrees C).
#'
#' Data are from: Barnola J; Raynaud D; Lorius C; Barkov N  (2003): Historical Carbon Dioxide Record from the Vostok Ice Core (417,160 - 2,342 years BP) and
#' Petit J R ; Raynaud D ; Lorius C ; Delaygue G ; Jouzel J ; Barkov N I ; Kotlyakov V M  (2000): Historical Isotopic Temperature Record from the Vostok Ice Core. CDIAC.
#'
#' @importFrom readr read_table
#' @importFrom utils download.file
#' @importFrom dplyr full_join select
#' @importFrom tidyr pivot_longer
#'
#' @examples
#' \donttest{
#' # Fetch Vostok paleo carbon and temperature data from cache if available:
#' vostok <- get_paleo()
#' #
#' # Force cache refresh:
#' vostok <- get_paleo(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_paleo(vostok) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item Historical Carbon Dioxide Record from the Vostok Ice Core (US Dept of Energy): \url{https://data.ess-dive.lbl.gov/datasets/doi:10.3334/CDIAC/CLI.006}
#' \item Petit J R ; Raynaud D ; Lorius C ; Delaygue G ; Jouzel J ; Barkov N I ; Kotlyakov V M  (2000): Historical Isotopic Temperature Record from the Vostok Ice Core. CDIAC. doi:10.3334/CDIAC/CLI.006. \url{https://data.ess-dive.lbl.gov/view/doi:10.3334/CDIAC/CLI.006}
#' \item Barnola J ; Raynaud D ; Lorius C ; Barkov N  (2003): Historical Carbon Dioxide Record from the Vostok Ice Core (417,160 - 2,342 years BP). None. doi:10.3334/CDIAC/ATG.009 \url{https://data.ess-dive.lbl.gov/view/doi:10.3334/CDIAC/ATG.009}
#'  }
#'
#' @export


get_paleo <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

  hs_path <- tools::R_user_dir("hockeystick","cache")

  if (use_cache) {
    if (file.exists(file.path(hs_path,'paleo.rds'))) return(invisible(readRDS((file.path(hs_path,'paleo.rds')))))
  }

file_url <- 'https://data.ess-dive.lbl.gov/catalog/d1/mn/v2/object/ess-dive-457358fdc81d3a5-20180726T203952542'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

dl <- tempfile()
status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

vostok <- readr::read_table(dl, col_names = FALSE, skip = 21, show_col_types = FALSE)
colnames(vostok) <- c('depth', 'age_ice', 'age_air', 'co2')

file_url <- 'https://data.ess-dive.lbl.gov/catalog/d1/mn/v2/object/ess-dive-1e57f3f83864c10-20180717T104354142744'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

dl <- tempfile()
status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

paleotemp <- readr::read_table(dl, col_names = FALSE, skip = 60, show_col_types = FALSE)
colnames(paleotemp) <- c('depth', 'age_ice', 'deuterium', 'temp')

suppressMessages( paleo <- dplyr::full_join(vostok, paleotemp) )
paleo <- dplyr::select(paleo, age_ice, co2, temp)
paleo <- tidyr::pivot_longer(paleo, -age_ice, values_drop_na = TRUE)

if (write_cache) saveRDS(paleo, file.path(hs_path, 'paleo.rds'))

invisible(paleo) }

#' Download and plot essential climate data
#'
#' Plots the Vostok ice core data retrieved using `get_paleo()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_paleo
#' @param dataset Name of the tibble generated by `get_paleo`, defaults to calling `get_paleo`
#' @param print (boolean) Display Vostok ice core ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#'
#' @return Invisibly returns a ggplot2 object with the Vostok chart
#'
#' @details `plot_paleo` invisibly returns a ggplot2 object with a pre-defined Vostok ice core chart using data from `get_paleo`.
#' The returned chart stacks carbon dioxide concentration over temperature over 420,000 years.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom patchwork wrap_plots
#' @importFrom scales unit_format
#'
#' @examples
#' \donttest{
#' # Fetch Vostok data:
#' vostok <- get_paleo()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_paleo(vostok)
#'
#' # Or just call plot_paleo(), which defaults to get_paleo() dataset
#' plot_paleo()
#'
#' p <- plot_paleo(vostok, print = FALSE)
#' # Modify plot such as: p + patchwork::plot_annotation(title='A Long History of Carbon') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' Barnola, J.-M., D. Raynaud, C. Lorius, and N.I. Barkov. 2003. Historical CO2 record from the Vostok ice core. In Trends: A Compendium of Data on Global Change. Carbon Dioxide Information Analysis Center, Oak Ridge National Laboratory, U.S. Department of Energy, Oak Ridge, Tenn., U.S.A
#'
#' @export

plot_paleo <- function(dataset = get_paleo(), print=TRUE) {

if (is.null(dataset)) return(invisible(NULL))

a <- ggplot(dataset[dataset$name=='co2',], aes(x=age_ice, y=value)) +geom_line(linewidth=.8, col='firebrick1') +scale_x_reverse(lim=c(423000,0)) +
            theme_bw() + theme(axis.title.x=element_blank(), axis.text.x=element_blank()) +labs(y=expression(CO[2]*' concentration' ))

b <- ggplot(dataset[dataset$name=='temp',], aes(x=age_ice, y=value)) +geom_line(linewidth=.8, col='dodgerblue2') +scale_x_reverse(lim=c(423000, 0), labels = scales::unit_format(unit='', scale = 1e-3)) +
            labs(x='Millennia before present', y='Temperature (C\U00B0)') +theme_bw()

plot <- patchwork::wrap_plots(a, b, ncol=1) + patchwork::plot_annotation(title = expression('Paleoclimate: The Link Between '*CO[2]*' and Temperature'),
      caption = 'Source: U.S. Department of Energy ESS-DIVE\nhttps://data.ess-dive.lbl.gov/datasets/doi:10.3334/CDIAC/ATG.009\nhttps://data.ess-dive.lbl.gov/datasets/doi:10.3334/CDIAC/CLI.006',
      subtitle='420,000 years from the Vostok ice core, Antarctica.', theme = theme(  plot.title = element_text(size = 14)))

if (print) suppressMessages( print(plot) )
invisible(plot)
}

#' Download and plot essential climate data
#'
#' Retrieves globally averaged marine surface methane monthly mean data from National Oceanic and Atmospheric Administration.
#' The Global Monitoring Division of NOAA’s Earth System Research Laboratory has measured methane since 1983 at a globally distributed network of air sampling sites.
#' A global average is constructed by first smoothing the data for each site as a function of time, and then smoothed values for each site are plotted as a function of latitude.
#' Global means are calculated from the latitude plot at each time step.
#' \url{https://gml.noaa.gov/ccgg/trends_ch4/}
#' \url{https://gml.noaa.gov/ccgg/about/global_means.html}
#'
#' @name get_methane
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with the monthly methane series
#'
#' @details `get_methane` invisibly returns a tibble with NOAA's monthly globally averaged methane measurement.
#' The returned object includes year, month, date, average, average uncertainty, trend, and trend uncertainty columns.
#' Trend is NOAA's published trend.  Please refer to above website for details. CH4 expressed as a mole fraction in dry air, nanomol/mol, abbreviated as ppb.
#'
#' @importFrom readr read_table
#' @importFrom lubridate ymd ceiling_date
#' @importFrom utils download.file
#'
#' @examples
#' \donttest{
#' # Fetch from cache if available, otherwise download:
#' ch4 <- get_methane()
#' #
#' # Force fetch from source:
#' ch4 <- get_methane(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_methane(ch4) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' Lan, X., K.W. Thoning, and E.J. Dlugokencky: Trends in globally-averaged CH4, N2O, and SF6 determined from NOAA Global Monitoring Laboratory measurements. Version 2022-11, \doi{10.15138/P8XG-AA10}
#'
#' @export
get_methane <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

hs_path <- tools::R_user_dir("hockeystick","cache")

if (use_cache & !write_cache) {
  if (file.exists(file.path(hs_path,'methane.rds'))) return(invisible(readRDS((file.path(hs_path,'methane.rds')))))
  }

file_url <- 'https://gml.noaa.gov/webdata/ccgg/trends/ch4/ch4_mm_gl.txt'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

dl <- tempfile()
status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

ch4 <- suppressMessages( read_table(dl, col_names = FALSE, skip = 65) )
colnames(ch4) <- c('year', 'month', 'date', 'average', 'average_unc', 'trend','trend_unc')
ch4$date <- ceiling_date(ymd(paste(ch4$year, ch4$month, '01',sep='-')), unit='month')-1

dir.create(hs_path, showWarnings = FALSE, recursive = TRUE)
if (write_cache) saveRDS(ch4, file.path(hs_path, 'ch4.rds'))

invisible(ch4)
}



#' Download and plot essential climate data
#'
#' Plots atmospheric methane data retrieved using `get_methane()` with ggplot2. The output ggplot2 object may be modified.
#'
#'
#' @name plot_methane
#' @param dataset Name of the tibble generated by `get_methane`
#' @param print (boolean) Display methane ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param annot (boolean) Include chart annotation with latest date and value, defaults to TRUE.
#'
#' @return Invisibly returns a ggplot2 object with methane chart
#'
#' @details `plot_methane` invisibly returns a ggplot2 object with a pre-defined methane chart using data from `get_methane`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom dplyr pull slice
#'
#' @examples
#' \donttest{
#' # Fetch methane data:
#' ch4 <- get_methane()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_methane(ch4)
#'
#' # Or just call plot_methane(), which defaults to get_methane() dataset
#' plot_methane()
#'
#' p <- plot_methane(ch4, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Trend in Atmospheric Methane') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_methane <- function(dataset = get_methane(),
                        print = TRUE, annot = TRUE) {

if (is.null(dataset)) return(invisible(NULL))

plot <- ggplot(dataset, aes(x=date, y=average)) +geom_line(color='dodgerblue2', alpha=0.7) + theme_bw(base_size=12) +
    scale_x_date(name=NULL, date_breaks='10 years', limits=c(ymd('1983-01-01'), ymd(paste0(max(dataset$year)+1,'-01-01'))), date_labels='%Y', minor_breaks = NULL) +
    scale_y_continuous(limits=c(1600, round(max(dataset$average),-3)), breaks=seq(1600, round(max(dataset$average),-3), 50)) +
    geom_line(aes(y=trend), linewidth=1, col='firebrick1') +
    labs(title=expression('Atmospheric Methane'), subtitle=expression('Globally Averaged Atmospheric '*CH[4]*' monthly mean ppb'),
    y=expression(CH[4]*' mole fraction (ppb)' ), caption='Source: NOAA/ESRL.\nhttps://gml.noaa.gov/ccgg/trends_ch4/')

if (annot) {

dt <- pull(slice(dataset, which.max(date)), date)
vl <- pull(slice(dataset, which.max(date)), average)

plot <- plot + annotate("text", x=as.Date('1989-01-01'), y=1970, label=paste(dt, vl,sep=": "), color='red')
}

if (print) print(plot)
invisible(plot)
}

#' Download and plot essential climate data
#'
#' Retrieves atmospheric carbon dioxide measurements from National Oceanic and Atmospheric Administration
#' Earth System Research Laboratories monitoring laboratory in Mauna Loa, Hawaii.
#' \url{https://gml.noaa.gov/ccgg/trends/data.html}
#'
#' @name get_carbon
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with the monthly carbon dioxide series
#'
#' @details `get_carbon` invisibly returns a tibble with NOAA's monthly average carbon dioxide measurement.
#' The returned object includes date, year, month, average, trend, std dev, and uncertainty columns.
#' Trend is NOAA's published trend.  Please refer to above website for details.
#'
#' @importFrom readr read_table
#' @importFrom lubridate ymd ceiling_date
#' @importFrom utils download.file
#'
#' @examples
#' \donttest{
#' # Fetch from cache if available, otherwise download:
#' maunaloa <- get_carbon()
#' #
#' # Force fetch from source:
#' maunaloa <- get_carbon(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_carbon(maunaloa) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' Dr. Pieter Tans, NOAA/GML \url{https://gml.noaa.gov/ccgg/trends/} and Dr. Ralph Keeling, Scripps Institution of Oceanography \url{https://scrippsco2.ucsd.edu/}.
#'
#'
#' C.D. Keeling, R.B. Bacastow, A.E. Bainbridge, C.A. Ekdahl, P.R. Guenther, and L.S. Waterman, (1976), Atmospheric carbon dioxide variations at Mauna Loa Observatory, Hawaii, \emph{Tellus}, vol. 28, 538-551
#' @export
get_carbon <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

hs_path <- tools::R_user_dir("hockeystick","cache")

if (use_cache & !write_cache) {
  if (file.exists(file.path(hs_path,'maunaloa.rds'))) return(invisible(readRDS((file.path(hs_path,'maunaloa.rds')))))
  }

file_url <- 'https://gml.noaa.gov/webdata/ccgg/trends/co2/co2_mm_mlo.csv'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

dl <- tempfile()
status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

maunaloa <- suppressMessages( read_csv(dl, col_names = FALSE, skip = 41) )
colnames(maunaloa) <- c('year', 'month', 'date', 'average', 'trend', 'ndays','stdev','unc')
maunaloa$date <- ceiling_date(ymd(paste(maunaloa$year, maunaloa$month, '01',sep='-')), unit='month')-1

dir.create(hs_path, showWarnings = FALSE, recursive = TRUE)
if (write_cache) saveRDS(maunaloa, file.path(hs_path, 'maunaloa.rds'))

invisible(maunaloa)
}



#' Download and plot essential climate data
#'
#' Plots carbon dioxide data retrieved using `get_carbon()` with ggplot2. The output ggplot2 object may be modified.
#'
#'
#' @name plot_carbon
#' @param dataset Name of the tibble generated by `get_carbon`
#' @param print (boolean) Display carbon dioxide ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param annot (boolean) Include chart annotation with latest date and value, defaults to TRUE.
#'
#' @return Invisibly returns a ggplot2 object with carbon dioxide chart
#'
#' @details `plot_carbon` invisibly returns a ggplot2 object with a pre-defined carbon dioxide chart using data from `get_carbon`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom dplyr pull slice
#'
#' @examples
#' \donttest{
#' # Fetch carbon dioxide data:
#' maunaloa <- get_carbon()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_carbon(maunaloa)
#'
#' # Or just call plot_carbon(), which defaults to get_carbon() dataset
#' plot_carbon()
#'
#' p <- plot_carbon(maunaloa, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='The Keeling Curve') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_carbon <- function(dataset = get_carbon(),
                        print = TRUE, annot = TRUE) {

if (is.null(dataset)) return(invisible(NULL))

plot <- ggplot(dataset, aes(x=date, y=average)) +geom_line(color='dodgerblue2', alpha=0.7) + theme_bw(base_size=12) +
    scale_x_date(name=NULL, date_breaks='10 years', limits=c(ymd('1954-01-01'), ymd(paste0(max(dataset$year)+1,'-01-01'))), date_labels='%Y', minor_breaks = NULL) +
    scale_y_continuous(limits=c(300, 25+round(max(dataset$average),-1)), breaks=seq(300, round(max(dataset$average)+25,-1), 25)) +
    geom_line(aes(y=trend), linewidth=1, col='firebrick1') +
    labs(title=expression('Atmospheric '*CO[2]*' (Keeling Curve)'), subtitle=expression('Mauna Loa '*CO[2]*' monthly mean ppm'),
    y=expression(CO[2]*' concentration in air' ), caption='Source: NOAA/ESRL and Scripps Institution of Oceanography.\nhttps://gml.noaa.gov/ccgg/trends/data.html')

if (annot) {

dt <- pull(slice(dataset, which.max(date)), date)
vl <- pull(slice(dataset, which.max(date)), average)

plot <- plot + annotate("text",x=as.Date('1968-01-01'), y=440, label=paste(dt, vl,sep=": "), color='red')
}

if (print) print(plot)
invisible(plot)
}

# Variables used in ggplot2 aes() mappings over package data frames
if (getRversion() >= "2.15.1") utils::globalVariables(
  c('lon', 'lat', 'value', 'delta', 'x', 'y', 'part', 'hole'))

#' Download and plot essential climate data
#'
#' Retrieves downscaled CMIP6 climate projection data from WorldClim
#' (\url{https://www.worldclim.org/data/cmip6/}). Data are monthly or bioclimatic
#' variables projected by a selected global climate model (GCM) under an SSP
#' scenario for one of four 20-year periods (2021-2100), at 10 arc-minute resolution.
#'
#' @name get_cmip6
#' @param var (string) Climate variable. One of \code{"bioc"} (bioclimatic variables,
#'   the default), \code{"tmin"}, \code{"tmax"}, or \code{"prec"} (precipitation).
#' @param bio (numeric) If \code{var = "bioc"}, which bioclimatic variable to return
#'   (1-19). Defaults to 1 (Annual Mean Temperature). The variables are:
#'   1 = Annual Mean Temperature; 2 = Mean Diurnal Range; 3 = Isothermality (BIO2/BIO7);
#'   4 = Temperature Seasonality (standard deviation); 5 = Max Temperature of Warmest Month;
#'   6 = Min Temperature of Coldest Month; 7 = Temperature Annual Range (BIO5-BIO6);
#'   8 = Mean Temperature of Wettest Quarter; 9 = Mean Temperature of Driest Quarter;
#'   10 = Mean Temperature of Warmest Quarter; 11 = Mean Temperature of Coldest Quarter;
#'   12 = Annual Precipitation; 13 = Precipitation of Wettest Month; 14 = Precipitation of Driest Month;
#'   15 = Precipitation Seasonality (coefficient of variation); 16 = Precipitation of Wettest Quarter;
#'   17 = Precipitation of Driest Quarter; 18 = Precipitation of Warmest Quarter;
#'   19 = Precipitation of Coldest Quarter. Variables 1-11 are in degrees C
#'   (seasonality variables 4 and 7 as applicable), 12-19 in mm. For the monthly
#'   variables (\code{var = "prec"}, \code{"tmin"}, or \code{"tmax"}), \code{bio}
#'   instead selects the calendar month, 1-12 (1 = January).
#' @param ssp (string) Shared Socioeconomic Pathway scenario. One of "126", "245",
#'   "370", or "585" (the default). SSP5-8.5 is the high-emissions upper bound and
#'   shows the strongest projected change; SSP2-4.5 is a middle-of-the-road
#'   scenario roughly consistent with current policies ("current trends continue").
#'   SSP1-2.6 assumes strong mitigation, and SSP3-7.0 is a high fragmented-world path.
#' @param period (string) Projection period. One of "2021-2040", "2041-2060",
#'   "2061-2080", or "2081-2100" (the default).
#' @param model (string) CMIP6 global climate model. Defaults to "MPI-ESM1-2-HR".
#'   Availability of variable/scenario combinations varies by model; the function
#'   checks the WorldClim index and reports what is available.
#' @param res (string) Resolution: "10m" (~18 km, default), "5m" (~9 km) or "2.5m"
#'   (~5 km). Finer resolutions are considerably larger downloads.
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE.
#'
#' @return Invisibly returns a tibble with columns \code{lon}, \code{lat}, and
#'   \code{value}: the projected values aggregated to a one-degree grid.
#'
#' @details `get_cmip6` downloads a WorldClim CMIP6 GeoTIFF, aggregates it to a
#'   one-degree grid (averaging), and returns a tibble suitable for mapping.
#'   Requires the `terra` package (installed automatically when needed).
#'
#' @examples
#' \donttest{
#' # Projected annual mean temperature under SSP585, 2081-2100:
#' proj <- get_cmip6(var='bioc', bio=1)
#' plot_cmip6(proj)
#' #
#' # Projected precipitation change under a moderate scenario:
#' prec <- get_cmip6(var='prec', ssp=245, period='2041-2060', use_cache=FALSE)
#' }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item WorldClim CMIP6 future climate data: \url{https://www.worldclim.org/data/cmip6/}
#' \item Fick, S.E. & Hijmans, R.J. (2017) WorldClim 2: new 1km spatial resolution climate
#'  surfaces for global land areas. \emph{Int. J. Climatol.} 37: 4302-4315.
#' }
#'
#' @importFrom tibble as_tibble
#' @importFrom utils read.table download.file
#'
#' @export

get_cmip6 <- function(var = 'bioc', bio = 1,
                      ssp = '585', period = '2081-2100',
                      model = 'MPI-ESM1-2-HR', res = '10m',
                      use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

  var <- match.arg(var, c('bioc', 'tmin', 'tmax', 'prec'))
  stopifnot(ssp %in% c('126', '245', '370', '585'))
  stopifnot(period %in% c('2021-2040', '2041-2060', '2061-2080', '2081-2100'))
  stopifnot(res %in% c('2.5m', '5m', '10m'))
  if (var == 'bioc') stopifnot(bio %in% 1:19)

  hs_path <- tools::R_user_dir("hockeystick", "cache")
  cachename <- paste0('cmip6_', var, if (var=='bioc') bio else '', '_',
                      model, '_ssp', ssp, '_', period, '.rds')

  if (use_cache && file.exists(file.path(hs_path, cachename)))
    return(invisible(readRDS(file.path(hs_path, cachename))))

  if (!requireNamespace("terra", quietly = TRUE))
    stop("get_cmip6 requires the 'terra' package. Install with install.packages('terra')")

  connected <- .isConnected('https://geodata.ucdavis.edu/cmip6/')
  if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

  fname <- paste0('wc2.1_', res, '_', var, '_', model, '_ssp', ssp, '_', period, '.tif')
  url <- paste0('https://geodata.ucdavis.edu/cmip6/', res, '/', model, '/ssp', ssp, '/', fname)

  # Verify combination exists using WorldClim's availability index
  idx <- tempfile()
  status <- tryCatch(suppressWarnings(download.file('https://geodata.ucdavis.edu/cmip6/files.txt',
                                                    idx, quiet = TRUE)), error = function(e) 1L)
  avail <- NULL
  if (identical(status, 0L)) {
    ff <- read.table(idx, sep = '_', stringsAsFactors = FALSE)
    avail <- unique(ff[[2]][ff[[1]] == var & ff[[3]] == paste0('ssp', ssp) & ff[[4]] == paste0(period, '.tif')])
  }
  if (!is.null(avail) && !(model %in% avail)) {
    message("Combination not available on WorldClim. Models with ", var,
            "/ssp", ssp, "/", period, ":")
    message(paste(sort(avail), collapse=', '))
    return(invisible(NULL))
  }

  dl <- tempfile(fileext = '.tif')
  status <- tryCatch(suppressWarnings(download.file(url, dl, mode = 'wb')), error = function(e) 1L)
  if (!identical(status, 0L)) {message("Unable to access remote resource."); return(invisible(NULL))}

  r <- terra::rast(dl)
  if (terra::nlyr(r) > 1) {
    layer_idx <- if (var == 'bioc') bio else bio # monthly vars: layers are months 1-12
    layer_name <- if (var == 'bioc') paste0('Bio', bio) else paste0('month', bio)
    r <- r[[layer_idx]]
  } else layer_name <- var

  # Aggregate 10 arc-min cells (~18km) up to ~1 degree to keep output manageable
  # Aggregate by a constant factor of 6 so output resolution scales with
  # the source: 1 degree at 10m, 0.5 at 5m, 0.25 at 2.5m
  agg <- terra::aggregate(r, fact = 6, fun = mean, na.rm = TRUE)
  df <- terra::as.data.frame(agg, xy = TRUE, na.rm = TRUE)
  colnames(df)[1:3] <- c('lon', 'lat', 'value')

  out <- as_tibble(df)
  attr(out, "hs_cmip6_meta") <- c(var = layer_name, var0 = var, model = model,
                                  ssp = paste0('SSP', ssp), period = period)

  if (write_cache) saveRDS(out, file.path(hs_path, cachename))
  invisible(out)
}


# Fetch and aggregate the matching WorldClim v2.1 baseline (1970-2000),
# returning a one-degree tibble identical in structure to get_cmip6 output
.get_cmip6_baseline <- function(var, bio, res = '10m') {

  if (!requireNamespace("terra", quietly = TRUE))
    stop("get_cmip6 requires the 'terra' package. Install with install.packages('terra')")

  hs_path <- tools::R_user_dir("hockeystick", "cache")
  dir.create(hs_path, showWarnings = FALSE, recursive = TRUE)

  stem <- switch(var, bioc = 'bio', var)
  zip_name <- paste0('wc2.1_', res, '_', stem, '.zip')
  zip_path <- file.path(hs_path, zip_name)

  layer_file <- if (var == 'bioc') {
    paste0('wc2.1_', res, '_bio_', bio, '.tif')
  } else {
    paste0('wc2.1_', res, '_', var, '_', sprintf('%02d', bio), '.tif')
  }
  layer_path <- file.path(hs_path, layer_file)

  if (!file.exists(layer_path)) {
    status <- tryCatch(suppressWarnings(download.file(
      paste0('https://geodata.ucdavis.edu/climate/worldclim/2_1/base/', zip_name),
      zip_path, mode = 'wb')), error = function(e) 1L)
    if (!identical(status, 0L)) {message("Unable to access remote resource."); return(NULL)}
    try(utils::unzip(zip_path, files = layer_file, exdir = hs_path), silent = TRUE)
    try(file.remove(zip_path), silent = TRUE)
    if (!file.exists(layer_path)) {message('Unexpected baseline archive contents.'); return(NULL)}
  }

  agg <- terra::aggregate(terra::rast(layer_path), fact = 6, fun = mean, na.rm = TRUE)
  df <- terra::as.data.frame(agg, xy = TRUE, na.rm = TRUE)
  colnames(df)[1:3] <- c('lon', 'lat', 'value')
  as_tibble(df)
}

#' Download and plot essential climate data
#'
#' Computes expected climate change as the difference between a CMIP6 projection
#' retrieved from WorldClim and its 1970-2000 reference-period baseline, on a
#' common one-degree grid.
#'
#' @name get_cmip6_anom
#' @param var,bio,ssp,period,model,res Passed through to \code{\link{get_cmip6}}.
#' @param use_cache (boolean) Return cached anomaly data if available, defaults to TRUE.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE.
#'
#' @return Invisibly returns a tibble with columns \code{lon}, \code{lat}, and
#'   \code{delta}: the projected change versus the 1970-2000 baseline for each
#'   cell. For temperature variables \code{delta} is the absolute change in
#'   degrees C; for precipitation variables (var "prec", or bioc 12-19) it is
#'   the percent change, capped at +/-100\% to limit instability where the
#'   baseline is near zero.
#'
#' @details `get_cmip6_anom` downloads the projection and the corresponding
#'   WorldClim v2.1 baseline, aggregates both to a one-degree grid, and subtracts
#'   cell by cell. Differencing removes most of the model's systematic bias,
#'   leaving the expected change. Polar regions can show spurious values where
#'   the downscaled grids differ in land-sea masking.
#'
#' @examples
#' \donttest{
#' anom <- get_cmip6_anom()
#' plot_cmip6_anom(anom)
#' }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references See \code{\link{get_cmip6}}.
#'
#' @importFrom tibble as_tibble
#' @export

get_cmip6_anom <- function(var = 'bioc', bio = 1,
                           ssp = '585', period = '2081-2100',
                           model = 'MPI-ESM1-2-HR', res = '10m',
                           use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

  cachename <- paste0('cmip6anom_', var, if (var == 'bioc') bio else '', '_',
                      model, '_ssp', ssp, '_', period, '.rds')
  hs_path <- tools::R_user_dir("hockeystick", "cache")

  if (use_cache && file.exists(file.path(hs_path, cachename)))
    return(invisible(readRDS(file.path(hs_path, cachename))))

  fut <- get_cmip6(var = var, bio = bio, ssp = ssp, period = period,
                   model = model, res = res, use_cache = use_cache, write_cache = TRUE)
  if (is.null(fut)) return(invisible(NULL))

  base <- .get_cmip6_baseline(var, bio, res = res)
  if (is.null(base)) return(invisible(NULL))

  m <- merge(fut, base, by = c('lon', 'lat'), suffixes = c('_fut', '_base'))
  out <- as_tibble(m[c('lon', 'lat', 'value_fut', 'value_base')])

  # Temperature: absolute change (deg C). Precipitation: percent change,
  # which is scale-independent across wet and dry regions, capped at +/-100%
  # so cells with near-zero baselines (hyper-arid) don't produce extreme values
  is_prec <- var == 'prec' || (var == 'bioc' && bio >= 12)
  out$delta <- if (is_prec) pmin(pmax(100 * (out$value_fut - out$value_base) / out$value_base, -100), 100)
               else out$value_fut - out$value_base
  out$delta[!is.finite(out$delta)] <- NA

  layer_name <- if (var == 'bioc') paste0('Bio', bio) else paste0('month', bio)
  attr(out, "hs_cmip6_meta") <- c(var = layer_name, var0 = var, model = model,
                                  ssp = paste0('SSP', ssp), period = period)

  if (write_cache) saveRDS(out, file.path(hs_path, cachename))
  invisible(out)
}


# Restrict a cmip6 tibble to a bounding box and/or country/state. Returns the
# cropped tibble, an optional border data frame for drawing, and a region label.
.crop_region <- function(dataset, xlim = NULL, ylim = NULL,
                         country = NULL, state = NULL, mask = TRUE) {

  border <- NULL
  region_lab <- NULL

  if (!is.null(country)) {
    if (!requireNamespace("geodata", quietly = TRUE))
      stop("Plotting by country requires the 'geodata' package. Install with install.packages('geodata')")
    if (!requireNamespace("terra", quietly = TRUE))
      stop("Plotting by country requires the 'terra' package. Install with install.packages('terra')")

    if (!is.null(state)) {
      # Admin-1 (state/province) boundaries from GADM; requires the country
      cntry_all <- geodata::gadm(country = country, level = 1,
                                 path = tools::R_user_dir("hockeystick", "cache"))
      sel <- tolower(cntry_all$NAME_1) == tolower(state)
      if (!any(sel)) {
        message("State/province not recognized: ", state, " (in ", country, "). Available:")
        message(paste(sort(unique(cntry_all$NAME_1)), collapse = ', '))
        return(NULL)
      }
      cntry <- cntry_all[which(sel), ]
      region_lab <- cntry$NAME_1[1]
    } else {
      cntry_all <- geodata::world(level = 0,
                                  path = tools::R_user_dir("hockeystick", "cache"))
      iso <- toupper(country)
      sel <- cntry_all$GID_0 == iso |
        tolower(cntry_all$NAME_0) == tolower(country)
      if (!any(sel)) {
        message("Country not recognized: ", country)
        return(NULL)
      }
      cntry <- cntry_all[which(sel), ]
      region_lab <- cntry$NAME_0[1]
    }

    e <- terra::ext(cntry)
    if (is.null(xlim)) xlim <- c(e$xmin, e$xmax)
    if (is.null(ylim)) ylim <- c(e$ymin, e$ymax)

    if (mask) {
      pts <- terra::vect(dataset[c('lon', 'lat')], crs = 'lonlat')
      inside <- terra::relate(pts, cntry, 'within')[, 1]
      dataset <- dataset[inside, ]
    }

    border <- as.data.frame(terra::geom(cntry))[c('x', 'y', 'hole', 'part')]
  }

  if (!is.null(xlim)) dataset <- dataset[dataset$lon >= xlim[1] & dataset$lon <= xlim[2], ]
  if (!is.null(ylim)) dataset <- dataset[dataset$lat >= ylim[1] & dataset$lat <= ylim[2], ]

  list(dataset = dataset, border = border, region_lab = region_lab,
       xlim = xlim, ylim = ylim)
}

#' Download and plot essential climate data
#'
#' Plots gridded CMIP6 climate projections retrieved using `get_cmip6()` with ggplot2.
#'
#' @name plot_cmip6
#' @param dataset Name of the tibble generated by `get_cmip6`
#' @param palette (string) Color palette name passed to RColorBrewer. Defaults to
#'   "Spectral" reversed for temperature-type variables.
#' @param xlim,ylim (numeric) Optional two-element longitude/latitude bounds to
#'   zoom the map to a region, e.g. \code{xlim = c(-130, -60), ylim = c(20, 70)}.
#' @param country (string) Optional country name or ISO 3166-1 alpha-3 code to
#'   zoom the map to that country. Requires the `geodata` package.
#' @param state (string) Optional state/province name (admin-1), used together
#'   with \code{country} to zoom to a subnational region, e.g.
#'   \code{country = "MEX", state = "Sonora"}. Requires the `geodata` package.
#' @param mask (boolean) If \code{country} is given, blank cells outside the
#'   country border, defaults to TRUE.
#' @param print (boolean) Display map, defaults to TRUE.
#'
#' @return Invisibly returns a ggplot2 object with the projection map.
#'
#' @details `plot_cmip6` returns a pre-defined ggplot2 world map of projected
#'   values. Regions may be selected by coordinates (\code{xlim}/\code{ylim}),
#'   by country, or by state/province. Users may further modify the output chart.
#'
#' @examples
#' \donttest{
#' proj <- get_cmip6()
#' plot_cmip6(proj)
#' #
#' # Zoom to the contiguous United States. Explicit xlim/ylim override the
#' # country's bounding box, excluding Alaska and Hawaii from the view:
#' plot_cmip6(proj, country = 'USA', xlim = c(-125, -66), ylim = c(24, 50))
#' #
#' # Or zoom to a single state:
#' plot_cmip6(proj, country = 'MEX', state = 'Sonora') }
#'
#' @import ggplot2
#'
#' @export

plot_cmip6 <- function(dataset = get_cmip6(), palette = 'Spectral',
                       xlim = NULL, ylim = NULL, country = NULL, state = NULL,
                       mask = TRUE, print = TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  reg <- .crop_region(dataset, xlim, ylim, country, state, mask)
  dataset <- reg$dataset
  if (nrow(dataset) == 0) {message('No data cells in the selected region.'); return(invisible(NULL))}

  meta <- attr(dataset, "hs_cmip6_meta")

  # Human-readable variable names and units for WorldClim codes
  bio_names <- c('Annual Mean Temperature', 'Mean Diurnal Range', 'Isothermality',
                 'Temperature Seasonality', 'Max Temperature of Warmest Month',
                 'Min Temperature of Coldest Month', 'Temperature Annual Range',
                 'Mean Temperature of Wettest Quarter', 'Mean Temperature of Driest Quarter',
                 'Mean Temperature of Warmest Quarter', 'Mean Temperature of Coldest Quarter',
                 'Annual Precipitation', 'Precipitation of Wettest Month',
                 'Precipitation of Driest Month', 'Precipitation Seasonality',
                 'Precipitation of Wettest Quarter', 'Precipitation of Driest Quarter',
                 'Precipitation of Warmest Quarter', 'Precipitation of Coldest Quarter')

  v <- meta[['var']]
  if (grepl('^Bio[0-9]+$', v)) {
    bio_n <- as.integer(sub('Bio', '', v))
    var_lab <- bio_names[bio_n]
    fill_lab <- if (bio_n %in% c(3,4,15)) 'Percent' else
                if (bio_n >= 12) expression("Precip (mm)") else
                if (bio_n %in% c(2,7)) expression("Range (" * degree * "C)") else
                expression("Temp (" * degree * "C)")
  } else if (grepl('^month[0-9]+$', v)) {
    month_n <- as.integer(sub('month', '', v))
    var_lab <- paste(month.name[month_n], if (meta[['var0']] == 'prec') 'Precipitation' else 'Temperature')
    fill_lab <- if (meta[['var0']] == 'prec') expression("Precip (mm)") else expression("Temp (" * degree * "C)")
  } else {
    var_lab <- switch(v, prec = 'Precipitation', tmin = 'Minimum Temperature', tmax = 'Maximum Temperature', v)
    fill_lab <- if (v == 'prec') expression("Precip (mm)") else expression("Temp (" * degree * "C)")
  }

  title_lab <- paste0(var_lab, ', ', meta[['period']])
  if (!is.null(reg$region_lab)) title_lab <- paste(title_lab, '-', reg$region_lab)
  n_colors <- min(9, length(unique(dataset$value)))

  lon_breaks <- if (is.null(reg$xlim) || diff(reg$xlim) > 180) seq(-180, 180, 60) else seq(-180, 180, 20)
  plot <- ggplot(dataset, aes(x = lon, y = lat, fill = value)) +
    geom_tile(width = 1, height = 1) +
    scale_fill_gradientn(colors = rev(RColorBrewer::brewer.pal(n_colors, palette)),
                         na.value = 'transparent', n.breaks = 8) +
    scale_x_continuous(name = NULL, breaks = lon_breaks) +
    scale_y_continuous(name = NULL, breaks = seq(-60, 80, 20)) +
    coord_fixed(ratio = 1, expand = FALSE, xlim = reg$xlim, ylim = reg$ylim) +
    theme_bw(base_size = 12) +
    theme(panel.grid = element_blank()) +
    labs(title = title_lab,
         subtitle = 'Downscaled CMIP6 projection, WorldClim v2.1',
         fill = fill_lab,
         caption = paste0(meta[['model']], ' ', meta[['ssp']],
                          ' | Source: WorldClim.org'))

  if (!is.null(reg$border))
    plot <- plot + geom_polygon(data = reg$border, aes(x = x, y = y, group = part, fill = NULL),
                                fill = NA, color = 'grey30', linewidth = 0.4)

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}


#' Download and plot essential climate data
#'
#' Plots expected climate change (projection minus 1970-2000 baseline) computed by
#' `get_cmip6_anom()` with a diverging color scale centered on zero.
#'
#' @name plot_cmip6_anom
#' @param dataset Name of the tibble generated by `get_cmip6_anom`
#' @param zero_centered (boolean) Use a symmetric diverging scale around zero,
#'   defaults to FALSE (scale runs from zero up).
#' @param xlim,ylim (numeric) Optional two-element longitude/latitude bounds to
#'   zoom the map to a region, e.g. \code{xlim = c(-130, -60), ylim = c(20, 70)}.
#'   The color scale is recomputed on the visible region.
#' @param country (string) Optional country name or ISO 3166-1 alpha-3 code to
#'   zoom the map to that country. Requires the `geodata` package.
#' @param state (string) Optional state/province name (admin-1), used together
#'   with \code{country} to zoom to a subnational region, e.g.
#'   \code{country = "MEX", state = "Sonora"}. Requires the `geodata` package.
#' @param mask (boolean) If \code{country} is given, blank cells outside the
#'   country border, defaults to TRUE.
#' @param print (boolean) Display map, defaults to TRUE.
#'
#' @return Invisibly returns a ggplot2 object with the change map.
#'
#' @details `plot_cmip6_anom` returns a pre-defined ggplot2 world map of expected
#'   change in degrees C (or percent for precipitation variables), with blue for
#'   cooling/drying and red for warming/wetting. Precipitation maps with both
#'   increases and decreases automatically use the diverging scale even when
#'   \code{zero_centered = FALSE}. Users may further modify the output chart.
#'
#' @import ggplot2
#' @importFrom scales label_number
#'
#' @examples
#' \donttest{
#' plot_cmip6_anom()
#' }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references See \code{\link{get_cmip6}}.
#'
#' @export

plot_cmip6_anom <- function(dataset = get_cmip6_anom(), zero_centered = FALSE,
                            xlim = NULL, ylim = NULL, country = NULL, state = NULL,
                            mask = TRUE, print = TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  reg <- .crop_region(dataset, xlim, ylim, country, state, mask)
  dataset <- reg$dataset
  if (nrow(dataset) == 0) {message('No data cells in the selected region.'); return(invisible(NULL))}

  meta <- attr(dataset, "hs_cmip6_meta")
  var_lab <- switch(meta[['var0']],
                    prec = 'Precipitation',
                    tmin = 'Minimum Temperature',
                    tmax = 'Maximum Temperature',
                    if (grepl('^Bio[0-9]+$', meta[['var']]))
                      c('Annual Mean Temperature', 'Mean Diurnal Range', 'Isothermality',
                        'Temperature Seasonality', 'Max Temperature of Warmest Month',
                        'Min Temperature of Coldest Month', 'Temperature Annual Range',
                        'Mean Temperature of Wettest Quarter', 'Mean Temperature of Driest Quarter',
                        'Mean Temperature of Warmest Quarter', 'Mean Temperature of Coldest Quarter',
                        'Annual Precipitation', 'Precipitation of Wettest Month',
                        'Precipitation of Driest Month', 'Precipitation Seasonality',
                        'Precipitation of Wettest Quarter', 'Precipitation of Driest Quarter',
                        'Precipitation of Warmest Quarter', 'Precipitation of Coldest Quarter')[
                          as.integer(sub('Bio', '', meta[['var']]))] else meta[['var']])

  is_prec <- meta[['var0']] == 'prec' ||
    (grepl('^Bio[0-9]+$', meta[['var']]) && as.integer(sub('Bio', '', meta[['var']])) >= 12)

  # Cap the scale at the 99th percentile so extreme outlier cells
  # don't compress the color range; outliers are squashed to end colors.
  # Precipitation (percent) changes can be negative: fall back to the
  # diverging scale so drying regions aren't misrepresented
  hi <- unname(stats::quantile(abs(dataset$delta), 0.99, na.rm = TRUE))
  has_negs <- any(dataset$delta < 0, na.rm = TRUE)
  use_diverging <- zero_centered || (is_prec && has_negs)
  if (use_diverging) {
    # Precipitation convention: blue = wetter, red = drier (opposite of temperature)
    low_col  <- if (is_prec) '#b2182b' else '#2166ac'
    high_col <- if (is_prec) '#2166ac' else '#b2182b'
    fill_scale <- scale_fill_gradient2(low = low_col, mid = 'white', high = high_col,
                                       midpoint = 0, limits = c(-hi, hi), n.breaks = 8,
                                       na.value = 'transparent', oob = scales::squish,
                                       labels = if (is_prec) scales::label_percent(scale = 1, accuracy = 1)
                                                else scales::label_number(accuracy = 1))
  } else {
    fill_scale <- scale_fill_gradientn(colors = RColorBrewer::brewer.pal(9, 'YlOrRd'),
                                       limits = c(0, hi), n.breaks = 8,
                                       na.value = 'transparent', oob = scales::squish,
                                       labels = if (is_prec) scales::label_percent(scale = 1, accuracy = 1)
                                                else scales::label_number(accuracy = 1))
  }

  lon_breaks <- if (is.null(reg$xlim) || diff(reg$xlim) > 180) seq(-180, 180, 60) else seq(-180, 180, 20)
  plot <- ggplot(dataset, aes(x = lon, y = lat, fill = delta)) +
    geom_tile(width = 1, height = 1) +
    fill_scale +
    scale_x_continuous(name = NULL, breaks = lon_breaks) +
    scale_y_continuous(name = NULL, breaks = seq(-60, 80, 20)) +
    coord_fixed(ratio = 1, expand = FALSE, xlim = reg$xlim, ylim = reg$ylim) +
    theme_bw(base_size = 12) +
    theme(panel.grid = element_blank()) +
    labs(title = paste('Expected Change in', var_lab, 'by', meta[['period']],
                       if (!is.null(reg$region_lab)) paste('-', reg$region_lab)),
         subtitle = expression("Versus 1970-2000 baseline, WorldClim CMIP6 downscaled"),
         fill = if (is_prec) expression(Delta * " (%)") else expression(Delta * "T (" * degree * "C)"),
         caption = paste0(meta[['model']], ' ', meta[['ssp']],
                          ' | Source: WorldClim.org'))

  if (!is.null(reg$border))
    plot <- plot + geom_polygon(data = reg$border, aes(x = x, y = y, group = part, fill = NULL),
                                fill = NA, color = 'grey30', linewidth = 0.4)

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}

#' Download and plot essential climate data
#'
#' Retrieves global mean sea level (GMSL) data from historic tide gauge and recent satellite altimeter observations (in mm).
#' Source for tide gauge data is Commonwealth Scientific and Industrial Research Organisation (CSIRO) as described in Church and White (2011).
#' \url{https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/} \cr \cr
#' Source for satellite data is NOAA Laboratory for Satellite Altimetry:
#' \url{https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/}
#'
#' @name get_sealevel
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with the global mean sea level time series (in mm) over 1880-2013 using tide gauges and since 1993 for satellite measurements.
#'
#' `get_sealevel` invisibly returns a tibble with mean sea level in mm time series from tide gauges and satellite observations.
#'
#' The satellite observations have been releveled so that their mean level in 1993 matches that of the tide gauges.
#'
#' The tide gauge data are no longer updated and cover the period from 1880 to 2013, per Church, J. A. and N.J. White (2011) \url{https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/}
#'
#' Satellite data are updated monthly or more frequently from the NOAA Laboratory for Satellite Altimetry. TOPEX and Jason-1,-2,-3 satellites dataset, with seasonal signals removed. \url{https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/}
#'
#' @importFrom readr read_csv
#' @importFrom lubridate ymd date_decimal round_date
#' @importFrom utils download.file read.csv unzip
#' @importFrom dplyr full_join summarize_all filter select
#' @importFrom tidyr pivot_longer
#'
#' @examples
#' \donttest{
#' # Fetch sea level from cache if available:
#' gmsl <- get_sealevel()
#' #
#' # Force cache refresh:
#' gmsl <- get_sealevel(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_sealevel(gmsl) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item CSIRO reconstructed tide gauge GMSL for 1880 to 2009: \url{https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/}
#' \item Church, J. A. and N.J. White (2011), Sea-level rise from the late 19th to the early 21st Century. \emph{Surveys in Geophysics}, doi:10.1007/s10712-011-9119-1. \url{https://link.springer.com/article/10.1007/s10712-011-9119-1}
#' \item NOAA Laboratory for Satellite Altimetry \url{https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/}
#'  }
#'
#' @export

get_sealevel <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

  hs_path <- tools::R_user_dir("hockeystick","cache")

  if (use_cache) {
    if (file.exists(file.path(hs_path,'sealevel.rds'))) return(invisible(readRDS((file.path(hs_path,'sealevel.rds')))))
  }

file_url <- 'https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/slr/slr_sla_gbl_free_ref_90.csv'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

dl <- tempfile()

status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

gmsl_sat <- utils::read.csv(dl, header = FALSE, skip = 6)
gmsl_sat$gmsl_sat <- rowMeans(gmsl_sat[,2:5], na.rm = TRUE)
colnames(gmsl_sat)[1] <- 'date'
gmsl_sat <- gmsl_sat[,c('date', 'gmsl_sat')]
gmsl_sat$date <- lubridate::ymd(lubridate::round_date(lubridate::date_decimal(gmsl_sat$date), 'day'))


file_url <- 'https://research.csiro.au/slrwavescoast/?ddownload=327'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

td <- tempdir()
dl <- tempfile(tmpdir=td)

status <- tryCatch(suppressWarnings(download.file(file_url, dl, mode='wb')), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

unzip(dl, 'church_white_gmsl_2011_up/CSIRO_Recons_gmsl_mo_2015.csv', exdir = td, overwrite = TRUE)
gmsl_tide <- readr::read_csv(file.path(td,'church_white_gmsl_2011_up/CSIRO_Recons_gmsl_mo_2015.csv'), col_types = readr::cols(`GMSL uncertainty (mm)` = readr::col_skip()))
colnames(gmsl_tide) <- c('date', 'gmsl_tide')
gmsl_tide$date <- lubridate::ymd(lubridate::round_date(lubridate::date_decimal(gmsl_tide$date),'day'))

gmsl <- suppressMessages( dplyr::full_join(gmsl_tide, gmsl_sat) )

diff <- dplyr::filter(gmsl, date >= as.Date('1993-01-01') & date < as.Date('1994-01-01'))
diff <- dplyr::summarize_all(diff, list(mean=mean), na.rm = TRUE)
diff <- diff$gmsl_tide_mean - diff$gmsl_sat_mean
gmsl$gmsl_sat <- gmsl$gmsl_sat + diff

gmsl <- tidyr::pivot_longer(gmsl, -date, values_drop_na = TRUE, names_to = 'method', values_to = 'gmsl')

if (write_cache) saveRDS(gmsl, file.path(hs_path, 'sealevel.rds'))

invisible(gmsl) }


#' Download and plot essential climate data
#'
#' Plots the global mean sea level data retrieved using `get_sealevel()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_sealevel
#' @param dataset Name of the tibble generated by `get_sealevel`, defaults to calling `get_sealevel`
#' @param print (boolean) Display sealevel ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#'
#' @return Invisibly returns a ggplot2 object with sealevel chart
#'
#' @details `plot_sealevel` invisibly returns a ggplot2 object with a pre-defined sealevel change chart using data from `get_sealevel`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom lubridate ymd ceiling_date
#'
#' @examples
#' \donttest{
#' # Fetch sealevel data:
#' gmsl <- get_sealevel()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_sealevel(gmsl)
#'
#' # Or just call plot_sealevel(), which defaults to get_sealevel() dataset
#' plot_sealevel()
#'
#' p <- plot_sealevel(gmsl, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Rising Waters') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_sealevel <- function(dataset = get_sealevel(), print=TRUE) {

if (is.null(dataset)) return(invisible(NULL))

plot <-  ggplot(dataset, aes(x=date, color=method, y=gmsl)) +geom_line(linewidth=1) + theme_bw(base_size = 12) +
         scale_x_date(name=NULL, breaks='15 years', limits = c(ymd('1878-01-01'), ceiling_date(max(dataset$date), 'years')), date_labels ='%Y') +
         scale_color_manual(values=c('dodgerblue2','firebrick1'), labels=c('Satellite observations','Coastal tide gauge records')) +theme(legend.position = c(0.30,0.84), legend.background=element_blank(), legend.title = element_blank()) +
         scale_y_continuous(n.breaks = 8) +
         labs(title='Sea Level Rise', subtitle='Tide gauges: 1880-2013; Satellite: 1992-present, calibrated to 1993 mean tide gauge.', y= 'Variation (mm)',
         caption='Sources: NOAA Laboratory for Satellite Altimetry (sat)\nhttps://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise\nCommonwealth Scientific and Industrial Research Organisation (tide gauge)\nhttps://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/')

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}

#' Download and plot essential climate data
#'
#' Plots global "warming stripes" graph in the style popularized by Ed Hawkins, based on temperature anomaly retrieved using `get_temp()`.
#' Function can output stripes chart with legend or a minimal chart. The output ggplot2 object may be further modified.
#'
#'
#' @name warming_stripes
#' @param dataset Name of the tibble generated by `get_temp`
#' @param stripe_only Display legend and axes, defaults to TRUE
#' @param print (boolean) Display warming stripe ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param col_strip Color palette to use.  Defaults to Red-Blue RColorBrewer palette.
#'
#' @return Invisibly returns a ggplot2 object with warming stripes
#'
#' @details `warming_stripes` invisibly returns a ggplot2 object with warming stripes chart using data from `get_temp`.
#' By default the chart is also displayed. User may modify color palette or remove axes and legend. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom RColorBrewer brewer.pal
#'
#' @examples
#' \donttest{
#' # Draw with axes and legend
#' stripes <- warming_stripes()
#'
#' # Draw stripes only
#' stripes <- warming_stripes(stripe_only = TRUE)
#'
#' # Don't display, store for further modifications
#' stripes <- warming_stripes(print = FALSE)
#'
#' # Change color palette
#' stripes <- warming_stripes(stripe_only = TRUE, col_strip = viridisLite::viridis(11)) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item Climate Lab. 2018. https://www.climate-lab-book.ac.uk/2018/warming-stripes/
#' \item GISS Surface Temperature Analysis (GISTEMP v4): \url{https://data.giss.nasa.gov/gistemp/}
#' \item GISTEMP Team, 2020: \emph{GISS Surface Temperature Analysis (GISTEMP), version 4.} NASA Goddard Institute for Space Studies.
#' \item Dr. Dominic Roye blog post "How to Create Warming Stripes in R": https://dominicroye.github.io/en/2018/how-to-create-warming-stripes-in-r/
#'  }
#'
#' @export

warming_stripes <- function(dataset = get_temp(), stripe_only = FALSE,
                            col_strip = RColorBrewer::brewer.pal(11, "RdBu"),
                            print = TRUE) {

if (is.null(dataset)) return(invisible(NULL))

theme_strip <- function() theme_bw(base_size=12) +
  theme(axis.text.y = element_blank(),
        axis.line.y = element_blank(),
        axis.ticks.y = element_blank(),
        axis.title = element_blank(),
        panel.grid.major = element_blank(),
        legend.title = element_blank(),
        axis.text.x = element_text(vjust = 0.5, angle = -90),
        panel.grid.minor = element_blank()
  )

if (!stripe_only) {
plot <- ggplot(dataset, aes(x = Year, y = 0, fill = `J-D`)) +
  geom_tile(height=2.6) +
  scale_x_date(date_breaks = "20 years", date_labels = "%Y", expand = c(0, 0)) +
  scale_y_continuous(expand = c(0, 0)) +
  scale_fill_gradientn(colors = rev(col_strip)) +
  guides(fill = guide_colorbar(barwidth = 1)) +
  labs(title = "Global surface temperature anomaly",
       subtitle= "Relative to 1951-1980 average",
       caption = 'Source: NASA Goddard Institute for Space Studies\nhttps://data.giss.nasa.gov/gistemp/') +
  theme_strip() } else

              {
plot <- ggplot(dataset, aes(x = Year, y = 1, fill = `J-D`)) +
  geom_tile(show.legend = FALSE) + scale_x_date(expand = c(0, 0)) +
  scale_y_continuous(expand = c(0, 0)) +
  scale_fill_gradientn(colors = rev(col_strip)) +
  theme_void() }

if (print) suppressMessages( print(plot) )
invisible(plot)
}


#' Download and plot essential climate data
#'
#' Plots a 2x2 grid of carbon, temperature, sea ice, and sea level charts.
#'
#'
#' @name climate_grid
#' @param print (boolean) Display climate grid ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#'
#' @return Invisibly returns a ggplot2 object with climate grid
#'
#' @details `warming_stripes` invisibly returns a ggplot2 object with 2x2 grid of carbon, temperature, sea ice, and sea level charts from `get_carbon`, `get_temp`, `get_seaice`, and `get_sealevel`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @import patchwork
#'
#' @examples
#' \donttest{
#' # Draw grid
#'
#' grid <- climate_grid() }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

climate_grid <- function(print = TRUE) {

  a <- plot_carbon(print = FALSE, annot=FALSE) +theme_bw(base_size = 9)

    b <- plot_temp(print = FALSE) +theme_bw(base_size = 9) +theme(legend.position = "none") +
    labs(title='Global Land-Ocean Temperature Index', subtitle='Global surface temperature relative to 1951-80 mean',
         y='Temperature Anomaly (C\U00B0)', caption='Source: NASA Goddard Institute for Space Studies\nhttps://data.giss.nasa.gov/gistemp/')
  c <- suppressMessages( plot_seaice(print = FALSE) +theme_bw(base_size = 9) )

  d <- plot_sealevel(print = FALSE)

  if (is.null(a) | is.null(b) | is.null(c) | is.null(d)) return(invisible(NULL))

  d <- d +  labs(title='Sea Level Rise', subtitle='Tide gauges: 1880-2009; Satellite: 1992-present.', y= 'Variation (mm)',
         caption='Sources: NOAA Laboratory for Satellite Altimetry (sat)\nhttps://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise\nCSIRO (tide gauge)\nhttp://www.cmar.csiro.au/sealevel/sl_data_cmar.html') +theme_bw(base_size = 9) +theme(legend.position = "none")

  plot <- patchwork::wrap_plots(a, b, c, d, ncol = 2) + patchwork::plot_annotation(title='Carbon and Global Warming')

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}



#' Download and plot essential climate data
#'
#' Plots a treemap of cumulative co2 emissions by country since 1900.
#'
#'
#' @name emissions_map
#' @param dataset Name of the tibble generated by `get_emissions`
#' @param print (boolean) Display emissions treemap, defaults to TRUE. Use FALSE to not display chart.
#' @param since (numeric) Start year for cumulative emissions, defaults to 1900 if omitted
#' @param number (numeric) Number of countries to display in treemap, defaults to all if omitted
#' @param title (string) Manually specify chart title
#'
#' @return Invisibly returns a ggplot2 object with emissions treemap
#'
#' @details `emissions_map` invisibly returns a ggplot2 object with cumulative emissions treemap by country since 1900 from `get_emissions`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @import dplyr
#' @import treemapify
#'
#' @examples
#' \donttest{
#' # Draw treemap
#'
#' co2map <- emissions_map()
#'
#' co2map <- emissions_map(since=2000, number=20, title="Top 20 Cumulative Emitters Since 2000") }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export


emissions_map <- function(dataset=get_emissions(), print = TRUE, since=1900, number="all",
                          title = substitute(paste(since,'-',to,' Cumulative '*CO[2]*" Emissions by Country"),
                                             list(since=since, to=as.character(dataset[nrow(dataset), 2])))) {

  if (is.null(dataset)) return(invisible(NULL))

  treemap <- dataset |> filter(year >= since) |> group_by(country) |>
    summarize(cumco2=sum(co2, na.rm = T)) |> arrange(-cumco2) |>
    filter(!grepl("World|Europe|Asia|Africa|America|OECD|transport|Oceania|Middle|countries", country))

  if(number!="all") treemap <- slice_head(treemap, n=number)

  plot <- ggplot(treemap, aes(area = cumco2, fill = cumco2, label = country)) + theme_minimal(base_size = 14) +
    geom_treemap() + geom_treemap_text(color=c("white", "black", rep("white", nrow(treemap)-2))) + scale_fill_viridis_c(option = "H") +
    theme(legend.position = "none") +
    labs(title = title, caption="Source: Global Carbon Project and Our World In Data")

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}

#' Download and plot essential climate data
#'
#' Merge NOAA carbon and NASA temperature datasets on common dates.
#'
#'
#' @name merge_carbontemp
#' @param carbon Name of the tibble generated by `get_carbon`
#' @param temp Name of the tibble generated by `get_temp`
#'
#' @return Invisibly returns a tibble with merged datasets from `get_carbon` and `get_temp` functions.
#'
#' @details `merge_carbontemp` invisibly returns a tibble with the merged data from from `get_carbon` and `get_temp` functions.
#' Tibble only includes data from dates when both datasets are available, essentially from 1960.
#'
#' @import lubridate
#' @import dplyr
#'
#' @examples
#' \donttest{
#' # Create merged tibble
#'
#' mergedcarbontemp <- merge_carbontemp() }
#'
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

merge_carbontemp <- function(carbon=get_carbon(), temp=get_temp() ) {

  if (is.null(carbon) | is.null(temp)) return(invisible(NULL))

temp <- temp |> select(1:13) |> tidyr::pivot_longer(cols = 2:13, names_to = 'Month', values_to = 'Anomaly') |>
  filter(!is.na(Anomaly))

temp <- temp |> mutate(Date = ceiling_date(lubridate::ym(paste(year(Year), Month)), 'month') - days(1)) |>
  select(Date, Anomaly)

carbon <- carbon |> select(Date=date, Year=year, Carbon=average)

output <- inner_join(carbon, temp, by='Date')

return(invisible(output))  }




#' Download and plot essential climate data
#'
#' Plots the global monthly mean temperature anomaly vs atmospheric carbon  with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_carbontemp
#' @param dataset Name of the tibble generated by `merge_carbontemp`
#' @param print (boolean) Display temperature anomaly ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#'
#' @return Invisibly returns a ggplot2 object with temperature anomaly vs carbon chart
#'
#' @details `plot_carbontemp` invisibly returns a ggplot2 object with a pre-defined temperature anomaly vs carbon chart using data from `merge_carbontemp`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @importFrom ggplot2 ggplot
#'
#' @examples
#' \donttest{
#' # Fetch temperature anomaly:
#' mergedtemp <- merge_carbontemp()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_carbontemp(mergedtemp)
#'
#' # Or just call plot_carbontemp(), which defaults to merge_carbontemp() dataset
#' plot_carbontemp()
#'
#' p <- plot_carbontemp(mergedtemp, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='The Signature of Climate Change') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_carbontemp <- function(dataset = merge_carbontemp(), print=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  plot <- ggplot(dataset, aes(x=Carbon, y=Anomaly, color=Year)) + geom_point() + theme_bw() + scale_y_continuous(n.breaks = 8) +
    scale_color_fermenter(palette = "YlOrRd", direction = 1, labels=c('1960-70s', '1980-90s', '2000-10s', '2020-40s')) +
    theme(legend.key.height = unit(1,"cm"), legend.text = element_text(vjust = -1.5)) +
    scale_x_continuous(n.breaks = 6) +
    labs(title=expression('Global Heating and Atmospheric '*CO[2]*' Since 1959'),
         subtitle='Monthly temperature relative to 1951-80 mean and Mauna Loa monthly mean', x=expression(title='Atmospheric '*CO[2]*' (ppm)'),
         y='Global Temperature Anomaly (C\U00B0)', color='20-year\nperiods',
         caption='Sources: NASA Goddard Institute for Space Studies\nNOAA/ESRL and Scripps Institution of Oceanography')

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}

#' Download and plot essential climate data
#'
#' Retrieves the combined global land- and sea-surface temperature anomaly (Land-Ocean Temperature Index, LOTI).
#' Source is NASA/GISS Surface Temperature Analysis (GISTEMP v4), an estimate of global surface temperature change.
#' \url{https://data.giss.nasa.gov/gistemp/}
#'
#' @name get_temp
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with the annual mean and monthly Combined Land-Surface Air and Sea-Surface Water Temperature Anomalies.
#'
#' `get_temp` invisibly returns a tibble with the NASA/GISS annual mean and monthly global temperature anomaly. Data are global from 1880 to present, and represent the deviations from the 1951-1980 mean.
#'
#' The returned object includes monthly and annual average anomalies, as well as seasonal anomalies.
#' GISS Surface Temperature Analysis (GISTEMP v4) is an estimate of global surface temperature change.
#'
#' Data are updated around the middle of every month using current data files from NOAA GHCN v4 (meteorological stations) and ERSST v5 (ocean areas).
#' Station data are combined as described in Hansen et al. (2010) \url{https://data.giss.nasa.gov/gistemp/references.html} and Lenssen et al. (2019) \url{https://www.giss.nasa.gov/pubs/abs/le05800h.html}
#'
#' @importFrom readr read_csv
#' @importFrom lubridate ymd
#' @importFrom utils download.file
#'
#' @examples
#' \donttest{
#' # Fetch temp anomaly from cache if available:
#' anomaly <- get_temp()
#' #
#' # Force cache refresh:
#' anomaly <- get_temp(use_cache=FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_temp(anomaly) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item GISS Surface Temperature Analysis (GISTEMP v4): \url{https://data.giss.nasa.gov/gistemp/}
#' \item GISTEMP Team, 2020: \emph{GISS Surface Temperature Analysis (GISTEMP), version 4.} NASA Goddard Institute for Space Studies.
#' \item Lenssen, N., G. Schmidt, J. Hansen, M. Menne, A. Persin, R. Ruedy, and D. Zyss, 2019:  Improvements in the GISTEMP uncertainty model.
#'  \emph{J. Geophys. Res. Atmos.}, 124, no. 12, 6307-6326, doi:10.1029/2018JD029522. \url{https://www.giss.nasa.gov/pubs/abs/le05800h.html}
#'  }
#'
#' @export
get_temp <- function(use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

hs_path <- tools::R_user_dir("hockeystick","cache")

if (use_cache) {
  if (file.exists(file.path(hs_path,'gisstemp.rds'))) return(invisible(readRDS((file.path(hs_path,'gisstemp.rds')))))
  }

file_url <- 'https://data.giss.nasa.gov/gistemp/tabledata_v4/GLB.Ts+dSST.csv'
connected <- .isConnected(file_url)
if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

dl <- tempfile()
status <- tryCatch(suppressWarnings(download.file(file_url, dl)), error = function(e) 1L)
if (status!=0L) {message("Unable to access remote resource."); return(invisible(NULL))}

gisstemp <- suppressMessages( readr::read_csv(dl, skip=1, na='***') )
# Compute year-to-date mean for current year
gisstemp[nrow(gisstemp), 'J-D'] <- mean(as.numeric(gisstemp[nrow(gisstemp), 2:13]), na.rm = T)
gisstemp$Year <- ymd(paste(gisstemp$Year,'12','31',sep='-'))
dir.create(hs_path, showWarnings = FALSE, recursive = TRUE)
if (write_cache) saveRDS(gisstemp, file.path(hs_path, 'gisstemp.rds'))

invisible(gisstemp)
}



#' Download and plot essential climate data
#'
#' Plots the global annual mean temperature anomaly retrieved using `get_temp()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_temp
#' @param dataset Name of the tibble generated by `get_temp`
#' @param print (boolean) Display temperature anomaly ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#'
#' @return Invisibly returns a ggplot2 object with temperature anomaly chart
#'
#' @details `plot_temp` invisibly returns a ggplot2 object with a pre-defined temperature annual mean anomaly chart using data from `get_temp`.
#' By default the chart is also displayed. Smooths using ggplot2's built-in loess smoother. Users may further modify the output ggplot2 chart.
#'
#' @importFrom ggplot2 ggplot
#'
#' @examples
#' \donttest{
#' # Fetch temperature anomaly:
#' anomaly <- get_temp()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_temp(anomaly)
#'
#' # Or just call plot_temp(), which defaults to get_temp() dataset
#' plot_temp()
#'
#' p <- plot_temp(anomaly, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='The Signature of Climate Change') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_temp <- function(dataset = get_temp(), print=TRUE) {

if (is.null(dataset)) return(invisible(NULL))

plot <- ggplot(dataset, aes(x=Year, y=`J-D`)) +geom_line(alpha=0.75, aes(color='Annual mean')) + theme_bw(base_size=12) +
  scale_x_date(name=NULL, limits=c(as.Date('1878-01-01'), ymd(max(dataset$Year))), date_breaks='15 years', date_labels='%Y') +
  scale_y_continuous(n.breaks = 9) +geom_smooth(linewidth=1.1, se=F, span=0.3, aes(color='Loess smoothing')) +
  labs(title='Global Land-Ocean Temperature Index (LOTI)', subtitle='Global surface temperature relative to 1951-1980 mean',
       y='Temperature Anomaly (C\U00B0)', caption='Source: NASA Goddard Institute for Space Studies\nhttps://data.giss.nasa.gov/gistemp/') +
  scale_color_manual(name=NULL, values=c('dodgerblue2','firebrick1')) +theme(legend.position = c(0.175, 0.825),legend.background=element_blank())

if (print) suppressMessages( print(plot) )
invisible(plot)
}



#' Download and plot essential climate data
#'
#' Plots the monthly mean temperature anomaly retrieved using `get_temp()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_temp_monthly
#' @param dataset Name of the tibble generated by `get_temp`
#' @param print (boolean) Display temperature anomaly ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#'
#' @return Invisibly returns a ggplot2 object with temperature anomaly chart
#'
#' @details `plot_temp_monthly` invisibly returns a ggplot2 object with a pre-defined temperature monthly mean anomaly chart using data from `get_temp`.
#' By default the chart is also displayed. Smooths using ggplot2's built-in loess smoother. Users may further modify the output ggplot2 chart.
#'
#' @importFrom ggplot2 ggplot
#' @importFrom dplyr mutate select
#' @importFrom lubridate ceiling_date days year
#'
#' @examples
#' \donttest{
#' # Fetch temperature anomaly:
#' anomaly <- get_temp()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_temp_monthly(anomaly)
#'
#' # Or just call plot_temp_monthly(), which defaults to get_temp() dataset
#' plot_temp_monthly()
#'
#' p <- plot_temp_monthly(anomaly, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='The Signature of Climate Change') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_temp_monthly <- function(dataset = get_temp(), print=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  dataset <- dataset |> select(1:13) |> tidyr::pivot_longer(cols = 2:13, names_to = 'Month', values_to = 'Anomaly') |>
    filter(!is.na(Anomaly))

  dataset <- dataset |> mutate(Date = ceiling_date(ym(paste(year(Year), Month)), 'month') - days(1))

  plot <- ggplot(dataset, aes(x=Date, y=Anomaly)) +geom_line(alpha=0.75, aes(color='Monthly mean')) + theme_bw(base_size=12) +
    scale_x_date(name=NULL, limits=c(as.Date('1878-01-01'), ymd(max(dataset$Year))), date_breaks='15 years', date_labels='%Y') +
    scale_y_continuous(n.breaks = 10) +geom_smooth(linewidth=1.1, se=FALSE, span=0.3, aes(color='Loess smoothing')) +
    labs(title='Global Land-Ocean Temperature Index (LOTI)', subtitle='Global surface temperature relative to 1951-1980 mean',
         y='Temperature Anomaly (C\U00B0)', caption='Source: NASA Goddard Institute for Space Studies\nhttps://data.giss.nasa.gov/gistemp/') +
    scale_color_manual(name=NULL, values=c('firebrick1','dodgerblue2')) +
    theme(legend.position = c(0.175, 0.825), legend.background=element_blank())

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}



#' Download and plot essential climate data
#'
#' Plots scatter of monthly temperature anomaly retrieved using `get_temp()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_temp_scatter
#' @param dataset Name of the tibble generated by `get_temp`
#' @param print (boolean) Display temperature anomaly ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param labellatest (boolean) Display date of latest value, default = TRUE
#' @param labelmax (boolean) Display date of max value, default = FALSE
#'
#' @return Invisibly returns a ggplot2 object with temperature anomaly chart
#'
#' @details `plot_temp_scatter` invisibly returns a ggplot2 object with a pre-defined temperature monthly mean anomaly chart using data from `get_temp`.
#' By default the chart is also displayed. Smooths using ggplot2's built-in loess smoother. Users may further modify the output ggplot2 chart.
#'
#' @importFrom ggplot2 ggplot
#' @importFrom dplyr mutate select
#' @importFrom lubridate ceiling_date days ym
#'
#' @examples
#' \donttest{
#' # Fetch temperature anomaly:
#' anomaly <- get_temp()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_temp_scatter(anomaly)
#'
#' # Or just call plot_temp_scatter(), which defaults to get_temp() dataset
#' plot_temp_scatter()
#'
#' p <- plot_temp_scatter(anomaly, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='The Signature of Climate Change') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_temp_scatter <- function(dataset = get_temp(), print=TRUE, labelmax=FALSE, labellatest=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  dataset <- dataset |> select(1:13) |> tidyr::pivot_longer(cols = 2:13, names_to = 'Month', values_to = 'Anomaly') |>
    filter(!is.na(Anomaly))

  dataset <- dataset |> mutate(Date = ceiling_date(ym(paste(year(Year), Month)), 'month') - days(1))

  col_strip <- RColorBrewer::brewer.pal(11, "RdBu")[-(5:7)]

  plot <- ggplot(dataset, aes(x=Year, y=Anomaly, color=Anomaly)) + geom_point(size=1) + theme_bw(base_size=12) +
    scale_x_date(name=NULL, limits=c(as.Date('1878-01-01'), ymd(max(dataset$Year))), date_breaks='15 years', date_labels='%Y') +
    scale_y_continuous(n.breaks = 8)  +
    labs(title='Global Land-Ocean Temperature Index (LOTI)', subtitle='Global surface temperature relative to 1951-1980 mean', color='',
         y='Monthly Temperature Anomaly (C\U00B0)', caption='Source: NASA Goddard Institute for Space Studies\nhttps://data.giss.nasa.gov/gistemp/') +
    theme(legend.position='inside', legend.position.inside = c(0.09, 0.85),legend.background=element_blank() ) +
    scale_color_gradientn(colors = rev(col_strip))

  if (labelmax) {
    dataset <- dataset |> mutate(max = ifelse(Anomaly == max(dataset$Anomaly), T, F))
    dataset <- dataset |> mutate(namemax = ifelse(max == T, substr(as.character(Date), 1, 7), ''))
    plot <- plot + geom_text(aes(y = Anomaly+0.1, label=dataset$namemax), size=3) }


  if (labellatest) {
    dataset$namelatest <- ''
    dataset[nrow(dataset),'namelatest'] <- substr(as.character(pull(dataset[nrow(dataset),'Date'])), 1, 7)
    plot <- plot + geom_text(aes(y = Anomaly+0.3, label=dataset$namelatest), size=3) }

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}

#' Download and plot essential climate data
#'
#' Retrieves Arctic or Antarctic annual Sea Ice Index (in million square km).
#' Source is the National Snow and Ice Data Center, defaults to Arctic (Northern Hemisphere) July sea ice extent.
#' \url{https://nsidc.org/data/explore-data} \cr \cr
#'
#' @name get_seaice
#' @param pole 'N' for Arctic or 'S' for Antarctic
#' @param month 2-digit month to retrieve sea ice for, defaults to 7 (July)
#' @param measure Must be 'extent' or 'area', defaults to 'extent'. Please see terminology link in references for details.
#' @param use_cache (boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data, or to change pole or month in cache.
#' @param write_cache (boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)
#'
#' @return Invisibly returns a tibble with the annual series of monthly Sea Ice Index since 1979 (in million square km).
#'
#' `get_seaice` invisibly returns a tibble with annual series of monthly Sea Ice Index since 1979 (in million square km).
#'
#' User may select Arctic or Antarctic sea ice extent or area (in millions of square kilometers) by year for a given month, specified by argument `month`.
#' Defaults to Arctic July sea ice extent.  \url{https://nsidc.org/sea-ice-today/about-data#area_extent}
#'
#' @importFrom lubridate ymd ceiling_date
#' @importFrom utils download.file
#' @importFrom readxl read_xlsx
#' @importFrom tibble tibble
#'
#' @examples
#' \donttest{
#' # Fetch sea ice from cache if available:
#' seaice <- get_seaice()
#' #
#' # Force cache refresh:
#' seaice <- get_seaice(use_cache = FALSE)
#' # change region and month
#' seaice <- get_seaice(pole='S', month=9, use_cache = FALSE)
#' #
#' # Review cache contents and last update dates:
#' hockeystick_cache_details()
#' #
#' # Plot output using package's built-in ggplot2 settings
#' plot_seaice(seaice) }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#' @references
#' \itemize{
#' \item NSIDC Data Archive: \url{https://nsidc.org/data/explore-data}
#' \item All About Sea Ice: \url{https://nsidc.org/learn/parts-cryosphere/sea-ice}
#' \item Sea Ice terminology: \url{https://nsidc.org/learn}
#'  }
#'
#' @export

get_seaice <- function(pole='N', month=7, measure='extent',
                       use_cache = TRUE, write_cache = getOption("hs_write_cache")) {

  if (pole!='S' & pole!='N') stop("pole must be 'N' or 'S'")
  if (!(month %in% 1:12)) stop("Month must be a value between 1 and 12")
  if (measure != 'extent' & measure != 'area') stop("measure must be 'extent' or 'area'")

  hs_path <- tools::R_user_dir("hockeystick","cache")

  if (use_cache) {
    message('Please set use_cache=FALSE if you are changing pole, month, or measure from last cached data.')
    if (file.exists(file.path(hs_path,'seaice.rds'))) return(invisible(readRDS((file.path(hs_path,'seaice.rds')))))
    }

  connected <- .isConnected('https://noaadata.apps.nsidc.org/NOAA/G02135/seaice_analysis/Sea_Ice_Index_Monthly_Data_with_Statistics_G02135_v4.0.xlsx')
  if (!connected) {message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

  filename <- 'https://noaadata.apps.nsidc.org/NOAA/G02135/seaice_analysis/Sea_Ice_Index_Monthly_Data_with_Statistics_G02135_v4.0.xlsx'

  dl <- tempfile()

  status <- tryCatch(suppressWarnings(download.file(filename, dl, mode = 'wb')), error = function(e) 1L)
  if (status!=0L){message("Retrieving remote data requires connectivity to source."); return(invisible(NULL))}

  sheet <- paste0(month.name[month], '-', pole, 'H')
  seaice <- suppressMessages( read_xlsx(dl, skip=9, sheet=sheet) )
  seaice$date <- lubridate::ceiling_date(lubridate::ymd(paste(seaice$year, seaice$month, '01', sep='-')), 'month')-1

  if (measure == 'extent') seaice <- seaice[,c('date', 'extent')] else seaice <- seaice[,c('date', 'area')]

  seaice <- tibble::tibble(seaice)

  if (write_cache) saveRDS(seaice, file.path(hs_path, 'seaice.rds'))

  invisible(seaice) }



#' Download and plot essential climate data
#'
#' Plots the Sea Ice Index data retrieved using `get_seaice()` with ggplot2. The output ggplot2 object may be further modified.
#'
#'
#' @name plot_seaice
#' @param dataset Name of the tibble generated by `get_seaice`, defaults to calling `get_seaice`
#' @param print (boolean) Display sea ice ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.
#' @param title chart title, defaults to Arctic Sea Ice
#' @return Invisibly returns a ggplot2 object with Sea Ice Index chart
#'
#' @details `plot_seaice` invisibly returns a ggplot2 object with a pre-defined Sea Ice Index chart using data from `get_seaice`.
#' By default the chart is also displayed. Users may further modify the output ggplot2 chart.
#'
#' @import ggplot2
#' @importFrom lubridate ymd ceiling_date month
#'
#' @examples
#' \donttest{
#' # Fetch sea ice data:
#' seaice <- get_seaice()
#' #
#' # Plot output using package's built-in ggplot2 defaults
#' plot_seaice(seaice)
#'
#' # Or just call plot_seaice(), which defaults to get_seaice() dataset
#' plot_seaice()
#'
#' p <- plot_seaice(seaice, print = FALSE)
#' # Modify plot such as: p + ggplot2::labs(title='Shrinking Arctic Sea Ice') }
#'
#' @author Hernando Cortina, \email{hch@@alum.mit.edu}
#'
#' @export

plot_seaice <- function(dataset = get_seaice(), title='Arctic Sea Ice', print=TRUE) {

  if (is.null(dataset)) return(invisible(NULL))

  subtitle <- paste0(as.character(lubridate::month(dataset[nrow(dataset),'date', drop=TRUE], label=TRUE, abbr=FALSE))," mean sea ice ", colnames(dataset)[2],". Linear regression in blue.")

  plot <-  ggplot(dataset, aes(x=date, y=.data[[colnames(dataset)[2]]])) +geom_line(linewidth=1, color='firebrick1') +
    scale_x_date(name=NULL, breaks='5 years', date_labels='%Y', limits=c(ymd('1978-01-01'), ceiling_date(max(dataset$date), 'years'))) +
    scale_y_continuous(n.breaks = 6, limits=c(0,max(dataset[,2]))) + geom_smooth(method='lm', se=FALSE, linetype=2, linewidth=0.5) + theme_bw(base_size = 12) +
    labs(title=title,
         subtitle=subtitle, y=expression("Million km"^2),
         caption='Source: National Snow & Ice Data Center\nhttps://nsidc.org/data/seaice_index')

  if (print) suppressMessages( print(plot) )
  invisible(plot)
}

test_that("Package loads without error", {
  expect_true("hockeystick" %in% loadedNamespaces())
})

test_that("Data getter functions are exported", {
  exported <- getNamespaceExports("hockeystick")

  expect_true("get_carbon" %in% exported)
  expect_true("get_methane" %in% exported)
  expect_true("get_temp" %in% exported)
  expect_true("get_temp2k" %in% exported)
  expect_true("get_paleo" %in% exported)
  expect_true("get_sealevel" %in% exported)
  expect_true("get_seaice" %in% exported)
  expect_true("get_hurricanes" %in% exported)
  expect_true("get_emissions" %in% exported)
  expect_true("get_dailytemp" %in% exported)
  expect_true("get_dailytempcop" %in% exported)
  expect_true("get_icecurves" %in% exported)
})

test_that("Plot functions are exported", {
  exported <- getNamespaceExports("hockeystick")

  expect_true("plot_carbon" %in% exported)
  expect_true("plot_methane" %in% exported)
  expect_true("plot_temp" %in% exported)
  expect_true("plot_temp_monthly" %in% exported)
  expect_true("plot_temp_scatter" %in% exported)
  expect_true("plot_temp2k" %in% exported)
  expect_true("plot_paleo" %in% exported)
  expect_true("plot_sealevel" %in% exported)
  expect_true("plot_seaice" %in% exported)
  expect_true("plot_icecurves" %in% exported)
  expect_true("plot_hurricanes" %in% exported)
  expect_true("plot_hurricane_nrg" %in% exported)
  expect_true("plot_emissions" %in% exported)
  expect_true("plot_emissions_with_land" %in% exported)
  expect_true("plot_dailytemp" %in% exported)
  expect_true("warming_stripes" %in% exported)
})

test_that("Cache functions are exported", {
  exported <- getNamespaceExports("hockeystick")
  
  expect_true("hockeystick_cache_list" %in% exported)
  expect_true("hockeystick_cache_delete" %in% exported)
  expect_true("hockeystick_cache_delete_all" %in% exported)
  expect_true("hockeystick_cache_details" %in% exported)
  expect_true("hockeystick_update_all" %in% exported)
})

test_that("Helper functions are exported", {
  exported <- getNamespaceExports("hockeystick")
  
  expect_true("merge_carbontemp" %in% exported)
  expect_true("plot_carbontemp" %in% exported)
  expect_true("climate_grid" %in% exported)
  expect_true("emissions_map" %in% exported)
})

test_that("hockeystick_cache_list is callable", {
  expect_true(is.function(hockeystick_cache_list))
})

test_that("hockeystick_cache_delete is callable", {
  expect_true(is.function(hockeystick_cache_delete))
})

test_that("hockeystick_cache_delete_all is callable", {
  expect_true(is.function(hockeystick_cache_delete_all))
})

test_that("hockeystick_cache_details is callable", {
  expect_true(is.function(hockeystick_cache_details))
})

test_that("hockeystick_update_all is callable", {
  expect_true(is.function(hockeystick_update_all))
})

test_that("Cache functions have expected signatures", {
  expect_equal(length(formals(hockeystick_cache_list)), 0)
  expect_equal(length(formals(hockeystick_cache_delete)), 2)
  expect_equal(length(formals(hockeystick_cache_delete_all)), 1)
})

---
title: "Using Hockeystick"
fig-format: svg
vignette: >
  %\VignetteIndexEntry{Using Hockeystick}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk:
    collapse: true
    dev: "svglite"
    comment: '#>'
---
`hockeystick` includes functions to download and visualized climate data.  Data is optionally cached.

### Retrieve NOAA/ESRL Mauna Loa CO<sub>2</sub> Observatory concentration data and plot:
```{r}
#| label: carbon
#| message: false
library(hockeystick)
ml_co2 <- get_carbon()
plot_carbon(ml_co2)
```

### Retrieve GCP global CO~2~ emissions and plot:
```{r}
#| label: emissions
#| message: false
emissions <- get_emissions()
plot_emissions(emissions)
plot_emissions_with_land(emissions)
```


### Visualize cumulative emissions by country:
```{r}
#| label: emissionsmap
#| message: false
emissions_map()
```

### Retrieve NASA/GISS global surface temperature anomaly data and plot:
```{r}
#| label: anomaly
#| message: false
anomaly <- get_temp()
plot_temp(anomaly)
```

### Plot relationship between temperature anomaly and carbon:
```{r}
#| label: tempcarbon
#| message: false
plot_carbontemp()
```


### Visualize warming using Ed Hawkins styled "warming stripes":
```{r}
#| label: stripes
#| message: false
warming_stripes()
warming_stripes(stripe_only = TRUE, col_strip = viridisLite::viridis(11))
```

### Retrieve tide gauge and satellite sea level data and plot:
```{r}
#| label: sealevel
#| message: false
gmsl <- get_sealevel()
plot_sealevel(gmsl)
```


### Retrieve July annual Arctic Sea Ice Index and plot:
```{r}
#| label: ice
#| message: false
seaice <- get_seaice()
plot_seaice(seaice)
```

`get_seaice()` arguments can be modified to download Antarctic sea ice, and allow any month.

You can also visualize sea ice by month and year:
```{r}
#| label: ice2
#| message: false
arcticice <- get_icecurves()
plot_icecurves(arcticice)
```

### Retrieve NOAA HURDAT2 hurricane data and plot:
```{r}
#| label: hurdat
#| message: false
hurricanes <- get_hurricanes()
plot_hurricanes(hurricanes)
plot_hurricane_nrg(hurricanes)
```

### Retrieve NOAA/ESRL CH<sub>4</sub> Globally averaged mean data and plot:
```{r}
#| label: methane
#| message: false
ch4 <- get_methane()
plot_methane(ch4)
```

### Retrieve Vostok paleo ice core data and plot:
```{r}
#| label: paleo
#| message: false
vostok <- get_paleo()
plot_paleo(vostok)
```

### Managing the cache
By default, no climate data is cached, and all data is downloaded every time any of the `get_` functions is called. To cache data for future use, use the `write_cache = TRUE` option, available in all of the `get_` functions. To download and cache all data use `hockeystick_update_all()`. To view the files, date, and size of cached data use `hockeystick_cache_details()`.  To re-download data from the source use the `use_cache = FALSE` argument in any of the `get_` functions, for example: `get_carbon(use_cache = FALSE, write_cache = TRUE)`.  To delete all cached data use `hockeystick_cache_delete_all()`.

Users may also cache data by default by adding `options(hs_write_cache = TRUE)`to their script or `.Rprofile` file.

---
title: "Using Hockeystick"
fig-format: svg
vignette: >
  %\VignetteIndexEntry{Using Hockeystick}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk:
    collapse: true
    dev: "svglite"
    comment: '#>'
---
`hockeystick` includes functions to download and visualized climate data.  Data is optionally cached.

### Retrieve NOAA/ESRL Mauna Loa CO<sub>2</sub> Observatory concentration data and plot:
```{r}
#| label: carbon
#| message: false
library(hockeystick)
ml_co2 <- get_carbon()
plot_carbon(ml_co2)
```

### Retrieve GCP global CO~2~ emissions and plot:
```{r}
#| label: emissions
#| message: false
emissions <- get_emissions()
plot_emissions(emissions)
plot_emissions_with_land(emissions)
```

### Retrieve 2081-2100 temperature projections and map:

```{r}
#| label: cmip6
#| message: false
proj <- get_cmip6()
plot_cmip6(proj)
```

### Retrieve 2081-2100 precipation anomaly and map:

```{r}
#| label: cmip6anom
#| message: false
anom <- get_cmip6_anom(var = 'prec')
plot_cmip6_anom(anom)
```

### Visualize cumulative emissions by country:
```{r}
#| label: emissionsmap
#| message: false
emissions_map()
```

### Retrieve NASA/GISS global surface temperature anomaly data and plot:
```{r}
#| label: anomaly
#| message: false
anomaly <- get_temp()
plot_temp(anomaly)
```

### Plot relationship between temperature anomaly and carbon:
```{r}
#| label: tempcarbon
#| message: false
plot_carbontemp()
```


### Visualize warming using Ed Hawkins styled "warming stripes":
```{r}
#| label: stripes
#| message: false
warming_stripes()
warming_stripes(stripe_only = TRUE, col_strip = viridisLite::viridis(11))
```

### Retrieve tide gauge and satellite sea level data and plot:
```{r}
#| label: sealevel
#| message: false
gmsl <- get_sealevel()
plot_sealevel(gmsl)
```


### Retrieve July annual Arctic Sea Ice Index and plot:
```{r}
#| label: ice
#| message: false
seaice <- get_seaice()
plot_seaice(seaice)
```

`get_seaice()` arguments can be modified to download Antarctic sea ice, and allow any month.

You can also visualize sea ice by month and year:
```{r}
#| label: ice2
#| message: false
arcticice <- get_icecurves()
plot_icecurves(arcticice)
```

### Retrieve NOAA HURDAT2 hurricane data and plot:
```{r}
#| label: hurdat
#| message: false
hurricanes <- get_hurricanes()
plot_hurricanes(hurricanes)
plot_hurricane_nrg(hurricanes)
```

### Retrieve GWIS wildfire data and plot:
```{r}
#| label: fires
#| message: false
usfires <- get_fires_area(place='USA', year = 2026)
plot_fires_area(usfires)
plot_fires_area(usfires, style = 'weekly')
```

### Retrieve NOAA/ESRL CH<sub>4</sub> Globally averaged mean data and plot:
```{r}
#| label: methane
#| message: false
ch4 <- get_methane()
plot_methane(ch4)
```

### Retrieve Vostok paleo ice core data and plot:
```{r}
#| label: paleo
#| message: false
vostok <- get_paleo()
plot_paleo(vostok)
```

### Managing the cache
By default, no climate data is cached, and all data is downloaded every time any of the `get_` functions is called. To cache data for future use, use the `write_cache = TRUE` option, available in all of the `get_` functions. To download and cache all data use `hockeystick_update_all()`. To view the files, date, and size of cached data use `hockeystick_cache_details()`.  To re-download data from the source use the `use_cache = FALSE` argument in any of the `get_` functions, for example: `get_carbon(use_cache = FALSE, write_cache = TRUE)`.  To delete all cached data use `hockeystick_cache_delete_all()`.

Users may also cache data by default by adding `options(hs_write_cache = TRUE)`to their script or `.Rprofile` file.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emissions.R
\name{get_emissions}
\alias{get_emissions}
\title{Download and plot essential climate data}
\usage{
get_emissions(use_cache = TRUE, write_cache = getOption("hs_write_cache"))
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with annual carbon dioxide emissions
}
\description{
Retrieves Global Carbon Project (GCP) annual global carbon dioxide emissions since 1750 from Our World In Data repository
\url{https://github.com/owid/co2-data}
}
\details{
\code{get_emissions} invisibly returns a tibble with GCP's annual carbon dioxide emissions from fossil fuels in aggregate and for every nation.
The returned object includes ISO code, country, year, co2 emissions, growth rates, per capita, and decompositions by industry and gas type.
Please refer to above website for details.
}
\examples{
\donttest{
# Fetch from cache if available:
emissions <- get_emissions()
#
# Force cache refresh:
emissions <- get_emissions(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_emissions(emissions) }

}
\references{
\url{https://www.globalcarbonproject.org/carbonbudget/}

Friedlingstein, P. et al (2020), Global Carbon Budget 2020, \emph{Earth System Science Data}, vol. 12, 3269-3340 \doi{10.5194/essd-12-3269-2020}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daily_temp.R
\name{plot_dailytemp}
\alias{plot_dailytemp}
\title{Download and plot essential climate data}
\usage{
plot_dailytemp(
  dataset = get_dailytempcop(),
  print = TRUE,
  anomaly = FALSE,
  maxtemp = FALSE,
  current_year = as.numeric(substr(Sys.Date(), 1, 4)),
  title_lab = "Daily Average Air Temperature",
  cop = TRUE
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_dailytempcop} or \code{get_dailytemp}}

\item{print}{(boolean) Display daily temperature ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}

\item{anomaly}{(boolean) Display current anomaly versus historic mean, defaults to FALSE.}

\item{maxtemp}{(boolean) Display current deviation versus historic max, defaults to FALSE.}

\item{current_year}{(numeric) Year to highlight in alternate color, defaults to current year.}

\item{title_lab}{(string) Title to override default chart title. Default title pulls region name from dataset attributes.}

\item{cop}{(boolean) Flag for chart caption, TRUE = Copernicus, FALSE =. ClimateReanalyzer.org}
}
\value{
Invisibly returns a ggplot2 object with daily temperature anomaly chart
}
\description{
Plots the daily temperatures since 1940 and current anomaly data retrieved using \code{get_dailytempcop()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_temp} invisibly returns a ggplot2 object with a pre-defined daily temperature anomaly chart using data from \code{get_dailytemp}.
By default the chart is also displayed. Plots one line per year, as well as mean and anomaly (which may be disabled). Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch temperature anomaly:
dailydata <- get_dailytempcop()
#
# Plot output using package's built-in ggplot2 defaults
plot_dailytemp(dailydata)

# Don't plot anomaly shading and highight specific year
plot_dailytemp(anomaly = FALSE, current_year = 2012)

# Or just call plot_temp(), which defaults to get_dailytempcop() dataset
plot_dailytemp()

p <- plot_dailytemp(dailydata, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Record Temperatures in 2023') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paleo.R
\name{get_paleo}
\alias{get_paleo}
\title{Download and plot essential climate data}
\usage{
get_paleo(use_cache = TRUE, write_cache = getOption("hs_write_cache"))
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with the age of the ice (years before C.E.), carbon dioxide (ppm) and temperature (degrees C).

\code{get_paleo} invisibly returns a tibble with Vostok ice core data: the age of the ice (years before C.E.), carbon dioxide (ppm) and temperature (degrees C).

Data are from: Barnola J; Raynaud D; Lorius C; Barkov N  (2003): Historical Carbon Dioxide Record from the Vostok Ice Core (417,160 - 2,342 years BP) and
Petit J R ; Raynaud D ; Lorius C ; Delaygue G ; Jouzel J ; Barkov N I ; Kotlyakov V M  (2000): Historical Isotopic Temperature Record from the Vostok Ice Core. CDIAC.
}
\description{
Retrieves historical carbon dioxide and temperature records from the Vostok ice core during the past 420,000 years.
Source of data is the U.S. Department of Energy’s (DOE) Environmental System Science Data Infrastructure for a Virtual Ecosystem (ESS-DIVE).
\url{https://ess-dive.lbl.gov/} and \url{https://data.ess-dive.lbl.gov/datasets/doi:10.3334/CDIAC/CLI.006}
}
\examples{
\donttest{
# Fetch Vostok paleo carbon and temperature data from cache if available:
vostok <- get_paleo()
#
# Force cache refresh:
vostok <- get_paleo(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_paleo(vostok) }

}
\references{
\itemize{
\item Historical Carbon Dioxide Record from the Vostok Ice Core (US Dept of Energy): \url{https://data.ess-dive.lbl.gov/datasets/doi:10.3334/CDIAC/CLI.006}
\item Petit J R ; Raynaud D ; Lorius C ; Delaygue G ; Jouzel J ; Barkov N I ; Kotlyakov V M  (2000): Historical Isotopic Temperature Record from the Vostok Ice Core. CDIAC. doi:10.3334/CDIAC/CLI.006. \url{https://data.ess-dive.lbl.gov/view/doi:10.3334/CDIAC/CLI.006}
\item Barnola J ; Raynaud D ; Lorius C ; Barkov N  (2003): Historical Carbon Dioxide Record from the Vostok Ice Core (417,160 - 2,342 years BP). None. doi:10.3334/CDIAC/ATG.009 \url{https://data.ess-dive.lbl.gov/view/doi:10.3334/CDIAC/ATG.009}
}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hurricanes.R
\name{plot_hurricanes}
\alias{plot_hurricanes}
\title{Download and plot essential climate data}
\usage{
plot_hurricanes(dataset = get_hurricanes(), cat = "major", print = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_hurricanes}}

\item{cat}{(string) Select which category of hurricane to plot. May be "major", "hurricane", or "storm". Defaults to "major".}

\item{print}{(boolean) Display hurricane ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with hurricanes chart
}
\description{
Plots the hurricane data retrieved using \code{get_hurricanes()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_hurricanes} invisibly returns a ggplot2 object with a pre-defined hurricane data chart using data from \code{get_hurricanes}.
By default the chart is also displayed. Smooths using ggplot2's built-in loess smoother. Users may further modify the output ggplot2 chart.
Categories may be "major" (category 3-5), "hurricane" (category 1-5), or "storm" (named storm).
}
\examples{
\donttest{
# Fetch hurricane data:
hurricanes <- get_hurricanes()
#
# Plot output using package's built-in ggplot2 defaults
plot_hurricanes(hurricanes)

# Or just call plot_hurricanes(), which defaults to get_hurricanes() dataset
plot_hurricanes()

p <- plot_hurricanes(hurricanes, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Growing number of North Atlantic named storms') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daily_temp.R
\name{get_dailytemp}
\alias{get_dailytemp}
\title{Download and plot essential climate data}
\usage{
get_dailytemp(
  use_cache = TRUE,
  write_cache = getOption("hs_write_cache"),
  region = "W",
  mean_start = if (region \%in\% c("WS", "NS", "ws", "ns")) 1982 else 1979,
  mean_end = 2000
)
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}

\item{region}{(string) Region selection, defaults to world air temperature. Options are: World Air "W", Northern Hemisphere Air "NW", Southern Hemisphere Air "SW", Tropics Air "TR", Arctic Air "AR", Antarctica Air "AN", World Sea Surface "WS", and North Atlantic Sea Surface "NS".}

\item{mean_start}{(numeric) Start year for historic mean, defaults to 1979.}

\item{mean_end}{(numeric) End year for historic mean, defaults to 2000.}
}
\value{
Invisibly returns a tibble with the daily 2-meter air or sea surface temperatures since 1940 as well as historic mean by day-of-year and current anomaly versus mean.

\code{get_dailytemp} invisibly returns a tibble with the daily temperatures since 1940 as well as mean by day-of-year and anomaly. Default to world data, but region can be selected among six options.

Region options include world air (default), Northern Hemisphere air, Southern Hemisphere air, Tropics air, Arctic air, Antarctic air, World sea surface and North Atlantic sea surface and is stored in attribute of output.
The historic daily mean-by-day period defaults to 1979-2000. This range can be optionally modified.

Data are updated daily. For day-of-year mean removes observations from February 29 on leap years.
}
\description{
Retrieves the daily air or sea-surface temperature data since 1940 from ClimateReanalyzer.org
Source is University of Maine Climate Change Institute.
\url{https://climatereanalyzer.org/clim/t2_daily/}
}
\examples{
\donttest{
# Fetch temp anomaly from cache if available:
dailytemps <- get_dailytemp()
#
# Force cache refresh:
dailytemps <- get_dailytemp(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_dailytemp(dailytemps)

# Change region to Arctic
arctictemp <- get_dailytemp(region='AR', use_cache=FALSE)}

}
\references{
\itemize{
\item ClimateReanalyzer.org: \url{https://climatereanalyzer.org/clim/t2_daily/}

Notes: daily mean surface air temperature (2-meter height) estimates from the ECMWF Reanalysis version 5 (ERA5) for the period January 1940 to present. ERA5 is a state-of-the-art numerical climate/weather modeling framework that ingests surface, radiosonde, and satellite observations to estimate the state of the atmosphere through time.
ERA5 files have a horizontal grid resolution of 0.25° x 0.25° (about 31km x 31km at 45°N). Each daily temperature represents an average across all model gridcells within the defined latitude/longitude bounds for the selected domain. The means are area-weighted to account for the convergence of longitude lines at the poles

}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fires.R
\name{plot_fires_area}
\alias{plot_fires_area}
\title{Download and plot essential climate data}
\usage{
plot_fires_area(
  dataset = get_fires_area(),
  var = c("area", "count"),
  style = c("cumulative", "weekly"),
  print = TRUE
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_fires_area}}

\item{var}{(string) Select which variable to plot. May be \code{"area"} (burnt area in hectares, the default)
or \code{"count"} (number of fire events).}

\item{style}{(string) Select cumulative or weekly view. May be \code{"cumulative"} (year-to-date totals,
the default) or \code{"weekly"} (weekly values).}

\item{print}{(boolean) Display the ggplot2 chart, defaults to TRUE. Use FALSE to suppress display.}
}
\value{
Invisibly returns a ggplot2 object with the wildfire chart
}
\description{
Plots the wildfire burnt area or fire count data retrieved using \code{get_fires_area()} with ggplot2.
The output ggplot2 object may be further modified.
}
\details{
\code{plot_fires_area} invisibly returns a ggplot2 object with a pre-defined wildfire chart using
data from \code{get_fires_area}. By default the chart is also displayed. The shaded ribbon shows the
historical minimum-to-maximum range since 2012, and the blue line shows the historical average.
Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch wildfire data:
fires <- get_fires_area()
#
# Plot output using package's built-in ggplot2 defaults
plot_fires_area(fires)

# Or just call plot_fires_area(), which defaults to get_fires_area() dataset
plot_fires_area()

# Plot weekly fire count instead of cumulative burnt area
plot_fires_area(fires, var = 'count', style = 'weekly')

p <- plot_fires_area(fires, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Global Wildfire Burnt Area') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extra_viz.R
\name{merge_carbontemp}
\alias{merge_carbontemp}
\title{Download and plot essential climate data}
\usage{
merge_carbontemp(carbon = get_carbon(), temp = get_temp())
}
\arguments{
\item{carbon}{Name of the tibble generated by \code{get_carbon}}

\item{temp}{Name of the tibble generated by \code{get_temp}}
}
\value{
Invisibly returns a tibble with merged datasets from \code{get_carbon} and \code{get_temp} functions.
}
\description{
Merge NOAA carbon and NASA temperature datasets on common dates.
}
\details{
\code{merge_carbontemp} invisibly returns a tibble with the merged data from from \code{get_carbon} and \code{get_temp} functions.
Tibble only includes data from dates when both datasets are available, essentially from 1960.
}
\examples{
\donttest{
# Create merged tibble

mergedcarbontemp <- merge_carbontemp() }


}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_cache.R
\name{print.hockeystick_cache_info}
\alias{print.hockeystick_cache_info}
\title{Display data cache info
Shows filenames and cache file sizes}
\usage{
\method{print}{hockeystick_cache_info}(x, ...)
}
\arguments{
\item{x}{filenames}

\item{...}{Additional parameters}
}
\value{
Display data cache info. No return value, called for side effect.
}
\description{
Display data cache info
Shows filenames and cache file sizes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/carbon.R
\name{plot_carbon}
\alias{plot_carbon}
\title{Download and plot essential climate data}
\usage{
plot_carbon(dataset = get_carbon(), print = TRUE, annot = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_carbon}}

\item{print}{(boolean) Display carbon dioxide ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}

\item{annot}{(boolean) Include chart annotation with latest date and value, defaults to TRUE.}
}
\value{
Invisibly returns a ggplot2 object with carbon dioxide chart
}
\description{
Plots carbon dioxide data retrieved using \code{get_carbon()} with ggplot2. The output ggplot2 object may be modified.
}
\details{
\code{plot_carbon} invisibly returns a ggplot2 object with a pre-defined carbon dioxide chart using data from \code{get_carbon}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch carbon dioxide data:
maunaloa <- get_carbon()
#
# Plot output using package's built-in ggplot2 defaults
plot_carbon(maunaloa)

# Or just call plot_carbon(), which defaults to get_carbon() dataset
plot_carbon()

p <- plot_carbon(maunaloa, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='The Keeling Curve') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hurricanes.R
\name{get_hurricanes}
\alias{get_hurricanes}
\title{Download and plot essential climate data}
\usage{
get_hurricanes(use_cache = TRUE, write_cache = getOption("hs_write_cache"))
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with the annual HURDAT hurricane data since 1851
}
\description{
Retrieves Atlantic basin hurricane data since 1851 from National Oceanic and Atmospheric Administration
HURDAT Atlantic Hurricane Database Re-analysis Project.
\url{https://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html}
}
\details{
\code{get_hurricanes} invisibly returns a tibble with NOAA's annual North Atlantic revised HURDAT hurricane data since 1851.
The returned object includes Year, and number of named storms, hurricanes, major hurricanes (S-S scale 3-5), Accumulated Cyclone Energy (ACE), and U.S. hurricane strikes.

ACE is an index that combines the number of systems, how long they existed and how intense they became. It is calculated by squaring the maximum sustained surface wind in the system every six hours that the cyclone is a Named Storm and summing it up for the season. Please refer to above website for details.
}
\examples{
\donttest{
# Fetch from cache if available:
hurricanes <- get_hurricanes()
#
# Force cache refresh:
hurricanes <- get_hurricanes(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_hurricanes(hurricanes) }

}
\references{
HURDAT North Atlantic Hurricane Database Re-analysis Project, Hurricane Research Division, NOAA \url{https://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html}.

Data from: \url{https://www.aoml.noaa.gov/hrd/hurdat/comparison_table.html}

\url{https://en.wikipedia.org/wiki/Accumulated_cyclone_energy}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/future_climate.R
\name{plot_cmip6_anom}
\alias{plot_cmip6_anom}
\title{Download and plot essential climate data}
\usage{
plot_cmip6_anom(
  dataset = get_cmip6_anom(),
  zero_centered = FALSE,
  xlim = NULL,
  ylim = NULL,
  country = NULL,
  state = NULL,
  mask = TRUE,
  print = TRUE
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_cmip6_anom}}

\item{zero_centered}{(boolean) Use a symmetric diverging scale around zero,
defaults to FALSE (scale runs from zero up).}

\item{xlim, ylim}{(numeric) Optional two-element longitude/latitude bounds to
zoom the map to a region, e.g. \code{xlim = c(-130, -60), ylim = c(20, 70)}.
The color scale is recomputed on the visible region.}

\item{country}{(string) Optional country name or ISO 3166-1 alpha-3 code to
zoom the map to that country. Requires the \code{geodata} package.}

\item{state}{(string) Optional state/province name (admin-1), used together
with \code{country} to zoom to a subnational region, e.g.
\code{country = "MEX", state = "Sonora"}. Requires the \code{geodata} package.}

\item{mask}{(boolean) If \code{country} is given, blank cells outside the
country border, defaults to TRUE.}

\item{print}{(boolean) Display map, defaults to TRUE.}
}
\value{
Invisibly returns a ggplot2 object with the change map.
}
\description{
Plots expected climate change (projection minus 1970-2000 baseline) computed by
\code{get_cmip6_anom()} with a diverging color scale centered on zero.
}
\details{
\code{plot_cmip6_anom} returns a pre-defined ggplot2 world map of expected
change in degrees C (or percent for precipitation variables), with blue for
cooling/drying and red for warming/wetting. Precipitation maps with both
increases and decreases automatically use the diverging scale even when
\code{zero_centered = FALSE}. Users may further modify the output chart.
}
\examples{
\donttest{
plot_cmip6_anom()
}

}
\references{
See \code{\link{get_cmip6}}.
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paleo.R
\name{plot_paleo}
\alias{plot_paleo}
\title{Download and plot essential climate data}
\usage{
plot_paleo(dataset = get_paleo(), print = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_paleo}, defaults to calling \code{get_paleo}}

\item{print}{(boolean) Display Vostok ice core ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with the Vostok chart
}
\description{
Plots the Vostok ice core data retrieved using \code{get_paleo()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_paleo} invisibly returns a ggplot2 object with a pre-defined Vostok ice core chart using data from \code{get_paleo}.
The returned chart stacks carbon dioxide concentration over temperature over 420,000 years.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch Vostok data:
vostok <- get_paleo()
#
# Plot output using package's built-in ggplot2 defaults
plot_paleo(vostok)

# Or just call plot_paleo(), which defaults to get_paleo() dataset
plot_paleo()

p <- plot_paleo(vostok, print = FALSE)
# Modify plot such as: p + patchwork::plot_annotation(title='A Long History of Carbon') }

}
\references{
Barnola, J.-M., D. Raynaud, C. Lorius, and N.I. Barkov. 2003. Historical CO2 record from the Vostok ice core. In Trends: A Compendium of Data on Global Change. Carbon Dioxide Information Analysis Center, Oak Ridge National Laboratory, U.S. Department of Energy, Oak Ridge, Tenn., U.S.A
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seaice.R
\name{get_seaice}
\alias{get_seaice}
\title{Download and plot essential climate data}
\usage{
get_seaice(
  pole = "N",
  month = 7,
  measure = "extent",
  use_cache = TRUE,
  write_cache = getOption("hs_write_cache")
)
}
\arguments{
\item{pole}{'N' for Arctic or 'S' for Antarctic}

\item{month}{2-digit month to retrieve sea ice for, defaults to 7 (July)}

\item{measure}{Must be 'extent' or 'area', defaults to 'extent'. Please see terminology link in references for details.}

\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data, or to change pole or month in cache.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with the annual series of monthly Sea Ice Index since 1979 (in million square km).

\code{get_seaice} invisibly returns a tibble with annual series of monthly Sea Ice Index since 1979 (in million square km).

User may select Arctic or Antarctic sea ice extent or area (in millions of square kilometers) by year for a given month, specified by argument \code{month}.
Defaults to Arctic July sea ice extent.  \url{https://nsidc.org/sea-ice-today/about-data#area_extent}
}
\description{
Retrieves Arctic or Antarctic annual Sea Ice Index (in million square km).
Source is the National Snow and Ice Data Center, defaults to Arctic (Northern Hemisphere) July sea ice extent.
\url{https://nsidc.org/data/explore-data} \cr \cr
}
\examples{
\donttest{
# Fetch sea ice from cache if available:
seaice <- get_seaice()
#
# Force cache refresh:
seaice <- get_seaice(use_cache = FALSE)
# change region and month
seaice <- get_seaice(pole='S', month=9, use_cache = FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_seaice(seaice) }

}
\references{
\itemize{
\item NSIDC Data Archive: \url{https://nsidc.org/data/explore-data}
\item All About Sea Ice: \url{https://nsidc.org/learn/parts-cryosphere/sea-ice}
\item Sea Ice terminology: \url{https://nsidc.org/learn}
}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extra_viz.R
\name{warming_stripes}
\alias{warming_stripes}
\title{Download and plot essential climate data}
\usage{
warming_stripes(
  dataset = get_temp(),
  stripe_only = FALSE,
  col_strip = RColorBrewer::brewer.pal(11, "RdBu"),
  print = TRUE
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_temp}}

\item{stripe_only}{Display legend and axes, defaults to TRUE}

\item{col_strip}{Color palette to use.  Defaults to Red-Blue RColorBrewer palette.}

\item{print}{(boolean) Display warming stripe ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with warming stripes
}
\description{
Plots global "warming stripes" graph in the style popularized by Ed Hawkins, based on temperature anomaly retrieved using \code{get_temp()}.
Function can output stripes chart with legend or a minimal chart. The output ggplot2 object may be further modified.
}
\details{
\code{warming_stripes} invisibly returns a ggplot2 object with warming stripes chart using data from \code{get_temp}.
By default the chart is also displayed. User may modify color palette or remove axes and legend. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Draw with axes and legend
stripes <- warming_stripes()

# Draw stripes only
stripes <- warming_stripes(stripe_only = TRUE)

# Don't display, store for further modifications
stripes <- warming_stripes(print = FALSE)

# Change color palette
stripes <- warming_stripes(stripe_only = TRUE, col_strip = viridisLite::viridis(11)) }

}
\references{
\itemize{
\item Climate Lab. 2018. https://www.climate-lab-book.ac.uk/2018/warming-stripes/
\item GISS Surface Temperature Analysis (GISTEMP v4): \url{https://data.giss.nasa.gov/gistemp/}
\item GISTEMP Team, 2020: \emph{GISS Surface Temperature Analysis (GISTEMP), version 4.} NASA Goddard Institute for Space Studies.
\item Dr. Dominic Roye blog post "How to Create Warming Stripes in R": https://dominicroye.github.io/en/2018/how-to-create-warming-stripes-in-r/
}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fires.R
\name{plot_fires_emissions}
\alias{plot_fires_emissions}
\title{Download and plot essential climate data}
\usage{
plot_fires_emissions(
  dataset = get_fires_emissions(),
  pollutant = c("CO2", "BC", "C", "CH4", "CO", "NMHC", "NOX", "OC", "PM2.5", "SO2", "TC"),
  style = c("cumulative", "weekly"),
  print = TRUE
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_fires_emissions}}

\item{pollutant}{(string) Select which pollutant to plot. May be one of \code{"CO2"} (the default),
\code{"BC"}, \code{"C"}, \code{"CH4"}, \code{"CO"}, \code{"NMHC"}, \code{"NOX"},
\code{"OC"}, \code{"PM2.5"}, \code{"SO2"}, or \code{"TC"}.}

\item{style}{(string) Select cumulative or weekly view. May be \code{"cumulative"} (year-to-date totals,
the default) or \code{"weekly"} (weekly values).}

\item{print}{(boolean) Display the ggplot2 chart, defaults to TRUE. Use FALSE to suppress display.}
}
\value{
Invisibly returns a ggplot2 object with the wildfire emissions chart
}
\description{
Plots the wildfire greenhouse gas emissions data retrieved using \code{get_fires_emissions()} with ggplot2.
The output ggplot2 object may be further modified.
}
\details{
\code{plot_fires_emissions} invisibly returns a ggplot2 object with a pre-defined wildfire
emissions chart using data from \code{get_fires_emissions}. By default the chart is also displayed.
The shaded ribbon shows the historical minimum-to-maximum range since 2003, and the blue line
shows the historical average. Emissions are expressed in millions of tons. Users may further
modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch wildfire emissions data:
emissions <- get_fires_emissions()
#
# Plot output using package's built-in ggplot2 defaults
plot_fires_emissions(emissions)

# Or just call plot_fires_emissions(), which defaults to get_fires_emissions() dataset
plot_fires_emissions()

# Plot weekly PM2.5 emissions instead of cumulative CO2
plot_fires_emissions(emissions, pollutant = 'PM2.5', style = 'weekly')

p <- plot_fires_emissions(emissions, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Global CO2 Emissions from Wildfires') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extra_viz.R
\name{plot_carbontemp}
\alias{plot_carbontemp}
\title{Download and plot essential climate data}
\usage{
plot_carbontemp(dataset = merge_carbontemp(), print = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{merge_carbontemp}}

\item{print}{(boolean) Display temperature anomaly ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with temperature anomaly vs carbon chart
}
\description{
Plots the global monthly mean temperature anomaly vs atmospheric carbon  with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_carbontemp} invisibly returns a ggplot2 object with a pre-defined temperature anomaly vs carbon chart using data from \code{merge_carbontemp}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch temperature anomaly:
mergedtemp <- merge_carbontemp()
#
# Plot output using package's built-in ggplot2 defaults
plot_carbontemp(mergedtemp)

# Or just call plot_carbontemp(), which defaults to merge_carbontemp() dataset
plot_carbontemp()

p <- plot_carbontemp(mergedtemp, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='The Signature of Climate Change') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_cache.R
\name{file_info_}
\alias{file_info_}
\title{Display cached file info}
\usage{
file_info_(x)
}
\arguments{
\item{x}{filenames}
}
\description{
Internal function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrumental_temp.R
\name{get_temp}
\alias{get_temp}
\title{Download and plot essential climate data}
\usage{
get_temp(use_cache = TRUE, write_cache = getOption("hs_write_cache"))
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with the annual mean and monthly Combined Land-Surface Air and Sea-Surface Water Temperature Anomalies.

\code{get_temp} invisibly returns a tibble with the NASA/GISS annual mean and monthly global temperature anomaly. Data are global from 1880 to present, and represent the deviations from the 1951-1980 mean.

The returned object includes monthly and annual average anomalies, as well as seasonal anomalies.
GISS Surface Temperature Analysis (GISTEMP v4) is an estimate of global surface temperature change.

Data are updated around the middle of every month using current data files from NOAA GHCN v4 (meteorological stations) and ERSST v5 (ocean areas).
Station data are combined as described in Hansen et al. (2010) \url{https://data.giss.nasa.gov/gistemp/references.html} and Lenssen et al. (2019) \url{https://www.giss.nasa.gov/pubs/abs/le05800h.html}
}
\description{
Retrieves the combined global land- and sea-surface temperature anomaly (Land-Ocean Temperature Index, LOTI).
Source is NASA/GISS Surface Temperature Analysis (GISTEMP v4), an estimate of global surface temperature change.
\url{https://data.giss.nasa.gov/gistemp/}
}
\examples{
\donttest{
# Fetch temp anomaly from cache if available:
anomaly <- get_temp()
#
# Force cache refresh:
anomaly <- get_temp(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_temp(anomaly) }

}
\references{
\itemize{
\item GISS Surface Temperature Analysis (GISTEMP v4): \url{https://data.giss.nasa.gov/gistemp/}
\item GISTEMP Team, 2020: \emph{GISS Surface Temperature Analysis (GISTEMP), version 4.} NASA Goddard Institute for Space Studies.
\item Lenssen, N., G. Schmidt, J. Hansen, M. Menne, A. Persin, R. Ruedy, and D. Zyss, 2019:  Improvements in the GISTEMP uncertainty model.
\emph{J. Geophys. Res. Atmos.}, 124, no. 12, 6307-6326, doi:10.1029/2018JD029522. \url{https://www.giss.nasa.gov/pubs/abs/le05800h.html}
}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icecurves.R
\name{plot_icecurves}
\alias{plot_icecurves}
\title{Download and plot essential climate data}
\usage{
plot_icecurves(dataset = get_icecurves(), print = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_icecurves}, defaults to calling \code{get_icecurves}}

\item{print}{(boolean) Display sea ice ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with Sea Ice Index chart
}
\description{
Plots the monthly Sea Ice Index data retrieved using \code{get_icecurves()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_icecurves} invisibly returns a ggplot2 object with a pre-defined Sea Ice Index chart using data from \code{get_icecurves}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
Chart consists of one line per year showing monthly sea ice from January through December. Current year is highlighted.
}
\examples{
\donttest{
# Fetch historic monthly sea ice data since 1979:
icecurves <- get_icecurves()
#
# Plot output using package's built-in ggplot2 defaults
plot_icecurves(icecurves)

# Or just call plot_icecurves(), which defaults to get_icecurves() dataset
plot_icecurves()

p <- plot_icecurves(icecurves, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Shrinking Arctic Sea Ice') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/carbon.R
\name{get_carbon}
\alias{get_carbon}
\title{Download and plot essential climate data}
\usage{
get_carbon(use_cache = TRUE, write_cache = getOption("hs_write_cache"))
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with the monthly carbon dioxide series
}
\description{
Retrieves atmospheric carbon dioxide measurements from National Oceanic and Atmospheric Administration
Earth System Research Laboratories monitoring laboratory in Mauna Loa, Hawaii.
\url{https://gml.noaa.gov/ccgg/trends/data.html}
}
\details{
\code{get_carbon} invisibly returns a tibble with NOAA's monthly average carbon dioxide measurement.
The returned object includes date, year, month, average, trend, std dev, and uncertainty columns.
Trend is NOAA's published trend.  Please refer to above website for details.
}
\examples{
\donttest{
# Fetch from cache if available, otherwise download:
maunaloa <- get_carbon()
#
# Force fetch from source:
maunaloa <- get_carbon(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_carbon(maunaloa) }

}
\references{
Dr. Pieter Tans, NOAA/GML \url{https://gml.noaa.gov/ccgg/trends/} and Dr. Ralph Keeling, Scripps Institution of Oceanography \url{https://scrippsco2.ucsd.edu/}.

C.D. Keeling, R.B. Bacastow, A.E. Bainbridge, C.A. Ekdahl, P.R. Guenther, and L.S. Waterman, (1976), Atmospheric carbon dioxide variations at Mauna Loa Observatory, Hawaii, \emph{Tellus}, vol. 28, 538-551
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seaice.R
\name{plot_seaice}
\alias{plot_seaice}
\title{Download and plot essential climate data}
\usage{
plot_seaice(dataset = get_seaice(), title = "Arctic Sea Ice", print = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_seaice}, defaults to calling \code{get_seaice}}

\item{title}{chart title, defaults to Arctic Sea Ice}

\item{print}{(boolean) Display sea ice ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with Sea Ice Index chart
}
\description{
Plots the Sea Ice Index data retrieved using \code{get_seaice()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_seaice} invisibly returns a ggplot2 object with a pre-defined Sea Ice Index chart using data from \code{get_seaice}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch sea ice data:
seaice <- get_seaice()
#
# Plot output using package's built-in ggplot2 defaults
plot_seaice(seaice)

# Or just call plot_seaice(), which defaults to get_seaice() dataset
plot_seaice()

p <- plot_seaice(seaice, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Shrinking Arctic Sea Ice') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2kyears_temp.R
\name{get_temp2k}
\alias{get_temp2k}
\title{Download and plot essential climate data}
\usage{
get_temp2k(use_cache = TRUE, write_cache = getOption("hs_write_cache"))
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with filtered and unfiltered temperature reconstructions and Cowtan & Way instrumental temperatures.

\code{get_temp2k} invisibly returns a tibble with the PAGES2k Consortium temperature reconstruction (years 1-2000 CE) and instrumental record (years 1850-2017 CE).  Temperatures represent deviations from the 1961-1990 mean.

The returned object includes annual average temperature anomalies as well as filtered anomalies using a 31-year Butterworth filter.
Reconstructions use seven different statistical methods that draw from a global collection of temperature-sensitive palaeoclimate records.

Methodology described in PAGES2k (2019) \url{https://www.nature.com/articles/s41561-019-0400-0}
}
\description{
Retrieves the Common Era Global Surface Temperature Reconstructions.
Source is PAGES2k Consortium and NOAA National Centers for Environmental Information.
\url{https://www.ncei.noaa.gov/access/paleo-search/study/26872}
}
\examples{
\donttest{
# Fetch temp anomaly from cache if available:
anomaly <- get_temp2k()
#
# Force cache refresh:
anomaly <- get_temp2k(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_temp2k(anomaly) }

}
\references{
\itemize{
\item PAGES2k Common Era Surface Temperature Reconstructions. \url{https://www.ncei.noaa.gov/access/paleo-search/study/26872}
\item PAGES 2k Consortium., Neukom, R., Barboza, L.A. et al. Consistent multidecadal variability in global temperature reconstructions and simulations over the Common Era. \emph{Nat. Geosci.} 12, 643–649 (2019).  \doi{10.1038/s41561-019-0400-0}
}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emissions.R
\name{plot_emissions_with_land}
\alias{plot_emissions_with_land}
\title{Download and plot essential climate data}
\usage{
plot_emissions_with_land(
  dataset = get_emissions(),
  start_year = 1900,
  region = "World",
  print = TRUE,
  annot = TRUE,
  title_expression = expression("Fossil + Land Use Change " * CO[2] * " Emissions"),
  yaxis_expression = expression("Gt " * CO[2] * " per year")
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_emissions}}

\item{start_year}{Year to start plot at. Defaults to 1900. Data is available since 1750.}

\item{region}{ISO code of region to plot. Defaults to 'OWID_WRL' which signifies entire world.}

\item{print}{(boolean) Display carbon dioxide emissions ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}

\item{annot}{(boolean) Include chart annotation with latest date and value, defaults to TRUE.}

\item{title_expression}{Chart title, defaults to CO2 emissions}

\item{yaxis_expression}{y-axis label, defaults to Gt CO2 emissions}
}
\value{
Invisibly returns a ggplot2 object with carbon dioxide emissions chart
}
\description{
Plots carbon dioxide emissions (including land use change) retrieved using \code{get_emissions()} with ggplot2. The output ggplot2 object may be modified. Alternative columns from the dataset may also be plotted.
}
\details{
\code{plot_emissions_with_land} invisibly returns a ggplot2 object with a pre-defined carbon dioxide emissions (including land use change) chart using data from \code{get_emissions}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch carbon dioxide emissions:
emissions <- get_emissions()

# Plot output (including land use change) using package's built-in ggplot2 defaults
plot_emissions_with_land(emissions)

# Or just call plot_emissions_with_land(), which defaults to get_emissions() dataset
plot_emissions_with_land()

# You can also select a region by name and starting year
plot_emissions_with_land(region='United States', start_year=1950)
p <- plot_emissions_with_land(emissions, print = FALSE)

# Modify plot such as: p + ggplot2::labs(title='Anthropogenic Carbon Emissions')}


}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extra_viz.R
\name{climate_grid}
\alias{climate_grid}
\title{Download and plot essential climate data}
\usage{
climate_grid(print = TRUE)
}
\arguments{
\item{print}{(boolean) Display climate grid ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with climate grid
}
\description{
Plots a 2x2 grid of carbon, temperature, sea ice, and sea level charts.
}
\details{
\code{warming_stripes} invisibly returns a ggplot2 object with 2x2 grid of carbon, temperature, sea ice, and sea level charts from \code{get_carbon}, \code{get_temp}, \code{get_seaice}, and \code{get_sealevel}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Draw grid

grid <- climate_grid() }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/future_climate.R
\name{get_cmip6}
\alias{get_cmip6}
\title{Download and plot essential climate data}
\usage{
get_cmip6(
  var = "bioc",
  bio = 1,
  ssp = "585",
  period = "2081-2100",
  model = "MPI-ESM1-2-HR",
  res = "10m",
  use_cache = TRUE,
  write_cache = getOption("hs_write_cache")
)
}
\arguments{
\item{var}{(string) Climate variable. One of \code{"bioc"} (bioclimatic variables,
the default), \code{"tmin"}, \code{"tmax"}, or \code{"prec"} (precipitation).}

\item{bio}{(numeric) If \code{var = "bioc"}, which bioclimatic variable to return
(1-19). Defaults to 1 (Annual Mean Temperature). The variables are:
1 = Annual Mean Temperature; 2 = Mean Diurnal Range; 3 = Isothermality (BIO2/BIO7);
4 = Temperature Seasonality (standard deviation); 5 = Max Temperature of Warmest Month;
6 = Min Temperature of Coldest Month; 7 = Temperature Annual Range (BIO5-BIO6);
8 = Mean Temperature of Wettest Quarter; 9 = Mean Temperature of Driest Quarter;
10 = Mean Temperature of Warmest Quarter; 11 = Mean Temperature of Coldest Quarter;
12 = Annual Precipitation; 13 = Precipitation of Wettest Month; 14 = Precipitation of Driest Month;
15 = Precipitation Seasonality (coefficient of variation); 16 = Precipitation of Wettest Quarter;
17 = Precipitation of Driest Quarter; 18 = Precipitation of Warmest Quarter;
19 = Precipitation of Coldest Quarter. Variables 1-11 are in degrees C
(seasonality variables 4 and 7 as applicable), 12-19 in mm. For the monthly
variables (\code{var = "prec"}, \code{"tmin"}, or \code{"tmax"}), \code{bio}
instead selects the calendar month, 1-12 (1 = January).}

\item{ssp}{(string) Shared Socioeconomic Pathway scenario. One of "126", "245",
"370", or "585" (the default). SSP5-8.5 is the high-emissions upper bound and
shows the strongest projected change; SSP2-4.5 is a middle-of-the-road
scenario roughly consistent with current policies ("current trends continue").
SSP1-2.6 assumes strong mitigation, and SSP3-7.0 is a high fragmented-world path.}

\item{period}{(string) Projection period. One of "2021-2040", "2041-2060",
"2061-2080", or "2081-2100" (the default).}

\item{model}{(string) CMIP6 global climate model. Defaults to "MPI-ESM1-2-HR".
Availability of variable/scenario combinations varies by model; the function
checks the WorldClim index and reports what is available.}

\item{res}{(string) Resolution: "10m" (~18 km, default), "5m" (~9 km) or "2.5m"
(~5 km). Finer resolutions are considerably larger downloads.}

\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE.}
}
\value{
Invisibly returns a tibble with columns \code{lon}, \code{lat}, and
\code{value}: the projected values aggregated to a one-degree grid.
}
\description{
Retrieves downscaled CMIP6 climate projection data from WorldClim
(\url{https://www.worldclim.org/data/cmip6/}). Data are monthly or bioclimatic
variables projected by a selected global climate model (GCM) under an SSP
scenario for one of four 20-year periods (2021-2100), at 10 arc-minute resolution.
}
\details{
\code{get_cmip6} downloads a WorldClim CMIP6 GeoTIFF, aggregates it to a
one-degree grid (averaging), and returns a tibble suitable for mapping.
Requires the \code{terra} package (installed automatically when needed).
}
\examples{
\donttest{
# Projected annual mean temperature under SSP585, 2081-2100:
proj <- get_cmip6(var='bioc', bio=1)
plot_cmip6(proj)
#
# Projected precipitation change under a moderate scenario:
prec <- get_cmip6(var='prec', ssp=245, period='2041-2060', use_cache=FALSE)
}

}
\references{
\itemize{
\item WorldClim CMIP6 future climate data: \url{https://www.worldclim.org/data/cmip6/}
\item Fick, S.E. & Hijmans, R.J. (2017) WorldClim 2: new 1km spatial resolution climate
surfaces for global land areas. \emph{Int. J. Climatol.} 37: 4302-4315.
}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sealevel.R
\name{get_sealevel}
\alias{get_sealevel}
\title{Download and plot essential climate data}
\usage{
get_sealevel(use_cache = TRUE, write_cache = getOption("hs_write_cache"))
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with the global mean sea level time series (in mm) over 1880-2013 using tide gauges and since 1993 for satellite measurements.

\code{get_sealevel} invisibly returns a tibble with mean sea level in mm time series from tide gauges and satellite observations.

The satellite observations have been releveled so that their mean level in 1993 matches that of the tide gauges.

The tide gauge data are no longer updated and cover the period from 1880 to 2013, per Church, J. A. and N.J. White (2011) \url{https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/}

Satellite data are updated monthly or more frequently from the NOAA Laboratory for Satellite Altimetry. TOPEX and Jason-1,-2,-3 satellites dataset, with seasonal signals removed. \url{https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/}
}
\description{
Retrieves global mean sea level (GMSL) data from historic tide gauge and recent satellite altimeter observations (in mm).
Source for tide gauge data is Commonwealth Scientific and Industrial Research Organisation (CSIRO) as described in Church and White (2011).
\url{https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/} \cr \cr
Source for satellite data is NOAA Laboratory for Satellite Altimetry:
\url{https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/}
}
\examples{
\donttest{
# Fetch sea level from cache if available:
gmsl <- get_sealevel()
#
# Force cache refresh:
gmsl <- get_sealevel(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_sealevel(gmsl) }

}
\references{
\itemize{
\item CSIRO reconstructed tide gauge GMSL for 1880 to 2009: \url{https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/}
\item Church, J. A. and N.J. White (2011), Sea-level rise from the late 19th to the early 21st Century. \emph{Surveys in Geophysics}, doi:10.1007/s10712-011-9119-1. \url{https://link.springer.com/article/10.1007/s10712-011-9119-1}
\item NOAA Laboratory for Satellite Altimetry \url{https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/}
}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sealevel.R
\name{plot_sealevel}
\alias{plot_sealevel}
\title{Download and plot essential climate data}
\usage{
plot_sealevel(dataset = get_sealevel(), print = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_sealevel}, defaults to calling \code{get_sealevel}}

\item{print}{(boolean) Display sealevel ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with sealevel chart
}
\description{
Plots the global mean sea level data retrieved using \code{get_sealevel()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_sealevel} invisibly returns a ggplot2 object with a pre-defined sealevel change chart using data from \code{get_sealevel}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch sealevel data:
gmsl <- get_sealevel()
#
# Plot output using package's built-in ggplot2 defaults
plot_sealevel(gmsl)

# Or just call plot_sealevel(), which defaults to get_sealevel() dataset
plot_sealevel()

p <- plot_sealevel(gmsl, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Rising Waters') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_cache.R
\name{getsize}
\alias{getsize}
\title{Get rounded size of file in kB}
\usage{
getsize(x)
}
\arguments{
\item{x}{filenames}
}
\description{
Internal function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daily_temp.R
\name{get_dailytempcop}
\alias{get_dailytempcop}
\title{Download and plot essential climate data}
\usage{
get_dailytempcop(
  use_cache = TRUE,
  write_cache = getOption("hs_write_cache"),
  region = "W"
)
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}

\item{region}{(string) Region selection, defaults to world air temperature. Options are: World Air "W".}
}
\value{
Invisibly returns a tibble with the daily 2-meter air temperatures since 1940 as well as historic mean by day-of-year and current anomaly versus mean.

\code{get_dailytempcop} invisibly returns a tibble with the daily temperatures since 1940 as well as mean by day-of-year and anomaly.

Region options include world air (default).
The historic daily mean-by-day period defaults to 1991-2020.

Data are updated daily.
}
\description{
Retrieves the daily air temperature since 1940 from the EU Copernicus Service
\url{https://cds.climate.copernicus.eu/#!/home}
}
\examples{
\donttest{
# Fetch temp anomaly from cache if available:
dailytemps <- get_dailytempcop()
#
# Force cache refresh:
dailytemps <- get_dailytempcop(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_dailytemp(dailytemps) }

}
\references{
\itemize{
\item Copernicus: \url{https://cds.climate.copernicus.eu/#!/home}

Notes: daily mean surface air temperature (2-meter height) estimates from the ECMWF Reanalysis version 5 (ERA5) for the period January 1940 to present. ERA5 is a state-of-the-art numerical climate/weather modeling framework that ingests surface, radiosonde, and satellite observations to estimate the state of the atmosphere through time.
ERA5 files have a horizontal grid resolution of 0.25° x 0.25° (about 31km x 31km at 45°N). Each daily temperature represents an average across all model gridcells within the defined latitude/longitude bounds for the selected domain. The means are area-weighted to account for the convergence of longitude lines at the poles

}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methane.R
\name{plot_methane}
\alias{plot_methane}
\title{Download and plot essential climate data}
\usage{
plot_methane(dataset = get_methane(), print = TRUE, annot = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_methane}}

\item{print}{(boolean) Display methane ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}

\item{annot}{(boolean) Include chart annotation with latest date and value, defaults to TRUE.}
}
\value{
Invisibly returns a ggplot2 object with methane chart
}
\description{
Plots atmospheric methane data retrieved using \code{get_methane()} with ggplot2. The output ggplot2 object may be modified.
}
\details{
\code{plot_methane} invisibly returns a ggplot2 object with a pre-defined methane chart using data from \code{get_methane}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch methane data:
ch4 <- get_methane()
#
# Plot output using package's built-in ggplot2 defaults
plot_methane(ch4)

# Or just call plot_methane(), which defaults to get_methane() dataset
plot_methane()

p <- plot_methane(ch4, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Trend in Atmospheric Methane') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_cache.R
\name{hscache_path}
\alias{hscache_path}
\title{Return path of data cache directory}
\usage{
hscache_path()
}
\value{
Return path of data cache directory
}
\description{
Internal Function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fires.R
\name{get_fires_emissions}
\alias{get_fires_emissions}
\title{Download and plot essential climate data}
\usage{
get_fires_emissions(
  place = "WORLD",
  year = as.numeric(format(Sys.Date(), "\%Y")),
  use_cache = TRUE,
  write_cache = getOption("hs_write_cache")
)
}
\arguments{
\item{place}{(string) Region or country for which to retrieve data. May be one of the
following regional aggregates: \code{"WORLD"} (default), \code{"UN_EUR"} (Europe),
\code{"UN_AFR"} (Africa), \code{"UN_ASI"} (Asia), \code{"UN_AME"} (Americas),
or \code{"UN_OCE"} (Oceania). Alternatively, supply an ISO 3166-1 alpha-3 country
code such as \code{"USA"}, \code{"CAN"}, or \code{"BRA"}.}

\item{year}{(numeric) Four-digit year for which to retrieve data. Must be 2012 or later.
Defaults to the current year.}

\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with weekly and cumulative wildfire emissions data for
the requested place and year. The \code{plt} column identifies the pollutant
(e.g. \code{"CO2"}, \code{"CO"}, \code{"PM2.5"}). Weekly value columns include
\code{curv} (current), \code{minv}, \code{maxv}, and \code{avgv}. Cumulative
columns are prefixed with \code{cum_}.
}
\description{
Retrieves global wildfire greenhouse gas emissions data from the
Global Wildfire Information System (GWIS), operated by the Copernicus
Emergency Management Service.
\url{https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend}
}
\details{
\code{get_fires_emissions} invisibly returns a tibble with weekly and year-to-date
cumulative wildfire emissions by pollutant, sourced from the GWIS emissions API.
Historical minimum, maximum, and average baselines are included for comparison.
Data coverage begins in 2012. Filter on the \code{plt} column to select a specific
pollutant (e.g. \code{filter(plt == "CO2")}). Cache filenames include the place
and year so that multiple queries can be cached independently.
}
\examples{
\donttest{
# Fetch global emissions data for the current year from cache if available:
emissions <- get_fires_emissions()
#
# Fetch CO2 emissions for the USA in 2023:
emissions_usa <- get_fires_emissions(place = 'USA', year = 2023)
#
# Force cache refresh:
emissions <- get_fires_emissions(use_cache = FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details() }

}
\references{
Global Wildfire Information System (GWIS), Copernicus Emergency Management Service / European Commission Joint Research Centre.
\url{https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hurricanes.R
\name{plot_hurricane_nrg}
\alias{plot_hurricane_nrg}
\title{Download and plot essential climate data}
\usage{
plot_hurricane_nrg(dataset = get_hurricanes(), print = TRUE)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_hurricanes}}

\item{print}{(boolean) Display hurricane ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with hurricane energy chart
}
\description{
Plots the hurricane energy data (ACE) retrieved using \code{get_hurricanes()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_hurricane_nrg} invisibly returns a ggplot2 object with a pre-defined hurricane energy data chart using data from \code{get_hurricanes}.
By default the chart is also displayed. Smooths using ggplot2's built-in loess smoother. Users may further modify the output ggplot2 chart.

ACE is an index that combines the number of systems, how long they existed and how intense they became. It is calculated by squaring the maximum sustained surface wind in the system every six hours that the cyclone is a Named Storm and summing it up for the season. Please refer to above website for details.
}
\examples{
\donttest{
# Fetch hurricane data:
hurricanes <- get_hurricanes()
#
# Plot output using package's built-in ggplot2 defaults
plot_hurricane_nrg(hurricanes)

# Or just call plot_hurricane_nrg(), which defaults to get_hurricanes() dataset
plot_hurricane_nrg()

p <- plot_hurricane_nrg(hurricanes, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Accumulated Cyclone Energy') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2kyears_temp.R
\name{plot_temp2k}
\alias{plot_temp2k}
\title{Download and plot essential climate data}
\usage{
plot_temp2k(
  dataset = get_temp2k(),
  instrumental = TRUE,
  filtered = TRUE,
  print = TRUE
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_temp2k}}

\item{instrumental}{(boolean) Include the Cowtan & Way instrumental temperatures through 2017. Defaults to TRUE.}

\item{filtered}{(boolean) Use the filtered temperatures provided by PAGES2k Consortium. Temperatures filtered using a 31-year Butterworth filter. Defaults to TRUE.}

\item{print}{(boolean) Display temperature anomaly ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}
}
\value{
Invisibly returns a ggplot2 object with temperature anomaly chart
}
\description{
Plots the Common Era 2000-year global temperature anomaly retrieved using \code{get_temp2k()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_temp2k} invisibly returns a ggplot2 object with a pre-defined temperature anomaly chart using data from \code{get_temp2k}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch temperature anomaly:
anomaly <- get_temp2k()
#
# Plot output using package's built-in ggplot2 defaults
plot_temp2k(anomaly)

# Or just call plot_temp2k(), which defaults to get_temp2k() dataset
plot_temp2k()

p <- plot_temp2k(anomaly, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='Temperature Increase in the Common Era') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/future_climate.R
\name{plot_cmip6}
\alias{plot_cmip6}
\title{Download and plot essential climate data}
\usage{
plot_cmip6(
  dataset = get_cmip6(),
  palette = "Spectral",
  xlim = NULL,
  ylim = NULL,
  country = NULL,
  state = NULL,
  mask = TRUE,
  print = TRUE
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_cmip6}}

\item{palette}{(string) Color palette name passed to RColorBrewer. Defaults to
"Spectral" reversed for temperature-type variables.}

\item{xlim, ylim}{(numeric) Optional two-element longitude/latitude bounds to
zoom the map to a region, e.g. \code{xlim = c(-130, -60), ylim = c(20, 70)}.}

\item{country}{(string) Optional country name or ISO 3166-1 alpha-3 code to
zoom the map to that country. Requires the \code{geodata} package.}

\item{state}{(string) Optional state/province name (admin-1), used together
with \code{country} to zoom to a subnational region, e.g.
\code{country = "MEX", state = "Sonora"}. Requires the \code{geodata} package.}

\item{mask}{(boolean) If \code{country} is given, blank cells outside the
country border, defaults to TRUE.}

\item{print}{(boolean) Display map, defaults to TRUE.}
}
\value{
Invisibly returns a ggplot2 object with the projection map.
}
\description{
Plots gridded CMIP6 climate projections retrieved using \code{get_cmip6()} with ggplot2.
}
\details{
\code{plot_cmip6} returns a pre-defined ggplot2 world map of projected
values. Regions may be selected by coordinates (\code{xlim}/\code{ylim}),
by country, or by state/province. Users may further modify the output chart.
}
\examples{
\donttest{
proj <- get_cmip6()
plot_cmip6(proj)
#
# Zoom to the contiguous United States. Explicit xlim/ylim override the
# country's bounding box, excluding Alaska and Hawaii from the view:
plot_cmip6(proj, country = 'USA', xlim = c(-125, -66), ylim = c(24, 50))
#
# Or zoom to a single state:
plot_cmip6(proj, country = 'MEX', state = 'Sonora') }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hockeystick-package.R
\docType{package}
\name{hockeystick-package}
\alias{hockeystick}
\alias{hockeystick-package}
\title{hockeystick: Download and Visualize Essential Global Heating Data}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Provides easy access to essential climate change datasets to non-climate experts. Users can download the latest raw data from authoritative sources and view it via pre-defined 'ggplot2' charts. Datasets include atmospheric CO2, methane, emissions, instrumental and proxy temperature records, CMIP6 projections, sea levels, Arctic/Antarctic sea-ice, Hurricanes, Wildfires, and Paleoclimate data. Sources include: NOAA Mauna Loa Laboratory \url{https://gml.noaa.gov/ccgg/trends/data.html}, Global Carbon Project \url{https://www.globalcarbonproject.org/carbonbudget/}, NASA GISTEMP \url{https://data.giss.nasa.gov/gistemp/}, National Snow and Sea Ice Data Center \url{https://nsidc.org/home}, CSIRO \url{https://research.csiro.au/slrwavescoast/sea-level/measurements-and-data/sea-level-data/}, NOAA Laboratory for Satellite Altimetry \url{https://www.star.nesdis.noaa.gov/socd/lsa/SeaLevelRise/} and HURDAT Atlantic Hurricane Database \url{https://www.aoml.noaa.gov/hrd/hurdat/Data_Storm.html}, Vostok Paleo carbon dioxide and temperature data: \doi{10.3334/CDIAC/ATG.009}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://cortinah.github.io/hockeystick/}
  \item \url{https://github.com/cortinah/hockeystick}
  \item Report bugs at \url{https://github.com/cortinah/hockeystick/issues}
}

}
\author{
\strong{Maintainer}: Hernando Cortina \email{hch@alum.mit.edu} (\href{https://orcid.org/0000-0001-6790-4870}{ORCID})

Authors:
\itemize{
  \item Hernando Cortina \email{hch@alum.mit.edu} (\href{https://orcid.org/0000-0001-6790-4870}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_cache.R
\name{hockeystick_cache}
\alias{hockeystick_cache}
\alias{hockeystick_cache_list}
\alias{hockeystick_cache_delete}
\alias{hockeystick_cache_delete_all}
\alias{hockeystick_cache_details}
\alias{hockeystick_update_all}
\title{Manage cached datasets}
\usage{
hockeystick_cache_list()

hockeystick_cache_delete(files, force = TRUE)

hockeystick_cache_delete_all(force = TRUE)

hockeystick_cache_details(files = NULL)

hockeystick_update_all()
}
\arguments{
\item{files}{(character) one or more complete file names}

\item{force}{(logical) Should files be force deleted? Default: \code{TRUE}}
}
\value{
\code{hockeystick_cache_list()} returns a character vector of full path filenames in cache.

\code{hockeystick_cache_delete()} no return value, called for side effect.

\code{hockeystick_cache_delete_all()} no return value, called for side effect.

\code{hockeystick_cache_details()} returns list of filenames and sizes of cached files.

\code{hockeystick_update_all()} updates all datasets and caches them. No return value, called for side effect.
}
\description{
Manage cached datasets
}
\details{
\code{cache_delete} only accepts 1 file name, while \code{cache_delete_all}
doesn't accept any names, but deletes all files. For deleting many
specific files, use \code{cache_delete} in a \code{\link[=lapply]{lapply()}} type call

We cache using \code{\link[tools:R_user_dir]{tools::R_user_dir()}}, find your cache
folder by executing \code{tools::R_user_dir("hockeystick","cache")}
}
\section{Functions}{

\itemize{
\item \code{hockeystick_cache_list()} returns a character vector of full path
file names in cache
\item \code{hockeystick_cache_delete()} deletes one or more files, returns nothing
\item \code{hockeystick_cache_delete_all()} delete all files, returns nothing
\item \code{hockeystick_cache_details()} prints file name and file size of each file, supply with one or more files, or no files (and get details for all available)
\item \code{hockeystick_update_all()} updates the cache with the latest co2, temperature, sea level, and sea ice data.
}
}

\examples{
\donttest{
# list files in cache
hockeystick_cache_list()

# List info for single files
hockeystick_cache_details(files = hockeystick_cache_list()[1])
hockeystick_cache_details(files = hockeystick_cache_list()[2])

# List info for all files
hockeystick_cache_details()

# Delete cached files by name
hockeystick_cache_delete(files = hockeystick_cache_list()[1])

# Update all hockeystick data and place in cache
hockeystick_update_all()

# Delete all cached data
hockeystick_cache_delete_all()
}
}
\references{
Caching data sets: ROpenSci guide to persistent config and data for R packages: https://blog.r-hub.io/2020/03/12/user-preferences/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/future_climate.R
\name{get_cmip6_anom}
\alias{get_cmip6_anom}
\title{Download and plot essential climate data}
\usage{
get_cmip6_anom(
  var = "bioc",
  bio = 1,
  ssp = "585",
  period = "2081-2100",
  model = "MPI-ESM1-2-HR",
  res = "10m",
  use_cache = TRUE,
  write_cache = getOption("hs_write_cache")
)
}
\arguments{
\item{var, bio, ssp, period, model, res}{Passed through to \code{\link{get_cmip6}}.}

\item{use_cache}{(boolean) Return cached anomaly data if available, defaults to TRUE.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE.}
}
\value{
Invisibly returns a tibble with columns \code{lon}, \code{lat}, and
\code{delta}: the projected change versus the 1970-2000 baseline for each
cell. For temperature variables \code{delta} is the absolute change in
degrees C; for precipitation variables (var "prec", or bioc 12-19) it is
the percent change, capped at +/-100\\% to limit instability where the
baseline is near zero.
}
\description{
Computes expected climate change as the difference between a CMIP6 projection
retrieved from WorldClim and its 1970-2000 reference-period baseline, on a
common one-degree grid.
}
\details{
\code{get_cmip6_anom} downloads the projection and the corresponding
WorldClim v2.1 baseline, aggregates both to a one-degree grid, and subtracts
cell by cell. Differencing removes most of the model's systematic bias,
leaving the expected change. Polar regions can show spurious values where
the downscaled grids differ in land-sea masking.
}
\examples{
\donttest{
anom <- get_cmip6_anom()
plot_cmip6_anom(anom)
}

}
\references{
See \code{\link{get_cmip6}}.
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrumental_temp.R
\name{plot_temp_scatter}
\alias{plot_temp_scatter}
\title{Download and plot essential climate data}
\usage{
plot_temp_scatter(
  dataset = get_temp(),
  print = TRUE,
  labelmax = FALSE,
  labellatest = TRUE
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_temp}}

\item{print}{(boolean) Display temperature anomaly ggplot2 chart, defaults to TRUE. Use FALSE to not display chart.}

\item{labelmax}{(boolean) Display date of max value, default = FALSE}

\item{labellatest}{(boolean) Display date of latest value, default = TRUE}
}
\value{
Invisibly returns a ggplot2 object with temperature anomaly chart
}
\description{
Plots scatter of monthly temperature anomaly retrieved using \code{get_temp()} with ggplot2. The output ggplot2 object may be further modified.
}
\details{
\code{plot_temp_scatter} invisibly returns a ggplot2 object with a pre-defined temperature monthly mean anomaly chart using data from \code{get_temp}.
By default the chart is also displayed. Smooths using ggplot2's built-in loess smoother. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Fetch temperature anomaly:
anomaly <- get_temp()
#
# Plot output using package's built-in ggplot2 defaults
plot_temp_scatter(anomaly)

# Or just call plot_temp_scatter(), which defaults to get_temp() dataset
plot_temp_scatter()

p <- plot_temp_scatter(anomaly, print = FALSE)
# Modify plot such as: p + ggplot2::labs(title='The Signature of Climate Change') }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fires.R
\name{get_fires_area}
\alias{get_fires_area}
\title{Download and plot essential climate data}
\usage{
get_fires_area(
  place = "WORLD",
  year = as.numeric(format(Sys.Date(), "\%Y")),
  use_cache = TRUE,
  write_cache = getOption("hs_write_cache")
)
}
\arguments{
\item{place}{(string) Region or country for which to retrieve data. May be one of the
following regional aggregates: \code{"WORLD"} (default), \code{"UN_EUR"} (Europe),
\code{"UN_AFR"} (Africa), \code{"UN_ASI"} (Asia), \code{"UN_AME"} (Americas),
or \code{"UN_OCE"} (Oceania). Alternatively, supply an ISO 3166-1 alpha-3 country
code such as \code{"USA"}, \code{"CAN"}, or \code{"BRA"}.}

\item{year}{(numeric) Four-digit year for which to retrieve data. Must be 2012 or later.
Defaults to the current year.}

\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with weekly and cumulative wildfire burnt area and fire
count data for the requested place and year. Weekly columns include \code{date},
\code{events}, \code{events_min}, \code{events_max}, \code{events_avg},
\code{area_ha}, \code{area_ha_min}, \code{area_ha_avg}, and \code{area_ha_max}.
Cumulative columns are prefixed with \code{cum_}.
}
\description{
Retrieves global wildfire burnt area and fire count data from the
Global Wildfire Information System (GWIS), operated by the Copernicus
Emergency Management Service.
\url{https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend}
}
\details{
\code{get_fires_area} invisibly returns a tibble with weekly and year-to-date cumulative
wildfire burnt area (in hectares) and fire event counts sourced from the GWIS API.
Historical minimum, maximum, and average baselines are included for comparison.
Data coverage begins in 2012. Cache filenames include the place and year so that
multiple queries can be cached independently.
}
\examples{
\donttest{
# Fetch global data for the current year from cache if available:
fires <- get_fires_area()
#
# Fetch data for Canada in 2023:
fires_can <- get_fires_area(place = 'CAN', year = 2023)
#
# Force cache refresh:
fires <- get_fires_area(use_cache = FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details() }

}
\references{
Global Wildfire Information System (GWIS), Copernicus Emergency Management Service / European Commission Joint Research Centre.
\url{https://gwis.jrc.ec.europa.eu/apps/gwis.statistics/seasonaltrend}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extra_viz.R
\name{emissions_map}
\alias{emissions_map}
\title{Download and plot essential climate data}
\usage{
emissions_map(
  dataset = get_emissions(),
  print = TRUE,
  since = 1900,
  number = "all",
  title = substitute(paste(since, "-", to, " Cumulative " * CO[2] *
    " Emissions by Country"), list(since = since, to =
    as.character(dataset[nrow(dataset), 2])))
)
}
\arguments{
\item{dataset}{Name of the tibble generated by \code{get_emissions}}

\item{print}{(boolean) Display emissions treemap, defaults to TRUE. Use FALSE to not display chart.}

\item{since}{(numeric) Start year for cumulative emissions, defaults to 1900 if omitted}

\item{number}{(numeric) Number of countries to display in treemap, defaults to all if omitted}

\item{title}{(string) Manually specify chart title}
}
\value{
Invisibly returns a ggplot2 object with emissions treemap
}
\description{
Plots a treemap of cumulative co2 emissions by country since 1900.
}
\details{
\code{emissions_map} invisibly returns a ggplot2 object with cumulative emissions treemap by country since 1900 from \code{get_emissions}.
By default the chart is also displayed. Users may further modify the output ggplot2 chart.
}
\examples{
\donttest{
# Draw treemap

co2map <- emissions_map()

co2map <- emissions_map(since=2000, number=20, title="Top 20 Cumulative Emitters Since 2000") }

}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icecurves.R
\name{get_icecurves}
\alias{get_icecurves}
\title{Download and plot essential climate data}
\usage{
get_icecurves(
  pole = "N",
  measure = "extent",
  use_cache = TRUE,
  write_cache = getOption("hs_write_cache")
)
}
\arguments{
\item{pole}{'N' for Arctic or 'S' for Antarctic}

\item{measure}{Must be 'extent' or 'area', defaults to 'extent'. Please see terminology link in references for details.}

\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data, or to change pole or month in cache.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with the series of monthly Sea Ice Index since 1979 (in million square km).

\code{get_icecurves} invisibly returns a tibble with time series of monthly Sea Ice Index since 1979 (in million square km).

User may select monthly Arctic or Antarctic sea ice extent or area (in millions of square kilometers).
Defaults to Arctic sea ice extent.  \url{https://nsidc.org/sea-ice-today/about-data#area_extent}
}
\description{
Retrieves Arctic or Antarctic monthly Sea Ice Index time series (in million square km).
Source is the National Snow and Ice Data Center, defaults to Arctic (Northern Hemisphere) monthly sea ice extent since 1979.
\url{https://nsidc.org/data/explore-data} \cr \cr
}
\examples{
\donttest{
# Fetch monthly sea ice history from cache if available:
icecurves <- get_icecurves()
#
# Force cache refresh:
icecurves <- get_icecurves(use_cache = FALSE)
# change region
south_icecurves <- get_icecurves(pole='S',  use_cache = FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_icecurves(icecurves) }

}
\references{
\itemize{
\item NSIDC Data Archive: \url{https://nsidc.org/data/explore-data}
\item All About Sea Ice: \url{https://nsidc.org/learn/parts-cryosphere/sea-ice}
\item Sea Ice terminology: \url{https://nsidc.org/learn}
}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methane.R
\name{get_methane}
\alias{get_methane}
\title{Download and plot essential climate data}
\usage{
get_methane(use_cache = TRUE, write_cache = getOption("hs_write_cache"))
}
\arguments{
\item{use_cache}{(boolean) Return cached data if available, defaults to TRUE. Use FALSE to fetch updated data.}

\item{write_cache}{(boolean) Write data to cache, defaults to FALSE. Use TRUE to write data to cache for later use. Can also be set using options(hs_write_cache=TRUE)}
}
\value{
Invisibly returns a tibble with the monthly methane series
}
\description{
Retrieves globally averaged marine surface methane monthly mean data from National Oceanic and Atmospheric Administration.
The Global Monitoring Division of NOAA’s Earth System Research Laboratory has measured methane since 1983 at a globally distributed network of air sampling sites.
A global average is constructed by first smoothing the data for each site as a function of time, and then smoothed values for each site are plotted as a function of latitude.
Global means are calculated from the latitude plot at each time step.
\url{https://gml.noaa.gov/ccgg/trends_ch4/}
\url{https://gml.noaa.gov/ccgg/about/global_means.html}
}
\details{
\code{get_methane} invisibly returns a tibble with NOAA's monthly globally averaged methane measurement.
The returned object includes year, month, date, average, average uncertainty, trend, and trend uncertainty columns.
Trend is NOAA's published trend.  Please refer to above website for details. CH4 expressed as a mole fraction in dry air, nanomol/mol, abbreviated as ppb.
}
\examples{
\donttest{
# Fetch from cache if available, otherwise download:
ch4 <- get_methane()
#
# Force fetch from source:
ch4 <- get_methane(use_cache=FALSE)
#
# Review cache contents and last update dates:
hockeystick_cache_details()
#
# Plot output using package's built-in ggplot2 settings
plot_methane(ch4) }

}
\references{
Lan, X., K.W. Thoning, and E.J. Dlugokencky: Trends in globally-averaged CH4, N2O, and SF6 determined from NOAA Global Monitoring Laboratory measurements. Version 2022-11, \doi{10.15138/P8XG-AA10}
}
\author{
Hernando Cortina, \email{hch@alum.mit.edu}
}
